@@ -16,7 +16,7 @@
 
 package com.google.common.cache;
 
-import static com.google.common.base.Objects.firstNonNull;
+import static com.google.common.base.MoreObjects.firstNonNull;
 import static com.google.common.base.Preconditions.checkNotNull;
 import static com.google.common.base.Preconditions.checkState;
 
@@ -26,7 +26,6 @@ import com.google.common.cache.AbstractCache.StatsCounter;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.util.concurrent.ExecutionError;
 import com.google.common.util.concurrent.UncheckedExecutionException;
-
 import java.util.AbstractCollection;
 import java.util.AbstractSet;
 import java.util.Collection;
@@ -40,8 +39,7 @@ import java.util.Set;
 import java.util.concurrent.Callable;
 import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.ExecutionException;
-
-import javax.annotation.Nullable;
+import org.checkerframework.checker.nullness.qual.Nullable;
 
 /**
  * LocalCache emulation for GWT.
@@ -53,7 +51,7 @@ import javax.annotation.Nullable;
  */
 public class LocalCache<K, V> implements ConcurrentMap<K, V> {
   private static final int UNSET_INT = CacheBuilder.UNSET_INT;
-  
+
   private final LinkedHashMap<K, Timestamped<V>> cachingHashMap;
   private final CacheLoader<? super K, V> loader;
   private final RemovalListener removalListener;
@@ -63,25 +61,26 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
   private final long expireAfterAccess;
 
   LocalCache(CacheBuilder<? super K, ? super V> builder, CacheLoader<? super K, V> loader) {
-    this.loader = loader;   
+    this.loader = loader;
     this.removalListener = builder.removalListener;
     this.expireAfterAccess = builder.expireAfterAccessNanos;
     this.expireAfterWrite = builder.expireAfterWriteNanos;
     this.statsCounter = builder.getStatsCounterSupplier().get();
-    
+
     /* Implements size-capped LinkedHashMap */
     final long maximumSize = builder.maximumSize;
-    this.cachingHashMap = new CapacityEnforcingLinkedHashMap<K, V>(
-        builder.getInitialCapacity(),
-        0.75f,
-        (builder.maximumSize != UNSET_INT),
-        builder.maximumSize,
-        statsCounter,
-        removalListener);
+    this.cachingHashMap =
+        new CapacityEnforcingLinkedHashMap<K, V>(
+            builder.getInitialCapacity(),
+            0.75f,
+            (builder.maximumSize != UNSET_INT),
+            builder.maximumSize,
+            statsCounter,
+            removalListener);
 
     this.ticker = firstNonNull(builder.ticker, Ticker.systemTicker());
   }
-  
+
   @Override
   public int size() {
     return cachingHashMap.size();
@@ -91,10 +90,10 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
   public boolean isEmpty() {
     return cachingHashMap.isEmpty();
   }
-  
+
   @Override
   public V get(Object key) {
-    key = checkNotNull(key);
+    checkNotNull(key);
     Timestamped<V> value = cachingHashMap.get(key);
 
     if (value == null) {
@@ -115,8 +114,8 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
 
   @Override
   public V put(K key, V value) {
-    key = checkNotNull(key);
-    value = checkNotNull(value);
+    checkNotNull(key);
+    checkNotNull(value);
     Timestamped<V> oldValue = cachingHashMap.put(key, new Timestamped<V>(value, ticker));
     if (oldValue == null) {
       return null;
@@ -130,12 +129,12 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
     Timestamped<V> stamped = cachingHashMap.remove(key);
     if (stamped != null) {
       V value = stamped.getValue();
-      
+
       if (!isExpired(stamped)) {
         alertListenerIfPresent(key, value, RemovalCause.EXPLICIT);
         return value;
       }
-  
+
       alertListenerIfPresent(key, value, RemovalCause.EXPIRED);
     }
     return null;
@@ -143,7 +142,7 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
 
   @Override
   public void putAll(Map<? extends K, ? extends V> m) {
-    for (Map.Entry<? extends K, ? extends V> entry : m.entrySet()) {
+    for (Entry<? extends K, ? extends V> entry : m.entrySet()) {
       put(entry.getKey(), entry.getValue());
     }
   }
@@ -151,7 +150,7 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
   @Override
   public void clear() {
     if (removalListener != null) {
-      for (Map.Entry<K, Timestamped<V>> entry : cachingHashMap.entrySet()) {
+      for (Entry<K, Timestamped<V>> entry : cachingHashMap.entrySet()) {
         alertListenerIfPresent(entry.getKey(), entry.getValue().getValue(), RemovalCause.EXPLICIT);
       }
     }
@@ -196,12 +195,12 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
     }
     return null;
   }
-  
+
   @Override
   public boolean containsKey(Object key) {
     return cachingHashMap.containsKey(key) && !isExpired(cachingHashMap.get(key));
   }
-  
+
   @Override
   public boolean containsValue(Object value) {
     for (Timestamped<V> val : cachingHashMap.values()) {
@@ -221,7 +220,7 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
 
     boolean expireWrite = (stamped.getWriteTimestamp() + expireAfterWrite <= currentTimeNanos());
     boolean expireAccess = (stamped.getAccessTimestamp() + expireAfterAccess <= currentTimeNanos());
-    
+
     if (expireAfterAccess == UNSET_INT) {
       return expireWrite;
     }
@@ -232,16 +231,18 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
     return expireWrite || expireAccess;
   }
 
+  @SuppressWarnings("GoodTime")
   private long currentTimeNanos() {
     return ticker.read();
   }
-  
+
   private void alertListenerIfPresent(Object key, Object value, RemovalCause cause) {
     if (removalListener != null) {
       removalListener.onRemoval(RemovalNotification.create(key, value, cause));
     }
   }
 
+  @SuppressWarnings("GoodTime") // timestamps as numeric primitives
   private V load(Object key) throws ExecutionException {
     long startTime = ticker.read();
     V calculatedValue;
@@ -276,9 +277,9 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
     statsCounter.recordLoadSuccess(ticker.read() - startTime);
     return calculatedValue;
   }
-  
+
   private V getIfPresent(Object key) {
-    key = checkNotNull(key);
+    checkNotNull(key);
     Timestamped<V> value = cachingHashMap.get(key);
 
     if (value == null) {
@@ -290,50 +291,51 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
       alertListenerIfPresent(key, value.getValue(), RemovalCause.EXPIRED);
       cachingHashMap.remove(key);
       return null;
-    }    
+    }
   }
 
-  private V getOrLoad(K key) throws ExecutionException{
+  private V getOrLoad(K key) throws ExecutionException {
     V value = get(key);
     if (value != null) {
       return value;
     }
     return load(key);
   }
-  
+
+  @SuppressWarnings("GoodTime") // timestamps as numeric primitives
   private static class Timestamped<V> {
     private final V value;
     private final Ticker ticker;
     private long writeTimestamp;
     private long accessTimestamp;
-    
+
     public Timestamped(V value, Ticker ticker) {
       this.value = checkNotNull(value);
       this.ticker = checkNotNull(ticker);
       this.writeTimestamp = ticker.read();
       this.accessTimestamp = this.writeTimestamp;
     }
-    
+
     public V getValue() {
       return value;
     }
-    
+
     public void updateTimestamp() {
       accessTimestamp = ticker.read();
     }
-  
+
     public long getAccessTimestamp() {
       return accessTimestamp;
     }
-    
+
     public long getWriteTimestamp() {
       return writeTimestamp;
     }
-    
+
     public boolean equals(Object o) {
       return value.equals(o);
     }
-    
+
     public int hashCode() {
       return value.hashCode();
     }
@@ -352,8 +354,8 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
       this(builder, null);
     }
 
-    protected LocalManualCache(CacheBuilder<? super K, ? super V> builder,
-        CacheLoader<? super K, V> loader) {
+    protected LocalManualCache(
+        CacheBuilder<? super K, ? super V> builder, CacheLoader<? super K, V> loader) {
       this.localCache = new LocalCache<K, V>(builder, loader);
     }
 
@@ -365,7 +367,7 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
       if (value != null) {
         return value;
       }
-      
+
       try {
         V newValue = valueLoader.call();
         localCache.put(key, newValue);
@@ -374,10 +376,9 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
         throw new ExecutionException(e);
       }
     }
-    
+
     @Override
-    @Nullable
-    public V getIfPresent(Object key) {
+    public @Nullable V getIfPresent(Object key) {
       return localCache.getIfPresent(key);
     }
 
@@ -388,7 +389,7 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
 
     @Override
     public void invalidate(Object key) {
-      key = checkNotNull(key);
+      checkNotNull(key);
       localCache.remove(key);
     }
 
@@ -414,11 +415,11 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
    * @param <K> the base key type
    * @param <V> the base value type
    */
-  public static class LocalLoadingCache<K, V>
-      extends LocalManualCache<K, V> implements LoadingCache<K, V> {
+  public static class LocalLoadingCache<K, V> extends LocalManualCache<K, V>
+      implements LoadingCache<K, V> {
 
-    LocalLoadingCache(CacheBuilder<? super K, ? super V> builder,
-        CacheLoader<? super K, V> loader) {
+    LocalLoadingCache(
+        CacheBuilder<? super K, ? super V> builder, CacheLoader<? super K, V> loader) {
       super(builder, checkNotNull(loader));
     }
 
@@ -451,26 +452,26 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
       }
       return ImmutableMap.copyOf(map);
     }
-    
+
     @Override
     public void refresh(K key) {
       throw new UnsupportedOperationException();
     }
   }
-  
+
   /**
-   * LinkedHashMap that enforces it's maximum size and logs events in a StatsCounter object
-   * and an optional RemovalListener.
-   * 
+   * LinkedHashMap that enforces it's maximum size and logs events in a StatsCounter object and an
+   * optional RemovalListener.
+   *
    * @param <K> the base key type
    * @param <V> the base value type
    */
   private class CapacityEnforcingLinkedHashMap<K, V> extends LinkedHashMap<K, Timestamped<V>> {
-    
+
     private final StatsCounter statsCounter;
     private final RemovalListener removalListener;
     private final long maximumSize;
-    
+
     public CapacityEnforcingLinkedHashMap(
         int initialCapacity,
         float loadFactor,
@@ -485,19 +486,18 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
     }
 
     @Override
-    protected boolean removeEldestEntry(Map.Entry<K, Timestamped<V>> ignored) {
+    protected boolean removeEldestEntry(Entry<K, Timestamped<V>> ignored) {
       boolean removal = (maximumSize == UNSET_INT) ? false : (size() > maximumSize);
       if ((removalListener != null) && removal) {
-        removalListener.onRemoval(RemovalNotification.create(
-            ignored.getKey(), 
-            ignored.getValue().getValue(),
-            RemovalCause.SIZE));
+        removalListener.onRemoval(
+            RemovalNotification.create(
+                ignored.getKey(), ignored.getValue().getValue(), RemovalCause.SIZE));
       }
       statsCounter.recordEviction();
       return removal;
     }
   }
-  
+
   /**
    * Any updates to LocalCache.Strength used in CacheBuilder need to be matched in this class for
    * compilation purposes.
@@ -528,14 +528,14 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
         return Equivalence.identity();
       }
     };
-    
+
     abstract Equivalence<Object> defaultEquivalence();
   }
-  
+
   /**
-   * <p>Implementation for the EntryIterator, which is used to build Key and Value iterators.
-   * 
-   * <p>Expiration is only checked on hasNext(), so as to ensure that a next() call never returns 
+   * Implementation for the EntryIterator, which is used to build Key and Value iterators.
+   *
+   * <p>Expiration is only checked on hasNext(), so as to ensure that a next() call never returns
    * null when hasNext() has already been called.
    */
   class EntryIterator implements Iterator<Entry<K, V>> {
@@ -550,13 +550,13 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
     @Override
     public Entry<K, V> next() {
       if (nextEntry == null) {
-        hasNext();
+        boolean unused = hasNext();
 
         if (nextEntry == null) {
           throw new NoSuchElementException();
         }
       }
-      
+
       lastEntry = nextEntry;
       nextEntry = null;
       return new WriteThroughEntry(lastEntry.getKey(), lastEntry.getValue().getValue());
@@ -585,16 +585,14 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
     }
   }
 
-  /**
-   * KeyIterator build on top of EntryIterator.
-   */
+  /** KeyIterator build on top of EntryIterator. */
   final class KeyIterator implements Iterator<K> {
     private EntryIterator iterator;
-    
+
     KeyIterator() {
       iterator = new EntryIterator();
     }
-    
+
     @Override
     public boolean hasNext() {
       return iterator.hasNext();
@@ -611,16 +609,14 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
     }
   }
 
-  /**
-   * ValueIterator build on top of EntryIterator.
-   */
+  /** ValueIterator build on top of EntryIterator. */
   final class ValueIterator implements Iterator<V> {
     private EntryIterator iterator;
-    
+
     ValueIterator() {
       iterator = new EntryIterator();
     }
-    
+
     @Override
     public boolean hasNext() {
       return iterator.hasNext();
@@ -663,10 +659,10 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
     Set<Entry<K, V>> es = entrySet;
     return (es != null) ? es : (entrySet = new EntrySet(this));
   }
-  
+
   /**
-   * Custom Entry class used by EntryIterator.next(), that relays setValue changes to the
-   * underlying map.
+   * Custom Entry class used by EntryIterator.next(), that relays setValue changes to the underlying
+   * map.
    */
   private final class WriteThroughEntry implements Entry<K, V> {
     final K key;
@@ -708,35 +704,32 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
       throw new UnsupportedOperationException();
     }
 
-    /**
-     * Returns a string representation of the form <code>{key}={value}</code>.
-     */
-    @Override 
+    @Override
     public String toString() {
       return getKey() + "=" + getValue();
     }
   }
-  
+
   // TODO(fry): Separate logic for consistency between emul and nonemul implementation.
   // TODO(fry): Look into Maps.KeySet and Maps.Values, which can ideally be reused here but are
   // currently only package visible.
   abstract class AbstractCacheSet<T> extends AbstractSet<T> {
     final ConcurrentMap<?, ?> map;
-    
+
     AbstractCacheSet(ConcurrentMap<?, ?> map) {
       this.map = map;
     }
-    
+
     @Override
     public int size() {
       return map.size();
     }
-    
+
     @Override
     public boolean isEmpty() {
       return map.isEmpty();
     }
-    
+
     @Override
     public void clear() {
       map.clear();
@@ -764,7 +757,7 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
       return map.remove(o) != null;
     }
   }
-  
+
   private final class Values extends AbstractCollection<V> {
     final ConcurrentMap<?, ?> map;
 
@@ -786,12 +779,12 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
     public int size() {
       return map.size();
     }
-    
+
     @Override
     public boolean isEmpty() {
       return map.isEmpty();
     }
-    
+
     @Override
     public void clear() {
       map.clear();
@@ -803,7 +796,7 @@ public class LocalCache<K, V> implements ConcurrentMap<K, V> {
     EntrySet(ConcurrentMap<?, ?> map) {
       super(map);
     }
-    
+
     @Override
     public Iterator<Entry<K, V>> iterator() {
       return new EntryIterator();
