@@ -19,6 +19,7 @@ package com.google.common.collect.testing.google;
 import static com.google.common.base.Preconditions.checkArgument;
 import static com.google.common.collect.testing.Helpers.mapEntry;
 
+import com.google.common.annotations.GwtIncompatible;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMultimap;
 import com.google.common.collect.Multimap;
@@ -41,9 +42,6 @@ import com.google.common.collect.testing.features.Feature;
 import com.google.common.collect.testing.features.ListFeature;
 import com.google.common.collect.testing.features.MapFeature;
 import com.google.common.testing.SerializableTester;
-
-import junit.framework.TestSuite;
-
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -56,17 +54,18 @@ import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Set;
+import junit.framework.TestSuite;
 
 /**
- * Creates, based on your criteria, a JUnit test suite that exhaustively tests
- * a {@code Multimap} implementation.
+ * Creates, based on your criteria, a JUnit test suite that exhaustively tests a {@code Multimap}
+ * implementation.
  *
  * @author Louis Wasserman
  */
-public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
-    PerCollectionSizeTestSuiteBuilder<
-    MultimapTestSuiteBuilder<K, V, M>,
-    TestMultimapGenerator<K, V, M>, M, Map.Entry<K, V>> {
+@GwtIncompatible
+public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>>
+    extends PerCollectionSizeTestSuiteBuilder<
+        MultimapTestSuiteBuilder<K, V, M>, TestMultimapGenerator<K, V, M>, M, Entry<K, V>> {
 
   public static <K, V, M extends Multimap<K, V>> MultimapTestSuiteBuilder<K, V, M> using(
       TestMultimapGenerator<K, V, M> generator) {
@@ -76,7 +75,7 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
   // Class parameters must be raw.
   @Override
   protected List<Class<? extends AbstractTester>> getTesters() {
-    return ImmutableList.<Class<? extends AbstractTester>> of(
+    return ImmutableList.<Class<? extends AbstractTester>>of(
         MultimapAsMapGetTester.class,
         MultimapAsMapTester.class,
         MultimapSizeTester.class,
@@ -86,6 +85,7 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
         MultimapContainsEntryTester.class,
         MultimapEntriesTester.class,
         MultimapEqualsTester.class,
+        MultimapForEachTester.class,
         MultimapGetTester.class,
         MultimapKeySetTester.class,
         MultimapKeysTester.class,
@@ -101,10 +101,8 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
 
   @Override
   protected List<TestSuite> createDerivedSuites(
-      FeatureSpecificTestSuiteBuilder<
-      ?,
-      ? extends OneSizeTestContainerGenerator<M, Map.Entry<K, V>>>
-      parentBuilder) {
+      FeatureSpecificTestSuiteBuilder<?, ? extends OneSizeTestContainerGenerator<M, Entry<K, V>>>
+          parentBuilder) {
     // TODO: Once invariant support is added, supply invariants to each of the
     // derived suites, to check that mutations to the derived collections are
     // reflected in the underlying map.
@@ -112,20 +110,25 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
     List<TestSuite> derivedSuites = super.createDerivedSuites(parentBuilder);
 
     if (parentBuilder.getFeatures().contains(CollectionFeature.SERIALIZABLE)) {
-      derivedSuites.add(MultimapTestSuiteBuilder.using(
-          new ReserializedMultimapGenerator<K, V, M>(parentBuilder.getSubjectGenerator()))
-          .withFeatures(computeReserializedMultimapFeatures(parentBuilder.getFeatures()))
-          .named(parentBuilder.getName() + " reserialized")
-          .suppressing(parentBuilder.getSuppressedTests())
-          .createTestSuite());
+      derivedSuites.add(
+          MultimapTestSuiteBuilder.using(
+                  new ReserializedMultimapGenerator<K, V, M>(parentBuilder.getSubjectGenerator()))
+              .withFeatures(computeReserializedMultimapFeatures(parentBuilder.getFeatures()))
+              .named(parentBuilder.getName() + " reserialized")
+              .suppressing(parentBuilder.getSuppressedTests())
+              .withSetUp(parentBuilder.getSetUp())
+              .withTearDown(parentBuilder.getTearDown())
+              .createTestSuite());
     }
 
-    derivedSuites.add(MapTestSuiteBuilder.using(
-        new AsMapGenerator<K, V, M>(parentBuilder.getSubjectGenerator()))
-        .withFeatures(computeAsMapFeatures(parentBuilder.getFeatures()))
-        .named(parentBuilder.getName() + ".asMap")
-        .suppressing(parentBuilder.getSuppressedTests())
-        .createTestSuite());
+    derivedSuites.add(
+        MapTestSuiteBuilder.using(new AsMapGenerator<K, V, M>(parentBuilder.getSubjectGenerator()))
+            .withFeatures(computeAsMapFeatures(parentBuilder.getFeatures()))
+            .named(parentBuilder.getName() + ".asMap")
+            .suppressing(parentBuilder.getSuppressedTests())
+            .withSetUp(parentBuilder.getSetUp())
+            .withTearDown(parentBuilder.getTearDown())
+            .createTestSuite());
 
     derivedSuites.add(computeEntriesTestSuite(parentBuilder));
     derivedSuites.add(computeMultimapGetTestSuite(parentBuilder));
@@ -137,10 +140,10 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
   }
 
   TestSuite computeValuesTestSuite(
-      FeatureSpecificTestSuiteBuilder<?, ?
-          extends OneSizeTestContainerGenerator<M, Map.Entry<K, V>>> parentBuilder) {
+      FeatureSpecificTestSuiteBuilder<?, ? extends OneSizeTestContainerGenerator<M, Entry<K, V>>>
+          parentBuilder) {
     return CollectionTestSuiteBuilder.using(
-        new ValuesGenerator<K, V, M>(parentBuilder.getSubjectGenerator()))
+            new ValuesGenerator<K, V, M>(parentBuilder.getSubjectGenerator()))
         .withFeatures(computeValuesFeatures(parentBuilder.getFeatures()))
         .named(parentBuilder.getName() + ".values")
         .suppressing(parentBuilder.getSuppressedTests())
@@ -148,51 +151,59 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
   }
 
   TestSuite computeEntriesTestSuite(
-      FeatureSpecificTestSuiteBuilder<?, ?
-          extends OneSizeTestContainerGenerator<M, Map.Entry<K, V>>> parentBuilder) {
+      FeatureSpecificTestSuiteBuilder<?, ? extends OneSizeTestContainerGenerator<M, Entry<K, V>>>
+          parentBuilder) {
     return CollectionTestSuiteBuilder.using(
-        new EntriesGenerator<K, V, M>(parentBuilder.getSubjectGenerator()))
+            new EntriesGenerator<K, V, M>(parentBuilder.getSubjectGenerator()))
         .withFeatures(computeEntriesFeatures(parentBuilder.getFeatures()))
         .named(parentBuilder.getName() + ".entries")
         .suppressing(parentBuilder.getSuppressedTests())
+        .withSetUp(parentBuilder.getSetUp())
+        .withTearDown(parentBuilder.getTearDown())
         .createTestSuite();
   }
 
   TestSuite computeMultimapGetTestSuite(
-      FeatureSpecificTestSuiteBuilder<?, ? extends
-          OneSizeTestContainerGenerator<M, Map.Entry<K, V>>> parentBuilder) {
+      FeatureSpecificTestSuiteBuilder<?, ? extends OneSizeTestContainerGenerator<M, Entry<K, V>>>
+          parentBuilder) {
     return CollectionTestSuiteBuilder.using(
-        new MultimapGetGenerator<K, V, M>(parentBuilder.getSubjectGenerator()))
+            new MultimapGetGenerator<K, V, M>(parentBuilder.getSubjectGenerator()))
         .withFeatures(computeMultimapGetFeatures(parentBuilder.getFeatures()))
         .named(parentBuilder.getName() + ".get[key]")
         .suppressing(parentBuilder.getSuppressedTests())
+        .withSetUp(parentBuilder.getSetUp())
+        .withTearDown(parentBuilder.getTearDown())
         .createTestSuite();
   }
 
   TestSuite computeMultimapAsMapGetTestSuite(
-      FeatureSpecificTestSuiteBuilder<?, ? extends
-          OneSizeTestContainerGenerator<M, Map.Entry<K, V>>> parentBuilder) {
+      FeatureSpecificTestSuiteBuilder<?, ? extends OneSizeTestContainerGenerator<M, Entry<K, V>>>
+          parentBuilder) {
     Set<Feature<?>> features = computeMultimapAsMapGetFeatures(parentBuilder.getFeatures());
     if (Collections.disjoint(features, EnumSet.allOf(CollectionSize.class))) {
       return new TestSuite();
     } else {
       return CollectionTestSuiteBuilder.using(
-          new MultimapAsMapGetGenerator<K, V, M>(parentBuilder.getSubjectGenerator()))
+              new MultimapAsMapGetGenerator<K, V, M>(parentBuilder.getSubjectGenerator()))
           .withFeatures(features)
           .named(parentBuilder.getName() + ".asMap[].get[key]")
           .suppressing(parentBuilder.getSuppressedTests())
+          .withSetUp(parentBuilder.getSetUp())
+          .withTearDown(parentBuilder.getTearDown())
           .createTestSuite();
     }
   }
 
   TestSuite computeKeysTestSuite(
-      FeatureSpecificTestSuiteBuilder<?, ? extends
-          OneSizeTestContainerGenerator<M, Map.Entry<K, V>>> parentBuilder) {
+      FeatureSpecificTestSuiteBuilder<?, ? extends OneSizeTestContainerGenerator<M, Entry<K, V>>>
+          parentBuilder) {
     return MultisetTestSuiteBuilder.using(
-        new KeysGenerator<K, V, M>(parentBuilder.getSubjectGenerator()))
+            new KeysGenerator<K, V, M>(parentBuilder.getSubjectGenerator()))
         .withFeatures(computeKeysFeatures(parentBuilder.getFeatures()))
         .named(parentBuilder.getName() + ".keys")
         .suppressing(parentBuilder.getSuppressedTests())
+        .withSetUp(parentBuilder.getSetUp())
+        .withTearDown(parentBuilder.getTearDown())
         .createTestSuite();
   }
 
@@ -207,8 +218,7 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
     return derivedFeatures;
   }
 
-  static Set<Feature<?>> computeEntriesFeatures(
-      Set<Feature<?>> multimapFeatures) {
+  static Set<Feature<?>> computeEntriesFeatures(Set<Feature<?>> multimapFeatures) {
     Set<Feature<?>> result = computeDerivedCollectionFeatures(multimapFeatures);
     if (multimapFeatures.contains(MapFeature.ALLOWS_NULL_ENTRY_QUERIES)) {
       result.add(CollectionFeature.ALLOWS_NULL_QUERIES);
@@ -216,8 +226,7 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
     return result;
   }
 
-  static Set<Feature<?>> computeValuesFeatures(
-      Set<Feature<?>> multimapFeatures) {
+  static Set<Feature<?>> computeValuesFeatures(Set<Feature<?>> multimapFeatures) {
     Set<Feature<?>> result = computeDerivedCollectionFeatures(multimapFeatures);
     if (multimapFeatures.contains(MapFeature.ALLOWS_NULL_VALUES)) {
       result.add(CollectionFeature.ALLOWS_NULL_VALUES);
@@ -228,8 +237,7 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
     return result;
   }
 
-  static Set<Feature<?>> computeKeysFeatures(
-      Set<Feature<?>> multimapFeatures) {
+  static Set<Feature<?>> computeKeysFeatures(Set<Feature<?>> multimapFeatures) {
     Set<Feature<?>> result = computeDerivedCollectionFeatures(multimapFeatures);
     if (multimapFeatures.contains(MapFeature.ALLOWS_NULL_KEYS)) {
       result.add(CollectionFeature.ALLOWS_NULL_VALUES);
@@ -248,8 +256,7 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
     return derivedFeatures;
   }
 
-  private static Set<Feature<?>> computeAsMapFeatures(
-      Set<Feature<?>> multimapFeatures) {
+  private static Set<Feature<?>> computeAsMapFeatures(Set<Feature<?>> multimapFeatures) {
     Set<Feature<?>> derivedFeatures = Helpers.copyToSet(multimapFeatures);
     derivedFeatures.remove(MapFeature.GENERAL_PURPOSE);
     derivedFeatures.remove(MapFeature.SUPPORTS_PUT);
@@ -262,25 +269,23 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
     return derivedFeatures;
   }
 
-  private static final Multimap<Feature<?>, Feature<?>> GET_FEATURE_MAP = ImmutableMultimap
-      .<Feature<?>, Feature<?>> builder()
-      .put(
-          MapFeature.FAILS_FAST_ON_CONCURRENT_MODIFICATION,
-          CollectionFeature.FAILS_FAST_ON_CONCURRENT_MODIFICATION)
-      .put(MapFeature.GENERAL_PURPOSE, ListFeature.SUPPORTS_ADD_WITH_INDEX)
-      .put(MapFeature.GENERAL_PURPOSE, ListFeature.SUPPORTS_REMOVE_WITH_INDEX)
-      .put(MapFeature.GENERAL_PURPOSE, ListFeature.SUPPORTS_SET)
-      .put(MapFeature.ALLOWS_NULL_VALUE_QUERIES,
-          CollectionFeature.ALLOWS_NULL_QUERIES)
-      .put(MapFeature.ALLOWS_NULL_VALUES, CollectionFeature.ALLOWS_NULL_VALUES)
-      .put(MapFeature.SUPPORTS_REMOVE, CollectionFeature.SUPPORTS_REMOVE)
-      .put(MapFeature.SUPPORTS_PUT, CollectionFeature.SUPPORTS_ADD)
-      .build();
+  private static final ImmutableMultimap<Feature<?>, Feature<?>> GET_FEATURE_MAP =
+      ImmutableMultimap.<Feature<?>, Feature<?>>builder()
+          .put(
+              MapFeature.FAILS_FAST_ON_CONCURRENT_MODIFICATION,
+              CollectionFeature.FAILS_FAST_ON_CONCURRENT_MODIFICATION)
+          .put(MapFeature.GENERAL_PURPOSE, ListFeature.SUPPORTS_ADD_WITH_INDEX)
+          .put(MapFeature.GENERAL_PURPOSE, ListFeature.SUPPORTS_REMOVE_WITH_INDEX)
+          .put(MapFeature.GENERAL_PURPOSE, ListFeature.SUPPORTS_SET)
+          .put(MapFeature.ALLOWS_NULL_VALUE_QUERIES, CollectionFeature.ALLOWS_NULL_QUERIES)
+          .put(MapFeature.ALLOWS_NULL_VALUES, CollectionFeature.ALLOWS_NULL_VALUES)
+          .put(MapFeature.SUPPORTS_REMOVE, CollectionFeature.SUPPORTS_REMOVE)
+          .put(MapFeature.SUPPORTS_PUT, CollectionFeature.SUPPORTS_ADD)
+          .build();
 
-  Set<Feature<?>> computeMultimapGetFeatures(
-      Set<Feature<?>> multimapFeatures) {
+  Set<Feature<?>> computeMultimapGetFeatures(Set<Feature<?>> multimapFeatures) {
     Set<Feature<?>> derivedFeatures = Helpers.copyToSet(multimapFeatures);
-    for (Map.Entry<Feature<?>, Feature<?>> entry : GET_FEATURE_MAP.entries()) {
+    for (Entry<Feature<?>, Feature<?>> entry : GET_FEATURE_MAP.entries()) {
       if (derivedFeatures.contains(entry.getKey())) {
         derivedFeatures.add(entry.getValue());
       }
@@ -295,10 +300,9 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
     return derivedFeatures;
   }
 
-  Set<Feature<?>> computeMultimapAsMapGetFeatures(
-      Set<Feature<?>> multimapFeatures) {
-    Set<Feature<?>> derivedFeatures = Helpers.copyToSet(
-        computeMultimapGetFeatures(multimapFeatures));
+  Set<Feature<?>> computeMultimapAsMapGetFeatures(Set<Feature<?>> multimapFeatures) {
+    Set<Feature<?>> derivedFeatures =
+        Helpers.copyToSet(computeMultimapGetFeatures(multimapFeatures));
     if (derivedFeatures.remove(CollectionSize.ANY)) {
       derivedFeatures.addAll(CollectionSize.ANY.getImpliedFeatures());
     }
@@ -306,12 +310,11 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
     return derivedFeatures;
   }
 
-  private static class AsMapGenerator<K, V, M extends Multimap<K, V>> implements
-      TestMapGenerator<K, Collection<V>>, DerivedGenerator {
-    private final OneSizeTestContainerGenerator<M, Map.Entry<K, V>> multimapGenerator;
+  private static class AsMapGenerator<K, V, M extends Multimap<K, V>>
+      implements TestMapGenerator<K, Collection<V>>, DerivedGenerator {
+    private final OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator;
 
-    public AsMapGenerator(
-        OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator) {
+    public AsMapGenerator(OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator) {
       this.multimapGenerator = multimapGenerator;
     }
 
@@ -331,7 +334,7 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
           ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator()).sampleKeys();
       SampleElements<V> sampleValues =
           ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator()).sampleValues();
-      return new SampleElements<Entry<K, Collection<V>>>(
+      return new SampleElements<>(
           mapEntry(sampleKeys.e0(), createCollection(sampleValues.e0())),
           mapEntry(sampleKeys.e1(), createCollection(sampleValues.e1())),
           mapEntry(sampleKeys.e2(), createCollection(sampleValues.e2())),
@@ -341,10 +344,10 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
 
     @Override
     public Map<K, Collection<V>> create(Object... elements) {
-      Set<K> keySet = new HashSet<K>();
-      List<Map.Entry<K, V>> builder = new ArrayList<Entry<K, V>>();
+      Set<K> keySet = new HashSet<>();
+      List<Entry<K, V>> builder = new ArrayList<>();
       for (Object o : elements) {
-        Map.Entry<K, Collection<V>> entry = (Entry<K, Collection<V>>) o;
+        Entry<K, Collection<V>> entry = (Entry<K, Collection<V>>) o;
         keySet.add(entry.getKey());
         for (V v : entry.getValue()) {
           builder.add(mapEntry(entry.getKey(), v));
@@ -362,17 +365,17 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
 
     @Override
     public Iterable<Entry<K, Collection<V>>> order(List<Entry<K, Collection<V>>> insertionOrder) {
-      Map<K, Collection<V>> map = new HashMap<K, Collection<V>>();
-      List<Map.Entry<K, V>> builder = new ArrayList<Entry<K, V>>();
+      Map<K, Collection<V>> map = new HashMap<>();
+      List<Entry<K, V>> builder = new ArrayList<>();
       for (Entry<K, Collection<V>> entry : insertionOrder) {
         for (V v : entry.getValue()) {
           builder.add(mapEntry(entry.getKey(), v));
         }
         map.put(entry.getKey(), entry.getValue());
       }
-      Iterable<Map.Entry<K, V>> ordered = multimapGenerator.order(builder);
-      LinkedHashMap<K, Collection<V>> orderedMap = new LinkedHashMap<K, Collection<V>>();
-      for (Map.Entry<K, V> entry : ordered) {
+      Iterable<Entry<K, V>> ordered = multimapGenerator.order(builder);
+      LinkedHashMap<K, Collection<V>> orderedMap = new LinkedHashMap<>();
+      for (Entry<K, V> entry : ordered) {
         orderedMap.put(entry.getKey(), map.get(entry.getKey()));
       }
       return orderedMap.entrySet();
@@ -393,10 +396,9 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
 
   static class EntriesGenerator<K, V, M extends Multimap<K, V>>
       implements TestCollectionGenerator<Entry<K, V>>, DerivedGenerator {
-    private final OneSizeTestContainerGenerator<M, Map.Entry<K, V>> multimapGenerator;
+    private final OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator;
 
-    public EntriesGenerator(
-        OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator) {
+    public EntriesGenerator(OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator) {
       this.multimapGenerator = multimapGenerator;
     }
 
@@ -429,47 +431,50 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
 
   static class ValuesGenerator<K, V, M extends Multimap<K, V>>
       implements TestCollectionGenerator<V> {
-    private final OneSizeTestContainerGenerator<M, Map.Entry<K, V>> multimapGenerator;
+    private final OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator;
 
-    public ValuesGenerator(
-        OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator) {
+    public ValuesGenerator(OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator) {
       this.multimapGenerator = multimapGenerator;
     }
 
     @Override
     public SampleElements<V> samples() {
-      return
-          ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator()).sampleValues();
+      return ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator())
+          .sampleValues();
     }
 
     @Override
     public Collection<V> create(Object... elements) {
-      K k = ((TestMultimapGenerator<K, V, M>)
-          multimapGenerator.getInnerGenerator()).sampleKeys().e0();
+      K k =
+          ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator())
+              .sampleKeys()
+              .e0();
       Entry<K, V>[] entries = new Entry[elements.length];
       for (int i = 0; i < elements.length; i++) {
         entries[i] = mapEntry(k, (V) elements[i]);
       }
-      return multimapGenerator.create(entries).values();
+      return multimapGenerator.create((Object[]) entries).values();
     }
 
     @SuppressWarnings("unchecked")
     @Override
     public V[] createArray(int length) {
-      return ((TestMultimapGenerator<K, V, M>)
-          multimapGenerator.getInnerGenerator()).createValueArray(length);
+      return ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator())
+          .createValueArray(length);
     }
 
     @Override
     public Iterable<V> order(List<V> insertionOrder) {
-      K k = ((TestMultimapGenerator<K, V, M>)
-          multimapGenerator.getInnerGenerator()).sampleKeys().e0();
-      List<Entry<K, V>> entries = new ArrayList<Entry<K, V>>();
+      K k =
+          ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator())
+              .sampleKeys()
+              .e0();
+      List<Entry<K, V>> entries = new ArrayList<>();
       for (V v : insertionOrder) {
         entries.add(mapEntry(k, v));
       }
       Iterable<Entry<K, V>> ordered = multimapGenerator.order(entries);
-      List<V> orderedValues = new ArrayList<V>();
+      List<V> orderedValues = new ArrayList<>();
       for (Entry<K, V> entry : ordered) {
         orderedValues.add(entry.getValue());
       }
@@ -477,12 +482,11 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
     }
   }
 
-  static class KeysGenerator<K, V, M extends Multimap<K, V>> implements
-      TestMultisetGenerator<K>, DerivedGenerator {
-    private final OneSizeTestContainerGenerator<M, Map.Entry<K, V>> multimapGenerator;
+  static class KeysGenerator<K, V, M extends Multimap<K, V>>
+      implements TestMultisetGenerator<K>, DerivedGenerator {
+    private final OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator;
 
-    public KeysGenerator(
-        OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator) {
+    public KeysGenerator(OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator) {
       this.multimapGenerator = multimapGenerator;
     }
 
@@ -502,8 +506,8 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
        * This is nasty and complicated, but it's the only way to make sure keys get mapped to enough
        * distinct values.
        */
-      Map.Entry[] entries = new Map.Entry[elements.length];
-      Map<K, Iterator<V>> valueIterators = new HashMap<K, Iterator<V>>();
+      Entry[] entries = new Entry[elements.length];
+      Map<K, Iterator<V>> valueIterators = new HashMap<>();
       for (int i = 0; i < elements.length; i++) {
         @SuppressWarnings("unchecked")
         K key = (K) elements[i];
@@ -514,30 +518,31 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
         }
         entries[i] = mapEntry((K) elements[i], valueItr.next());
       }
-      return multimapGenerator.create(entries).keys();
+      return multimapGenerator.create((Object[]) entries).keys();
     }
 
     private Iterator<V> sampleValuesIterator() {
-      return ((TestMultimapGenerator<K, V, M>) multimapGenerator
-          .getInnerGenerator()).sampleValues().iterator();
+      return ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator())
+          .sampleValues()
+          .iterator();
     }
 
     @SuppressWarnings("unchecked")
     @Override
     public K[] createArray(int length) {
-      return ((TestMultimapGenerator<K, V, M>)
-          multimapGenerator.getInnerGenerator()).createKeyArray(length);
+      return ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator())
+          .createKeyArray(length);
     }
 
     @Override
     public Iterable<K> order(List<K> insertionOrder) {
       Iterator<V> valueIter = sampleValuesIterator();
-      List<Entry<K, V>> entries = new ArrayList<Entry<K, V>>();
+      List<Entry<K, V>> entries = new ArrayList<>();
       for (K k : insertionOrder) {
         entries.add(mapEntry(k, valueIter.next()));
       }
       Iterable<Entry<K, V>> ordered = multimapGenerator.order(entries);
-      List<K> orderedValues = new ArrayList<K>();
+      List<K> orderedValues = new ArrayList<>();
       for (Entry<K, V> entry : ordered) {
         orderedValues.add(entry.getKey());
       }
@@ -547,11 +552,9 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
 
   static class MultimapGetGenerator<K, V, M extends Multimap<K, V>>
       implements TestCollectionGenerator<V> {
-    final OneSizeTestContainerGenerator<M, Map.Entry<K, V>> multimapGenerator;
+    final OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator;
 
-    public MultimapGetGenerator(
-        OneSizeTestContainerGenerator<
-        M, Map.Entry<K, V>> multimapGenerator) {
+    public MultimapGetGenerator(OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator) {
       this.multimapGenerator = multimapGenerator;
     }
 
@@ -569,14 +572,16 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
 
     @Override
     public Iterable<V> order(List<V> insertionOrder) {
-      K k = ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator())
-          .sampleKeys().e0();
-      List<Entry<K, V>> entries = new ArrayList<Entry<K, V>>();
+      K k =
+          ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator())
+              .sampleKeys()
+              .e0();
+      List<Entry<K, V>> entries = new ArrayList<>();
       for (V v : insertionOrder) {
         entries.add(mapEntry(k, v));
       }
       Iterable<Entry<K, V>> orderedEntries = multimapGenerator.order(entries);
-      List<V> values = new ArrayList<V>();
+      List<V> values = new ArrayList<>();
       for (Entry<K, V> entry : orderedEntries) {
         values.add(entry.getValue());
       }
@@ -586,12 +591,14 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
     @Override
     public Collection<V> create(Object... elements) {
       Entry<K, V>[] array = multimapGenerator.createArray(elements.length);
-      K k = ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator())
-          .sampleKeys().e0();
+      K k =
+          ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator())
+              .sampleKeys()
+              .e0();
       for (int i = 0; i < elements.length; i++) {
         array[i] = mapEntry(k, (V) elements[i]);
       }
-      return multimapGenerator.create(array).get(k);
+      return multimapGenerator.create((Object[]) array).get(k);
     }
   }
 
@@ -599,53 +606,53 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
       extends MultimapGetGenerator<K, V, M> {
 
     public MultimapAsMapGetGenerator(
-        OneSizeTestContainerGenerator<
-        M, Map.Entry<K, V>> multimapGenerator) {
+        OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator) {
       super(multimapGenerator);
     }
 
     @Override
     public Collection<V> create(Object... elements) {
       Entry<K, V>[] array = multimapGenerator.createArray(elements.length);
-      K k = ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator())
-          .sampleKeys().e0();
+      K k =
+          ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator())
+              .sampleKeys()
+              .e0();
       for (int i = 0; i < elements.length; i++) {
         array[i] = mapEntry(k, (V) elements[i]);
       }
-      return multimapGenerator.create(array).asMap().get(k);
+      return multimapGenerator.create((Object[]) array).asMap().get(k);
     }
   }
 
   private static class ReserializedMultimapGenerator<K, V, M extends Multimap<K, V>>
       implements TestMultimapGenerator<K, V, M> {
-    private final OneSizeTestContainerGenerator<M, Map.Entry<K, V>> multimapGenerator;
+    private final OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator;
 
     public ReserializedMultimapGenerator(
-        OneSizeTestContainerGenerator<
-        M, Map.Entry<K, V>> multimapGenerator) {
+        OneSizeTestContainerGenerator<M, Entry<K, V>> multimapGenerator) {
       this.multimapGenerator = multimapGenerator;
     }
 
     @Override
-    public SampleElements<Map.Entry<K, V>> samples() {
+    public SampleElements<Entry<K, V>> samples() {
       return multimapGenerator.samples();
     }
 
     @Override
-    public Map.Entry<K, V>[] createArray(int length) {
+    public Entry<K, V>[] createArray(int length) {
       return multimapGenerator.createArray(length);
     }
 
     @Override
-    public Iterable<Map.Entry<K, V>> order(
-        List<Map.Entry<K, V>> insertionOrder) {
+    public Iterable<Entry<K, V>> order(List<Entry<K, V>> insertionOrder) {
       return multimapGenerator.order(insertionOrder);
     }
 
     @Override
     public M create(Object... elements) {
-      return SerializableTester.reserialize(((TestMultimapGenerator<K, V, M>) multimapGenerator
-          .getInnerGenerator()).create(elements));
+      return SerializableTester.reserialize(
+          ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator())
+              .create(elements));
     }
 
     @Override
@@ -662,8 +669,7 @@ public class MultimapTestSuiteBuilder<K, V, M extends Multimap<K, V>> extends
 
     @Override
     public SampleElements<K> sampleKeys() {
-      return ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator())
-          .sampleKeys();
+      return ((TestMultimapGenerator<K, V, M>) multimapGenerator.getInnerGenerator()).sampleKeys();
     }
 
     @Override
