@@ -20,9 +20,8 @@ import com.google.common.annotations.GwtCompatible;
 import com.google.common.collect.Multimap;
 import com.google.common.collect.testing.SampleElements;
 import com.google.common.collect.testing.TestContainerGenerator;
-
 import java.util.Collection;
-import java.util.Map;
+import java.util.Map.Entry;
 
 /**
  * Creates multimaps, containing sample elements, to be tested.
@@ -31,7 +30,7 @@ import java.util.Map;
  */
 @GwtCompatible
 public interface TestMultimapGenerator<K, V, M extends Multimap<K, V>>
-    extends TestContainerGenerator<M, Map.Entry<K, V>> {
+    extends TestContainerGenerator<M, Entry<K, V>> {
 
   K[] createKeyArray(int length);
 
