@@ -19,11 +19,12 @@ package com.google.common.testing;
 import static com.google.common.base.Preconditions.checkArgument;
 import static com.google.common.base.Preconditions.checkNotNull;
 
-import com.google.common.annotations.Beta;
+import com.google.common.annotations.GwtIncompatible;
 import com.google.common.base.Converter;
 import com.google.common.base.Objects;
 import com.google.common.collect.ClassToInstanceMap;
 import com.google.common.collect.ImmutableList;
+import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import com.google.common.collect.MutableClassToInstanceMap;
@@ -31,10 +32,8 @@ import com.google.common.reflect.Invokable;
 import com.google.common.reflect.Parameter;
 import com.google.common.reflect.Reflection;
 import com.google.common.reflect.TypeToken;
-
-import junit.framework.Assert;
-import junit.framework.AssertionFailedError;
-
+import java.lang.annotation.Annotation;
+import java.lang.reflect.AnnotatedType;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Member;
@@ -42,41 +41,43 @@ import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import java.lang.reflect.ParameterizedType;
 import java.lang.reflect.Type;
+import java.lang.reflect.TypeVariable;
 import java.util.Arrays;
 import java.util.List;
 import java.util.concurrent.ConcurrentMap;
-
-import javax.annotation.Nullable;
+import java.util.function.Function;
+import junit.framework.Assert;
+import junit.framework.AssertionFailedError;
+import org.checkerframework.checker.nullness.qual.Nullable;
 
 /**
  * A test utility that verifies that your methods and constructors throw {@link
- * NullPointerException} or {@link UnsupportedOperationException} whenever null
- * is passed to a parameter that isn't annotated with {@link Nullable}.
+ * NullPointerException} or {@link UnsupportedOperationException} whenever null is passed to a
+ * parameter whose declaration or type isn't annotated with an annotation with the simple name
+ * {@code Nullable}, {@code CheckForNull}, {@link NullableType}, or {@link NullableDecl}.
  *
- * <p>The tested methods and constructors are invoked -- each time with one
- * parameter being null and the rest not null -- and the test fails if no
- * expected exception is thrown. {@code NullPointerTester} uses best effort to
- * pick non-null default values for many common JDK and Guava types, and also
- * for interfaces and public classes that have public parameter-less
- * constructors. When the non-null default value for a particular parameter type
- * cannot be provided by {@code NullPointerTester}, the caller can provide a
- * custom non-null default value for the parameter type via {@link #setDefault}.
+ * <p>The tested methods and constructors are invoked -- each time with one parameter being null and
+ * the rest not null -- and the test fails if no expected exception is thrown. {@code
+ * NullPointerTester} uses best effort to pick non-null default values for many common JDK and Guava
+ * types, and also for interfaces and public classes that have public parameter-less constructors.
+ * When the non-null default value for a particular parameter type cannot be provided by {@code
+ * NullPointerTester}, the caller can provide a custom non-null default value for the parameter type
+ * via {@link #setDefault}.
  *
  * @author Kevin Bourrillion
  * @since 10.0
  */
-@Beta
+@GwtIncompatible
 public final class NullPointerTester {
 
-  private final ClassToInstanceMap<Object> defaults =
-      MutableClassToInstanceMap.create();
+  private final ClassToInstanceMap<Object> defaults = MutableClassToInstanceMap.create();
   private final List<Member> ignoredMembers = Lists.newArrayList();
 
   private ExceptionTypePolicy policy = ExceptionTypePolicy.NPE_OR_UOE;
 
   /**
-   * Sets a default value that can be used for any parameter of type
-   * {@code type}. Returns this object.
+   * Sets a default value that can be used for any parameter of type {@code type}. Returns this
+   * object.
    */
   public <T> NullPointerTester setDefault(Class<T> type, T value) {
     defaults.putInstance(type, checkNotNull(value));
@@ -94,8 +95,18 @@ public final class NullPointerTester {
   }
 
   /**
-   * Runs {@link #testConstructor} on every constructor in class {@code c} that
-   * has at least {@code minimalVisibility}.
+   * Ignore {@code constructor} in the tests that follow. Returns this object.
+   *
+   * @since 22.0
+   */
+  public NullPointerTester ignore(Constructor<?> constructor) {
+    ignoredMembers.add(checkNotNull(constructor));
+    return this;
+  }
+
+  /**
+   * Runs {@link #testConstructor} on every constructor in class {@code c} that has at least {@code
+   * minimalVisibility}.
    */
   public void testConstructors(Class<?> c, Visibility minimalVisibility) {
     for (Constructor<?> constructor : c.getDeclaredConstructors()) {
@@ -105,18 +116,14 @@ public final class NullPointerTester {
     }
   }
 
-  /**
-   * Runs {@link #testConstructor} on every public constructor in class {@code
-   * c}.
-   */
+  /** Runs {@link #testConstructor} on every public constructor in class {@code c}. */
   public void testAllPublicConstructors(Class<?> c) {
     testConstructors(c, Visibility.PUBLIC);
   }
 
   /**
-   * Runs {@link #testMethod} on every static method of class {@code c} that has
-   * at least {@code minimalVisibility}, including those "inherited" from
-   * superclasses of the same package.
+   * Runs {@link #testMethod} on every static method of class {@code c} that has at least {@code
+   * minimalVisibility}, including those "inherited" from superclasses of the same package.
    */
   public void testStaticMethods(Class<?> c, Visibility minimalVisibility) {
     for (Method method : minimalVisibility.getStaticMethods(c)) {
@@ -127,17 +134,17 @@ public final class NullPointerTester {
   }
 
   /**
-   * Runs {@link #testMethod} on every public static method of class {@code c},
-   * including those "inherited" from superclasses of the same package.
+   * Runs {@link #testMethod} on every public static method of class {@code c}, including those
+   * "inherited" from superclasses of the same package.
    */
   public void testAllPublicStaticMethods(Class<?> c) {
     testStaticMethods(c, Visibility.PUBLIC);
   }
 
   /**
-   * Runs {@link #testMethod} on every instance method of the class of
-   * {@code instance} with at least {@code minimalVisibility}, including those
-   * inherited from superclasses of the same package.
+   * Runs {@link #testMethod} on every instance method of the class of {@code instance} with at
+   * least {@code minimalVisibility}, including those inherited from superclasses of the same
+   * package.
    */
   public void testInstanceMethods(Object instance, Visibility minimalVisibility) {
     for (Method method : getInstanceMethodsToTest(instance.getClass(), minimalVisibility)) {
@@ -156,21 +163,18 @@ public final class NullPointerTester {
   }
 
   /**
-   * Runs {@link #testMethod} on every public instance method of the class of
-   * {@code instance}, including those inherited from superclasses of the same
-   * package.
+   * Runs {@link #testMethod} on every public instance method of the class of {@code instance},
+   * including those inherited from superclasses of the same package.
    */
   public void testAllPublicInstanceMethods(Object instance) {
     testInstanceMethods(instance, Visibility.PUBLIC);
   }
 
   /**
-   * Verifies that {@code method} produces a {@link NullPointerException}
-   * or {@link UnsupportedOperationException} whenever <i>any</i> of its
-   * non-{@link Nullable} parameters are null.
+   * Verifies that {@code method} produces a {@link NullPointerException} or {@link
+   * UnsupportedOperationException} whenever <i>any</i> of its non-nullable parameters are null.
    *
-   * @param instance the instance to invoke {@code method} on, or null if
-   *     {@code method} is static
+   * @param instance the instance to invoke {@code method} on, or null if {@code method} is static
    */
   public void testMethod(@Nullable Object instance, Method method) {
     Class<?>[] types = method.getParameterTypes();
@@ -180,15 +184,16 @@ public final class NullPointerTester {
   }
 
   /**
-   * Verifies that {@code ctor} produces a {@link NullPointerException} or
-   * {@link UnsupportedOperationException} whenever <i>any</i> of its
-   * non-{@link Nullable} parameters are null.
+   * Verifies that {@code ctor} produces a {@link NullPointerException} or {@link
+   * UnsupportedOperationException} whenever <i>any</i> of its non-nullable parameters are null.
    */
   public void testConstructor(Constructor<?> ctor) {
     Class<?> declaringClass = ctor.getDeclaringClass();
-    checkArgument(Modifier.isStatic(declaringClass.getModifiers())
-        || declaringClass.getEnclosingClass() == null,
-        "Cannot test constructor of non-static inner class: %s", declaringClass.getName());
+    checkArgument(
+        Modifier.isStatic(declaringClass.getModifiers())
+            || declaringClass.getEnclosingClass() == null,
+        "Cannot test constructor of non-static inner class: %s",
+        declaringClass.getName());
     Class<?>[] types = ctor.getParameterTypes();
     for (int nullIndex = 0; nullIndex < types.length; nullIndex++) {
       testConstructorParameter(ctor, nullIndex);
@@ -196,13 +201,11 @@ public final class NullPointerTester {
   }
 
   /**
-   * Verifies that {@code method} produces a {@link NullPointerException} or
-   * {@link UnsupportedOperationException} when the parameter in position {@code
-   * paramIndex} is null.  If this parameter is marked {@link Nullable}, this
-   * method does nothing.
+   * Verifies that {@code method} produces a {@link NullPointerException} or {@link
+   * UnsupportedOperationException} when the parameter in position {@code paramIndex} is null. If
+   * this parameter is marked nullable, this method does nothing.
    *
-   * @param instance the instance to invoke {@code method} on, or null if
-   *     {@code method} is static
+   * @param instance the instance to invoke {@code method} on, or null if {@code method} is static
    */
   public void testMethodParameter(
       @Nullable final Object instance, final Method method, int paramIndex) {
@@ -211,10 +214,9 @@ public final class NullPointerTester {
   }
 
   /**
-   * Verifies that {@code ctor} produces a {@link NullPointerException} or
-   * {@link UnsupportedOperationException} when the parameter in position {@code
-   * paramIndex} is null.  If this parameter is marked {@link Nullable}, this
-   * method does nothing.
+   * Verifies that {@code ctor} produces a {@link NullPointerException} or {@link
+   * UnsupportedOperationException} when the parameter in position {@code paramIndex} is null. If
+   * this parameter is marked nullable, this method does nothing.
    */
   public void testConstructorParameter(Constructor<?> ctor, int paramIndex) {
     ctor.setAccessible(true);
@@ -223,31 +225,30 @@ public final class NullPointerTester {
 
   /** Visibility of any method or constructor. */
   public enum Visibility {
-
     PACKAGE {
-      @Override boolean isVisible(int modifiers) {
+      @Override
+      boolean isVisible(int modifiers) {
         return !Modifier.isPrivate(modifiers);
       }
     },
 
     PROTECTED {
-      @Override boolean isVisible(int modifiers) {
+      @Override
+      boolean isVisible(int modifiers) {
         return Modifier.isPublic(modifiers) || Modifier.isProtected(modifiers);
       }
     },
 
     PUBLIC {
-      @Override boolean isVisible(int modifiers) {
+      @Override
+      boolean isVisible(int modifiers) {
         return Modifier.isPublic(modifiers);
       }
     };
 
     abstract boolean isVisible(int modifiers);
 
-    /**
-     * Returns {@code true} if {@code member} is visible under {@code this}
-     * visibility.
-     */
+    /** Returns {@code true} if {@code member} is visible under {@code this} visibility. */
     final boolean isVisible(Member member) {
       return isVisible(member.getModifiers());
     }
@@ -277,7 +278,7 @@ public final class NullPointerTester {
       // a file.
       String visiblePackage = Reflection.getPackageName(cls);
       ImmutableList.Builder<Method> builder = ImmutableList.builder();
-      for (Class<?> type : TypeToken.of(cls).getTypes().classes().rawTypes()) {
+      for (Class<?> type : TypeToken.of(cls).getTypes().rawTypes()) {
         if (!Reflection.getPackageName(type).equals(visiblePackage)) {
           break;
         }
@@ -304,31 +305,31 @@ public final class NullPointerTester {
       this.parameterTypes = parameterTypes;
     }
 
-    @Override public boolean equals(Object obj) {
+    @Override
+    public boolean equals(Object obj) {
       if (obj instanceof Signature) {
         Signature that = (Signature) obj;
-        return name.equals(that.name)
-            && parameterTypes.equals(that.parameterTypes);
+        return name.equals(that.name) && parameterTypes.equals(that.parameterTypes);
       }
       return false;
     }
 
-    @Override public int hashCode() {
+    @Override
+    public int hashCode() {
       return Objects.hashCode(name, parameterTypes);
     }
   }
 
   /**
-   * Verifies that {@code invokable} produces a {@link NullPointerException} or
-   * {@link UnsupportedOperationException} when the parameter in position {@code
-   * paramIndex} is null.  If this parameter is marked {@link Nullable}, this
-   * method does nothing.
+   * Verifies that {@code invokable} produces a {@link NullPointerException} or {@link
+   * UnsupportedOperationException} when the parameter in position {@code paramIndex} is null. If
+   * this parameter is marked nullable, this method does nothing.
    *
-   * @param instance the instance to invoke {@code invokable} on, or null if
-   *     {@code invokable} is static
+   * @param instance the instance to invoke {@code invokable} on, or null if {@code invokable} is
+   *     static
    */
-  private void testParameter(Object instance, Invokable<?, ?> invokable,
-      int paramIndex, Class<?> testedClass) {
+  private void testParameter(
+      Object instance, Invokable<?, ?> invokable, int paramIndex, Class<?> testedClass) {
     if (isPrimitiveOrNullable(invokable.getParameters().get(paramIndex))) {
       return; // there's nothing to test
     }
@@ -337,15 +338,30 @@ public final class NullPointerTester {
       @SuppressWarnings("unchecked") // We'll get a runtime exception if the type is wrong.
       Invokable<Object, ?> unsafe = (Invokable<Object, ?>) invokable;
       unsafe.invoke(instance, params);
-      Assert.fail("No exception thrown for parameter at index " + paramIndex
-          + " from " + invokable + Arrays.toString(params) + " for " + testedClass);
+      Assert.fail(
+          "No exception thrown for parameter at index "
+              + paramIndex
+              + " from "
+              + invokable
+              + Arrays.toString(params)
+              + " for "
+              + testedClass);
     } catch (InvocationTargetException e) {
       Throwable cause = e.getCause();
       if (policy.isExpectedType(cause)) {
         return;
       }
-      AssertionFailedError error = new AssertionFailedError(
-          "wrong exception thrown from " + invokable + ": " + cause);
+      AssertionFailedError error =
+          new AssertionFailedError(
+              String.format(
+                  "wrong exception thrown from %s when passing null to %s parameter at index %s.%n"
+                      + "Full parameters: %s%n"
+                      + "Actual exception message: %s",
+                  invokable,
+                  invokable.getParameters().get(paramIndex).getType(),
+                  paramIndex,
+                  Arrays.toString(params),
+                  cause));
       error.initCause(cause);
       throw error;
     } catch (IllegalAccessException e) {
@@ -397,10 +413,8 @@ public final class NullPointerTester {
       return defaultType;
     }
     if (type.getRawType() == Converter.class) {
-      TypeToken<?> convertFromType = type.resolveType(
-          Converter.class.getTypeParameters()[0]);
-      TypeToken<?> convertToType = type.resolveType(
-          Converter.class.getTypeParameters()[1]);
+      TypeToken<?> convertFromType = type.resolveType(Converter.class.getTypeParameters()[0]);
+      TypeToken<?> convertToType = type.resolveType(Converter.class.getTypeParameters()[1]);
       @SuppressWarnings("unchecked") // returns default for both F and T
       T defaultConverter = (T) defaultConverter(convertFromType, convertToType);
       return defaultConverter;
@@ -414,10 +428,13 @@ public final class NullPointerTester {
   private <F, T> Converter<F, T> defaultConverter(
       final TypeToken<F> convertFromType, final TypeToken<T> convertToType) {
     return new Converter<F, T>() {
-      @Override protected T doForward(F a) {
+      @Override
+      protected T doForward(F a) {
         return doConvert(convertToType);
       }
-      @Override protected F doBackward(T b) {
+
+      @Override
+      protected F doBackward(T b) {
         return doConvert(convertFromType);
       }
 
@@ -429,8 +446,7 @@ public final class NullPointerTester {
 
   private static TypeToken<?> getFirstTypeParameter(Type type) {
     if (type instanceof ParameterizedType) {
-      return TypeToken.of(
-          ((ParameterizedType) type).getActualTypeArguments()[0]);
+      return TypeToken.of(((ParameterizedType) type).getActualTypeArguments()[0]);
     } else {
       return TypeToken.of(Object.class);
     }
@@ -438,7 +454,8 @@ public final class NullPointerTester {
 
   private <T> T newDefaultReturningProxy(final TypeToken<T> type) {
     return new DummyProxy() {
-      @Override <R> R dummyReturnValue(TypeToken<R> returnType) {
+      @Override
+      <R> R dummyReturnValue(TypeToken<R> returnType) {
         return getDefaultValue(returnType);
       }
     }.newProxy(type);
@@ -456,22 +473,106 @@ public final class NullPointerTester {
     return param.getType().getRawType().isPrimitive() || isNullable(param);
   }
 
-  private static boolean isNullable(Parameter param) {
-    return param.isAnnotationPresent(Nullable.class);
+  private static final ImmutableSet<String> NULLABLE_ANNOTATION_SIMPLE_NAMES =
+      ImmutableSet.of(
+          "CheckForNull", "Nullable", "NullableDecl", "NullableType", "ParametricNullness");
+
+  static boolean isNullable(Invokable<?, ?> invokable) {
+    return isNullable(invokable.getAnnotatedReturnType().getAnnotations())
+        || isNullable(invokable.getAnnotations());
+  }
+
+  static boolean isNullable(Parameter param) {
+    return isNullable(param.getAnnotatedType().getAnnotations())
+        || isNullable(param.getAnnotations())
+        || isNullableTypeVariable(param.getAnnotatedType().getType());
+  }
+
+  private static boolean isNullableTypeVariable(Type type) {
+    if (!(type instanceof TypeVariable)) {
+      return false;
+    }
+    TypeVariable<?> var = (TypeVariable<?>) type;
+    AnnotatedType[] bounds = GET_ANNOTATED_BOUNDS.apply(var);
+    for (AnnotatedType bound : bounds) {
+      // Until Java 15, the isNullableTypeVariable case here won't help:
+      // https://bugs.openjdk.java.net/browse/JDK-8202469
+      if (isNullable(bound.getAnnotations()) || isNullableTypeVariable(bound.getType())) {
+        return true;
+      }
+    }
+    return false;
+  }
+
+  private static boolean isNullable(Annotation[] annotations) {
+    for (Annotation annotation : annotations) {
+      if (NULLABLE_ANNOTATION_SIMPLE_NAMES.contains(annotation.annotationType().getSimpleName())) {
+        return true;
+      }
+    }
+    return false;
+  }
+
+  // This is currently required because of j2objc restrictions.
+  private static final Function<TypeVariable<?>, AnnotatedType[]> GET_ANNOTATED_BOUNDS =
+      initGetAnnotatedBounds();
+
+  private static Function<TypeVariable<?>, AnnotatedType[]> initGetAnnotatedBounds() {
+    AnnotatedType[] noBounds = new AnnotatedType[0];
+    Method getAnnotatedBounds;
+    try {
+      getAnnotatedBounds = TypeVariable.class.getMethod("getAnnotatedBounds");
+    } catch (ReflectiveOperationException e) {
+      return v -> noBounds;
+    }
+    return v -> {
+      try {
+        return (AnnotatedType[]) getAnnotatedBounds.invoke(v);
+      } catch (ReflectiveOperationException e) {
+        return noBounds;
+      }
+    };
   }
 
   private boolean isIgnored(Member member) {
-    return member.isSynthetic() || ignoredMembers.contains(member);
+    return member.isSynthetic() || ignoredMembers.contains(member) || isEquals(member);
   }
 
   /**
-   * Strategy for exception type matching used by {@link NullPointerTester}.
+   * Returns true if the given member is a method that overrides {@link Object#equals(Object)}.
+   *
+   * <p>The documentation for {@link Object#equals} says it should accept null, so don't require an
+   * explicit {@code @Nullable} annotation (see <a
+   * href="https://github.com/google/guava/issues/1819">#1819</a>).
+   *
+   * <p>It is not necessary to consider visibility, return type, or type parameter declarations. The
+   * declaration of a method with the same name and formal parameters as {@link Object#equals} that
+   * is not public and boolean-returning, or that declares any type parameters, would be rejected at
+   * compile-time.
    */
+  private static boolean isEquals(Member member) {
+    if (!(member instanceof Method)) {
+      return false;
+    }
+    Method method = (Method) member;
+    if (!method.getName().contentEquals("equals")) {
+      return false;
+    }
+    Class<?>[] parameters = method.getParameterTypes();
+    if (parameters.length != 1) {
+      return false;
+    }
+    if (!parameters[0].equals(Object.class)) {
+      return false;
+    }
+    return true;
+  }
+
+  /** Strategy for exception type matching used by {@link NullPointerTester}. */
   private enum ExceptionTypePolicy {
 
     /**
-     * Exceptions should be {@link NullPointerException} or
-     * {@link UnsupportedOperationException}.
+     * Exceptions should be {@link NullPointerException} or {@link UnsupportedOperationException}.
      */
     NPE_OR_UOE() {
       @Override
@@ -482,8 +583,7 @@ public final class NullPointerTester {
     },
 
     /**
-     * Exceptions should be {@link NullPointerException},
-     * {@link IllegalArgumentException}, or
+     * Exceptions should be {@link NullPointerException}, {@link IllegalArgumentException}, or
      * {@link UnsupportedOperationException}.
      */
     NPE_IAE_OR_UOE() {
