@@ -16,7 +16,7 @@
 
 package com.google.common.base;
 
-import static com.google.common.base.CharMatcher.WHITESPACE;
+import static com.google.common.base.CharMatcher.whitespace;
 import static com.google.common.collect.Lists.newArrayList;
 
 import com.google.common.annotations.GwtCompatible;
@@ -26,10 +26,6 @@ import com.google.common.testing.ClassSanityTester;
 import com.google.common.testing.EqualsTester;
 import com.google.common.testing.NullPointerTester;
 import com.google.common.testing.SerializableTester;
-
-import junit.framework.AssertionFailedError;
-import junit.framework.TestCase;
-
 import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -38,6 +34,8 @@ import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
 import java.util.regex.Pattern;
+import junit.framework.AssertionFailedError;
+import junit.framework.TestCase;
 
 /**
  * Unit test for {@link Predicates}.
@@ -50,27 +48,33 @@ public class PredicatesTest extends TestCase {
   private static final Predicate<Integer> FALSE = Predicates.alwaysFalse();
   private static final Predicate<Integer> NEVER_REACHED =
       new Predicate<Integer>() {
-    @Override
-    public boolean apply(Integer i) {
-      throw new AssertionFailedError(
-          "This predicate should never have been evaluated");
-    }
-  };
+        @Override
+        public boolean apply(Integer i) {
+          throw new AssertionFailedError("This predicate should never have been evaluated");
+        }
+      };
 
   /** Instantiable predicate with reasonable hashCode() and equals() methods. */
   static class IsOdd implements Predicate<Integer>, Serializable {
     private static final long serialVersionUID = 0x150ddL;
+
     @Override
     public boolean apply(Integer i) {
       return (i.intValue() & 1) == 1;
     }
-    @Override public int hashCode() {
+
+    @Override
+    public int hashCode() {
       return 0x150dd;
     }
-    @Override public boolean equals(Object obj) {
+
+    @Override
+    public boolean equals(Object obj) {
       return obj instanceof IsOdd;
     }
-    @Override public String toString() {
+
+    @Override
+    public String toString() {
       return "IsOdd";
     }
   }
@@ -78,8 +82,8 @@ public class PredicatesTest extends TestCase {
   /**
    * Generates a new Predicate per call.
    *
-   * <p>Creating a new Predicate each time helps catch cases where code is
-   * using {@code x == y} instead of {@code x.equals(y)}.
+   * <p>Creating a new Predicate each time helps catch cases where code is using {@code x == y}
+   * instead of {@code x.equals(y)}.
    */
   private static IsOdd isOdd() {
     return new IsOdd();
@@ -101,7 +105,7 @@ public class PredicatesTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   public void testAlwaysTrue_serialization() {
     checkSerialization(Predicates.alwaysTrue());
   }
@@ -122,7 +126,7 @@ public class PredicatesTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   public void testAlwaysFalse_serialization() {
     checkSerialization(Predicates.alwaysFalse());
   }
@@ -171,7 +175,7 @@ public class PredicatesTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   public void testNot_serialization() {
     checkSerialization(Predicates.not(isOdd()));
   }
@@ -194,7 +198,7 @@ public class PredicatesTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   @SuppressWarnings("unchecked") // varargs
   public void testAnd_serializationNoArgs() {
     checkSerialization(Predicates.and());
@@ -209,8 +213,7 @@ public class PredicatesTest extends TestCase {
   public void testAnd_equalityOneArg() {
     Object[] notEqualObjects = {Predicates.and(NEVER_REACHED, FALSE)};
     new EqualsTester()
-        .addEqualityGroup(
-            Predicates.and(NEVER_REACHED), Predicates.and(NEVER_REACHED))
+        .addEqualityGroup(Predicates.and(NEVER_REACHED), Predicates.and(NEVER_REACHED))
         .addEqualityGroup(notEqualObjects)
         .addEqualityGroup(Predicates.and(isOdd()))
         .addEqualityGroup(Predicates.and())
@@ -218,7 +221,7 @@ public class PredicatesTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   @SuppressWarnings("unchecked") // varargs
   public void testAnd_serializationOneArg() {
     checkSerialization(Predicates.and(isOdd()));
@@ -233,16 +236,14 @@ public class PredicatesTest extends TestCase {
   @SuppressWarnings("unchecked") // varargs
   public void testAnd_equalityBinary() {
     new EqualsTester()
-        .addEqualityGroup(
-            Predicates.and(TRUE, NEVER_REACHED),
-            Predicates.and(TRUE, NEVER_REACHED))
+        .addEqualityGroup(Predicates.and(TRUE, NEVER_REACHED), Predicates.and(TRUE, NEVER_REACHED))
         .addEqualityGroup(Predicates.and(NEVER_REACHED, TRUE))
         .addEqualityGroup(Predicates.and(TRUE))
         .addEqualityGroup(Predicates.or(TRUE, NEVER_REACHED))
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   public void testAnd_serializationBinary() {
     checkSerialization(Predicates.and(TRUE, isOdd()));
   }
@@ -267,7 +268,7 @@ public class PredicatesTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   @SuppressWarnings("unchecked") // varargs
   public void testAnd_serializationTernary() {
     checkSerialization(Predicates.and(TRUE, isOdd(), FALSE));
@@ -294,7 +295,7 @@ public class PredicatesTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   @SuppressWarnings("unchecked") // varargs
   public void testAnd_serializationIterable() {
     checkSerialization(Predicates.and(Arrays.asList(TRUE, FALSE)));
@@ -319,12 +320,13 @@ public class PredicatesTest extends TestCase {
 
   public void testAnd_iterableDefensivelyCopied() {
     final List<Predicate<Object>> list = newArrayList();
-    Iterable<Predicate<Object>> iterable = new Iterable<Predicate<Object>>() {
-      @Override
-      public Iterator<Predicate<Object>> iterator() {
-        return list.iterator();
-      }
-    };
+    Iterable<Predicate<Object>> iterable =
+        new Iterable<Predicate<Object>>() {
+          @Override
+          public Iterator<Predicate<Object>> iterator() {
+            return list.iterator();
+          }
+        };
     Predicate<Object> predicate = Predicates.and(iterable);
     assertTrue(predicate.apply(1));
     list.add(Predicates.alwaysFalse());
@@ -349,7 +351,7 @@ public class PredicatesTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   @SuppressWarnings("unchecked") // varargs
   public void testOr_serializationNoArgs() {
     checkSerialization(Predicates.or());
@@ -364,8 +366,7 @@ public class PredicatesTest extends TestCase {
   @SuppressWarnings("unchecked") // varargs
   public void testOr_equalityOneArg() {
     new EqualsTester()
-        .addEqualityGroup(
-            Predicates.or(NEVER_REACHED), Predicates.or(NEVER_REACHED))
+        .addEqualityGroup(Predicates.or(NEVER_REACHED), Predicates.or(NEVER_REACHED))
         .addEqualityGroup(Predicates.or(NEVER_REACHED, TRUE))
         .addEqualityGroup(Predicates.or(TRUE))
         .addEqualityGroup(Predicates.or())
@@ -373,7 +374,7 @@ public class PredicatesTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   @SuppressWarnings("unchecked") // varargs
   public void testOr_serializationOneArg() {
     checkSerialization(Predicates.or(isOdd()));
@@ -392,16 +393,14 @@ public class PredicatesTest extends TestCase {
   @SuppressWarnings("unchecked") // varargs
   public void testOr_equalityBinary() {
     new EqualsTester()
-        .addEqualityGroup(
-            Predicates.or(FALSE, NEVER_REACHED),
-            Predicates.or(FALSE, NEVER_REACHED))
+        .addEqualityGroup(Predicates.or(FALSE, NEVER_REACHED), Predicates.or(FALSE, NEVER_REACHED))
         .addEqualityGroup(Predicates.or(NEVER_REACHED, FALSE))
         .addEqualityGroup(Predicates.or(TRUE))
         .addEqualityGroup(Predicates.and(FALSE, NEVER_REACHED))
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   public void testOr_serializationBinary() {
     checkSerialization(Predicates.or(isOdd(), TRUE));
   }
@@ -418,15 +417,14 @@ public class PredicatesTest extends TestCase {
   public void testOr_equalityTernary() {
     new EqualsTester()
         .addEqualityGroup(
-            Predicates.or(FALSE, NEVER_REACHED, TRUE),
-            Predicates.or(FALSE, NEVER_REACHED, TRUE))
+            Predicates.or(FALSE, NEVER_REACHED, TRUE), Predicates.or(FALSE, NEVER_REACHED, TRUE))
         .addEqualityGroup(Predicates.or(TRUE, NEVER_REACHED, FALSE))
         .addEqualityGroup(Predicates.or(TRUE))
         .addEqualityGroup(Predicates.and(FALSE, NEVER_REACHED, TRUE))
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   @SuppressWarnings("unchecked") // varargs
   public void testOr_serializationTernary() {
     checkSerialization(Predicates.or(FALSE, isOdd(), TRUE));
@@ -434,8 +432,7 @@ public class PredicatesTest extends TestCase {
 
   @SuppressWarnings("unchecked") // varargs
   public void testOr_applyIterable() {
-    Predicate<Integer> vacuouslyFalse =
-        Predicates.or(Collections.<Predicate<Integer>>emptyList());
+    Predicate<Integer> vacuouslyFalse = Predicates.or(Collections.<Predicate<Integer>>emptyList());
     Predicate<Integer> troo = Predicates.or(Collections.singletonList(TRUE));
     /*
      * newLinkedList() takes varargs. TRUE and FALSE are both instances of
@@ -460,7 +457,7 @@ public class PredicatesTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   @SuppressWarnings("unchecked") // varargs
   public void testOr_serializationIterable() {
     Predicate<Integer> pre = Predicates.or(Arrays.asList(TRUE, FALSE));
@@ -487,12 +484,13 @@ public class PredicatesTest extends TestCase {
 
   public void testOr_iterableDefensivelyCopied() {
     final List<Predicate<Object>> list = newArrayList();
-    Iterable<Predicate<Object>> iterable = new Iterable<Predicate<Object>>() {
-      @Override
-      public Iterator<Predicate<Object>> iterator() {
-        return list.iterator();
-      }
-    };
+    Iterable<Predicate<Object>> iterable =
+        new Iterable<Predicate<Object>>() {
+          @Override
+          public Iterator<Predicate<Object>> iterator() {
+            return list.iterator();
+          }
+        };
     Predicate<Object> predicate = Predicates.or(iterable);
     assertFalse(predicate.apply(1));
     list.add(Predicates.alwaysTrue());
@@ -519,7 +517,7 @@ public class PredicatesTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   public void testIsEqualTo_serialization() {
     checkSerialization(Predicates.equalTo(1));
   }
@@ -538,19 +536,17 @@ public class PredicatesTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   public void testIsEqualToNull_serialization() {
     checkSerialization(Predicates.equalTo(null));
   }
 
   /**
-   * Tests for Predicates.instanceOf(x).
-   * TODO: Fix the comment style after fixing annotation stripper to remove
-   * comments properly.  Currently, all tests before the comments are removed
-   * as well.
+   * Tests for Predicates.instanceOf(x). TODO: Fix the comment style after fixing annotation
+   * stripper to remove comments properly. Currently, all tests before the comments are removed as
+   * well.
    */
-
-  @GwtIncompatible("Predicates.instanceOf")
+  @GwtIncompatible // Predicates.instanceOf
   public void testIsInstanceOf_apply() {
     Predicate<Object> isInteger = Predicates.instanceOf(Integer.class);
 
@@ -560,7 +556,7 @@ public class PredicatesTest extends TestCase {
     assertFalse(isInteger.apply(null));
   }
 
-  @GwtIncompatible("Predicates.instanceOf")
+  @GwtIncompatible // Predicates.instanceOf
   public void testIsInstanceOf_subclass() {
     Predicate<Object> isNumber = Predicates.instanceOf(Number.class);
 
@@ -570,7 +566,7 @@ public class PredicatesTest extends TestCase {
     assertFalse(isNumber.apply(null));
   }
 
-  @GwtIncompatible("Predicates.instanceOf")
+  @GwtIncompatible // Predicates.instanceOf
   public void testIsInstanceOf_interface() {
     Predicate<Object> isComparable = Predicates.instanceOf(Comparable.class);
 
@@ -580,25 +576,24 @@ public class PredicatesTest extends TestCase {
     assertFalse(isComparable.apply(null));
   }
 
-  @GwtIncompatible("Predicates.instanceOf")
+  @GwtIncompatible // Predicates.instanceOf
   public void testIsInstanceOf_equality() {
     new EqualsTester()
         .addEqualityGroup(
-            Predicates.instanceOf(Integer.class),
-            Predicates.instanceOf(Integer.class))
+            Predicates.instanceOf(Integer.class), Predicates.instanceOf(Integer.class))
         .addEqualityGroup(Predicates.instanceOf(Number.class))
         .addEqualityGroup(Predicates.instanceOf(Float.class))
         .testEquals();
   }
 
-  @GwtIncompatible("Predicates.instanceOf, SerializableTester")
+  @GwtIncompatible // Predicates.instanceOf, SerializableTester
   public void testIsInstanceOf_serialization() {
     checkSerialization(Predicates.instanceOf(Integer.class));
   }
 
-  @GwtIncompatible("Predicates.assignableFrom")
-  public void testIsAssignableFrom_apply() {
-    Predicate<Class<?>> isInteger = Predicates.assignableFrom(Integer.class);
+  @GwtIncompatible // Predicates.subtypeOf
+  public void testSubtypeOf_apply() {
+    Predicate<Class<?>> isInteger = Predicates.subtypeOf(Integer.class);
 
     assertTrue(isInteger.apply(Integer.class));
     assertFalse(isInteger.apply(Float.class));
@@ -606,43 +601,39 @@ public class PredicatesTest extends TestCase {
     try {
       isInteger.apply(null);
       fail();
-    } catch (NullPointerException expected) {}
+    } catch (NullPointerException expected) {
+    }
   }
 
-  @GwtIncompatible("Predicates.assignableFrom")
-  public void testIsAssignableFrom_subclass() {
-    Predicate<Class<?>> isNumber = Predicates.assignableFrom(Number.class);
+  @GwtIncompatible // Predicates.subtypeOf
+  public void testSubtypeOf_subclass() {
+    Predicate<Class<?>> isNumber = Predicates.subtypeOf(Number.class);
 
     assertTrue(isNumber.apply(Integer.class));
     assertTrue(isNumber.apply(Float.class));
   }
 
-  @GwtIncompatible("Predicates.assignableFrom")
-  public void testIsAssignableFrom_interface() {
-    Predicate<Class<?>> isComparable =
-        Predicates.assignableFrom(Comparable.class);
+  @GwtIncompatible // Predicates.subtypeOf
+  public void testSubtypeOf_interface() {
+    Predicate<Class<?>> isComparable = Predicates.subtypeOf(Comparable.class);
 
     assertTrue(isComparable.apply(Integer.class));
     assertTrue(isComparable.apply(Float.class));
   }
 
-  @GwtIncompatible("Predicates.assignableFrom")
-  public void testIsAssignableFrom_equality() {
+  @GwtIncompatible // Predicates.subtypeOf
+  public void testSubtypeOf_equality() {
     new EqualsTester()
-        .addEqualityGroup(
-            Predicates.assignableFrom(Integer.class),
-            Predicates.assignableFrom(Integer.class))
-        .addEqualityGroup(Predicates.assignableFrom(Number.class))
-        .addEqualityGroup(Predicates.assignableFrom(Float.class))
+        .addEqualityGroup(Predicates.subtypeOf(Integer.class))
+        .addEqualityGroup(Predicates.subtypeOf(Number.class))
+        .addEqualityGroup(Predicates.subtypeOf(Float.class))
         .testEquals();
   }
 
-  @GwtIncompatible("Predicates.assignableFrom, SerializableTester")
-  public void testIsAssignableFrom_serialization() {
-    Predicate<Class<?>> predicate =
-        Predicates.assignableFrom(Integer.class);
-    Predicate<Class<?>> reserialized =
-        SerializableTester.reserializeAndAssert(predicate);
+  @GwtIncompatible // Predicates.subtypeOf, SerializableTester
+  public void testSubtypeOf_serialization() {
+    Predicate<Class<?>> predicate = Predicates.subtypeOf(Integer.class);
+    Predicate<Class<?>> reserialized = SerializableTester.reserializeAndAssert(predicate);
 
     assertEvalsLike(predicate, reserialized, Integer.class);
     assertEvalsLike(predicate, reserialized, Float.class);
@@ -666,7 +657,7 @@ public class PredicatesTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   public void testIsNull_serialization() {
     Predicate<String> pre = Predicates.isNull();
     Predicate<String> post = SerializableTester.reserializeAndAssert(pre);
@@ -687,7 +678,7 @@ public class PredicatesTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   public void testNotNull_serialization() {
     checkSerialization(Predicates.notNull());
   }
@@ -709,13 +700,16 @@ public class PredicatesTest extends TestCase {
     Collection<Integer> differentNums = ImmutableSet.of(1, 3, 5);
 
     new EqualsTester()
-        .addEqualityGroup(Predicates.in(nums), Predicates.in(nums),
-            Predicates.in(sameOrder), Predicates.in(differentOrder))
+        .addEqualityGroup(
+            Predicates.in(nums),
+            Predicates.in(nums),
+            Predicates.in(sameOrder),
+            Predicates.in(differentOrder))
         .addEqualityGroup(Predicates.in(differentNums))
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   public void testIn_serialization() {
     checkSerialization(Predicates.in(Arrays.asList(1, 2, 3, null)));
   }
@@ -724,12 +718,13 @@ public class PredicatesTest extends TestCase {
     class CollectionThatThrowsNPE<T> extends ArrayList<T> {
       private static final long serialVersionUID = 1L;
 
-      @Override public boolean contains(Object element) {
+      @Override
+      public boolean contains(Object element) {
         Preconditions.checkNotNull(element);
         return super.contains(element);
       }
     }
-    Collection<Integer> nums = new CollectionThatThrowsNPE<Integer>();
+    Collection<Integer> nums = new CollectionThatThrowsNPE<>();
     Predicate<Integer> isFalse = Predicates.in(nums);
     assertFalse(isFalse.apply(null));
   }
@@ -738,11 +733,12 @@ public class PredicatesTest extends TestCase {
     class CollectionThatThrowsCCE<T> extends ArrayList<T> {
       private static final long serialVersionUID = 1L;
 
-      @Override public boolean contains(Object element) {
+      @Override
+      public boolean contains(Object element) {
         throw new ClassCastException("");
       }
     }
-    Collection<Integer> nums = new CollectionThatThrowsCCE<Integer>();
+    Collection<Integer> nums = new CollectionThatThrowsCCE<>();
     nums.add(3);
     Predicate<Integer> isThree = Predicates.in(nums);
     assertFalse(isThree.apply(3));
@@ -761,26 +757,30 @@ public class PredicatesTest extends TestCase {
     // Predicate<Integer> p4 = Predicates.<Integer>in(nums);
   }
 
-  @GwtIncompatible("NullPointerTester")
+  @GwtIncompatible // NullPointerTester
   public void testNullPointerExceptions() {
     NullPointerTester tester = new NullPointerTester();
     tester.testAllPublicStaticMethods(Predicates.class);
   }
 
   @SuppressWarnings("unchecked") // varargs
-  @GwtIncompatible("SerializbleTester")
+  @GwtIncompatible // SerializbleTester
   public void testCascadingSerialization() throws Exception {
     // Eclipse says Predicate<Integer>; javac says Predicate<Object>.
-    Predicate<? super Integer> nasty = Predicates.not(Predicates.and(
-        Predicates.or(
-            Predicates.equalTo((Object) 1), Predicates.equalTo(null),
-            Predicates.alwaysFalse(), Predicates.alwaysTrue(),
-            Predicates.isNull(), Predicates.notNull(),
-            Predicates.in(Arrays.asList(1)))));
+    Predicate<? super Integer> nasty =
+        Predicates.not(
+            Predicates.and(
+                Predicates.or(
+                    Predicates.equalTo((Object) 1),
+                    Predicates.equalTo(null),
+                    Predicates.alwaysFalse(),
+                    Predicates.alwaysTrue(),
+                    Predicates.isNull(),
+                    Predicates.notNull(),
+                    Predicates.in(Arrays.asList(1)))));
     assertEvalsToFalse(nasty);
 
-    Predicate<? super Integer> stillNasty =
-        SerializableTester.reserializeAndAssert(nasty);
+    Predicate<? super Integer> stillNasty = SerializableTester.reserializeAndAssert(nasty);
 
     assertEvalsToFalse(stillNasty);
   }
@@ -791,7 +791,7 @@ public class PredicatesTest extends TestCase {
 
     @Override
     public String apply(String string) {
-      return WHITESPACE.trimFrom(string);
+      return whitespace().trimFrom(string);
     }
   }
 
@@ -815,7 +815,7 @@ public class PredicatesTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   public void testComposeSerialization() {
     Function<String, String> trim = TrimStringFunction.INSTANCE;
     Predicate<String> equalsFoo = Predicates.equalTo("Foo");
@@ -824,30 +824,26 @@ public class PredicatesTest extends TestCase {
   }
 
   /**
-   * Tests for Predicates.contains(Pattern) and .containsPattern(String).
-   * We assume the regex level works, so there are only trivial tests of that
-   * aspect.
-   * TODO: Fix comment style once annotation stripper is fixed.
+   * Tests for Predicates.contains(Pattern) and .containsPattern(String). We assume the regex level
+   * works, so there are only trivial tests of that aspect. TODO: Fix comment style once annotation
+   * stripper is fixed.
    */
-
-  @GwtIncompatible("Predicates.containsPattern")
+  @GwtIncompatible // Predicates.containsPattern
   public void testContainsPattern_apply() {
-    Predicate<CharSequence> isFoobar =
-        Predicates.containsPattern("^Fo.*o.*bar$");
+    Predicate<CharSequence> isFoobar = Predicates.containsPattern("^Fo.*o.*bar$");
     assertTrue(isFoobar.apply("Foxyzoabcbar"));
     assertFalse(isFoobar.apply("Foobarx"));
   }
 
-  @GwtIncompatible("Predicates.containsPattern")
+  @GwtIncompatible // Predicates.containsPattern
   public void testContains_apply() {
-    Predicate<CharSequence> isFoobar =
-        Predicates.contains(Pattern.compile("^Fo.*o.*bar$"));
+    Predicate<CharSequence> isFoobar = Predicates.contains(Pattern.compile("^Fo.*o.*bar$"));
 
     assertTrue(isFoobar.apply("Foxyzoabcbar"));
     assertFalse(isFoobar.apply("Foobarx"));
   }
 
-  @GwtIncompatible("NullPointerTester")
+  @GwtIncompatible // NullPointerTester
   public void testContainsPattern_nulls() throws Exception {
     NullPointerTester tester = new NullPointerTester();
     Predicate<CharSequence> isWooString = Predicates.containsPattern("Woo");
@@ -855,35 +851,30 @@ public class PredicatesTest extends TestCase {
     tester.testAllPublicInstanceMethods(isWooString);
   }
 
-  @GwtIncompatible("NullPointerTester")
+  @GwtIncompatible // NullPointerTester
   public void testContains_nulls() throws Exception {
     NullPointerTester tester = new NullPointerTester();
-    Predicate<CharSequence> isWooPattern =
-        Predicates.contains(Pattern.compile("Woo"));
+    Predicate<CharSequence> isWooPattern = Predicates.contains(Pattern.compile("Woo"));
 
     tester.testAllPublicInstanceMethods(isWooPattern);
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   public void testContainsPattern_serialization() {
     Predicate<CharSequence> pre = Predicates.containsPattern("foo");
     Predicate<CharSequence> post = SerializableTester.reserializeAndAssert(pre);
     assertEquals(pre.apply("foo"), post.apply("foo"));
   }
 
-  @GwtIncompatible("java.util.regex.Pattern")
+  @GwtIncompatible // java.util.regex.Pattern
   public void testContains_equals() {
     new EqualsTester()
         .addEqualityGroup(
-            Predicates.contains(Pattern.compile("foo")),
-            Predicates.containsPattern("foo"))
-        .addEqualityGroup(
-            Predicates.contains(
-                Pattern.compile("foo", Pattern.CASE_INSENSITIVE)))
-        .addEqualityGroup(
-            Predicates.containsPattern("bar"))
+            Predicates.contains(Pattern.compile("foo")), Predicates.containsPattern("foo"))
+        .addEqualityGroup(Predicates.contains(Pattern.compile("foo", Pattern.CASE_INSENSITIVE)))
+        .addEqualityGroup(Predicates.containsPattern("bar"))
         .testEquals();
-      }
+  }
 
   public void assertEqualHashCode(
       Predicate<? super Integer> expected, Predicate<? super Integer> actual) {
@@ -895,30 +886,24 @@ public class PredicatesTest extends TestCase {
     Predicate<Integer> p2 = isOdd();
 
     // Make sure that hash codes are not computed per-instance.
-    assertEqualHashCode(
-        Predicates.not(p1),
-        Predicates.not(p1));
+    assertEqualHashCode(Predicates.not(p1), Predicates.not(p1));
 
-    assertEqualHashCode(
-        Predicates.and(p1, p2),
-        Predicates.and(p1, p2));
+    assertEqualHashCode(Predicates.and(p1, p2), Predicates.and(p1, p2));
 
-    assertEqualHashCode(
-        Predicates.or(p1, p2),
-        Predicates.or(p1, p2));
+    assertEqualHashCode(Predicates.or(p1, p2), Predicates.or(p1, p2));
 
     // While not a contractual requirement, we'd like the hash codes for ands
     // & ors of the same predicates to not collide.
     assertTrue(Predicates.and(p1, p2).hashCode() != Predicates.or(p1, p2).hashCode());
   }
 
-  @GwtIncompatible("reflection")
+  @GwtIncompatible // reflection
   public void testNulls() throws Exception {
     new ClassSanityTester().forAllPublicStaticMethods(Predicates.class).testNulls();
   }
 
-  @GwtIncompatible("reflection")
-  @SuppressUnderAndroid // TODO(cpovirk): ClassNotFoundException: com.google.common.base.Function
+  @GwtIncompatible // reflection
+  @AndroidIncompatible // TODO(cpovirk): ClassNotFoundException: com.google.common.base.Function
   public void testEqualsAndSerializable() throws Exception {
     new ClassSanityTester().forAllPublicStaticMethods(Predicates.class).testEqualsAndSerializable();
   }
@@ -940,17 +925,14 @@ public class PredicatesTest extends TestCase {
   }
 
   private static void assertEvalsLike(
-      Predicate<? super Integer> expected,
-      Predicate<? super Integer> actual) {
+      Predicate<? super Integer> expected, Predicate<? super Integer> actual) {
     assertEvalsLike(expected, actual, 0);
     assertEvalsLike(expected, actual, 1);
     assertEvalsLike(expected, actual, null);
   }
 
   private static <T> void assertEvalsLike(
-      Predicate<? super T> expected,
-      Predicate<? super T> actual,
-      T input) {
+      Predicate<? super T> expected, Predicate<? super T> actual, T input) {
     Boolean expectedResult = null;
     RuntimeException expectedRuntimeException = null;
     try {
@@ -970,16 +952,13 @@ public class PredicatesTest extends TestCase {
     assertEquals(expectedResult, actualResult);
     if (expectedRuntimeException != null) {
       assertNotNull(actualRuntimeException);
-      assertEquals(
-          expectedRuntimeException.getClass(),
-          actualRuntimeException.getClass());
+      assertEquals(expectedRuntimeException.getClass(), actualRuntimeException.getClass());
     }
   }
 
-  @GwtIncompatible("SerializableTester")
+  @GwtIncompatible // SerializableTester
   private static void checkSerialization(Predicate<? super Integer> predicate) {
-    Predicate<? super Integer> reserialized =
-        SerializableTester.reserializeAndAssert(predicate);
+    Predicate<? super Integer> reserialized = SerializableTester.reserializeAndAssert(predicate);
     assertEvalsLike(predicate, reserialized);
   }
 }
