@@ -16,29 +16,26 @@
 
 package com.google.common.base;
 
+import static com.google.common.collect.ImmutableList.toImmutableList;
 import static com.google.common.truth.Truth.assertThat;
 
 import com.google.common.annotations.GwtCompatible;
 import com.google.common.annotations.GwtIncompatible;
+import com.google.common.base.Splitter.MapSplitter;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.testing.NullPointerTester;
-
-import junit.framework.TestCase;
-
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.regex.Pattern;
+import junit.framework.TestCase;
 
-/**
- * @author Julien Silland
- */
+/** @author Julien Silland */
 @GwtCompatible(emulated = true)
 public class SplitterTest extends TestCase {
 
   private static final Splitter COMMA_SPLITTER = Splitter.on(',');
 
-  @SuppressWarnings("CheckReturnValue")
   public void testSplitNullString() {
     try {
       COMMA_SPLITTER.split(null);
@@ -54,12 +51,11 @@ public class SplitterTest extends TestCase {
   }
 
   /**
-   * All of the infrastructure of split and splitToString is identical, so we
-   * do one test of splitToString. All other cases should be covered by testing
-   * of split.
+   * All of the infrastructure of split and splitToString is identical, so we do one test of
+   * splitToString. All other cases should be covered by testing of split.
    *
-   * <p>TODO(user): It would be good to make all the relevant tests run on
-   * both split and splitToString automatically.
+   * <p>TODO(user): It would be good to make all the relevant tests run on both split and
+   * splitToString automatically.
    */
   public void testCharacterSimpleSplitToList() {
     String simple = "a,b,c";
@@ -67,9 +63,15 @@ public class SplitterTest extends TestCase {
     assertThat(letters).containsExactly("a", "b", "c").inOrder();
   }
 
+  public void testCharacterSimpleSplitToStream() {
+    String simple = "a,b,c";
+    List<String> letters = COMMA_SPLITTER.splitToStream(simple).collect(toImmutableList());
+    assertThat(letters).containsExactly("a", "b", "c").inOrder();
+  }
+
   public void testToString() {
-    assertEquals("[]", Splitter.on(',').split("").toString());
-    assertEquals("[a, b, c]", Splitter.on(',').split("a,b,c").toString());
+    assertEquals("[]", COMMA_SPLITTER.split("").toString());
+    assertEquals("[a, b, c]", COMMA_SPLITTER.split("a,b,c").toString());
     assertEquals("[yam, bam, jam, ham]", Splitter.on(", ").split("yam, bam, jam, ham").toString());
   }
 
@@ -103,18 +105,17 @@ public class SplitterTest extends TestCase {
     assertThat(letters).containsExactly("", "a", "b", "c").inOrder();
   }
 
-  public void testCharacterSplitWithMulitpleLetters() {
-    Iterable<String> testCharacteringMotto = Splitter.on('-').split(
-        "Testing-rocks-Debugging-sucks");
+  public void testCharacterSplitWithMultipleLetters() {
+    Iterable<String> testCharacteringMotto =
+        Splitter.on('-').split("Testing-rocks-Debugging-sucks");
     assertThat(testCharacteringMotto)
         .containsExactly("Testing", "rocks", "Debugging", "sucks")
         .inOrder();
   }
 
   public void testCharacterSplitWithMatcherDelimiter() {
-    Iterable<String> testCharacteringMotto = Splitter
-        .on(CharMatcher.WHITESPACE)
-        .split("Testing\nrocks\tDebugging sucks");
+    Iterable<String> testCharacteringMotto =
+        Splitter.on(CharMatcher.whitespace()).split("Testing\nrocks\tDebugging sucks");
     assertThat(testCharacteringMotto)
         .containsExactly("Testing", "rocks", "Debugging", "sucks")
         .inOrder();
@@ -122,22 +123,19 @@ public class SplitterTest extends TestCase {
 
   public void testCharacterSplitWithDoubleDelimiterOmitEmptyStrings() {
     String doubled = "a..b.c";
-    Iterable<String> letters = Splitter.on('.')
-        .omitEmptyStrings().split(doubled);
+    Iterable<String> letters = Splitter.on('.').omitEmptyStrings().split(doubled);
     assertThat(letters).containsExactly("a", "b", "c").inOrder();
   }
 
   public void testCharacterSplitEmptyToken() {
     String emptyToken = "a. .c";
-    Iterable<String> letters = Splitter.on('.').trimResults()
-        .split(emptyToken);
+    Iterable<String> letters = Splitter.on('.').trimResults().split(emptyToken);
     assertThat(letters).containsExactly("a", "", "c").inOrder();
   }
 
   public void testCharacterSplitEmptyTokenOmitEmptyStrings() {
     String emptyToken = "a. .c";
-    Iterable<String> letters = Splitter.on('.')
-        .omitEmptyStrings().trimResults().split(emptyToken);
+    Iterable<String> letters = Splitter.on('.').omitEmptyStrings().trimResults().split(emptyToken);
     assertThat(letters).containsExactly("a", "c").inOrder();
   }
 
@@ -161,11 +159,12 @@ public class SplitterTest extends TestCase {
   }
 
   public void testCharacterSplitWithTrim() {
-    String jacksons = "arfo(Marlon)aorf, (Michael)orfa, afro(Jackie)orfa, "
-        + "ofar(Jemaine), aff(Tito)";
-    Iterable<String> family = COMMA_SPLITTER
-        .trimResults(CharMatcher.anyOf("afro").or(CharMatcher.WHITESPACE))
-        .split(jacksons);
+    String jacksons =
+        "arfo(Marlon)aorf, (Michael)orfa, afro(Jackie)orfa, " + "ofar(Jemaine), aff(Tito)";
+    Iterable<String> family =
+        COMMA_SPLITTER
+            .trimResults(CharMatcher.anyOf("afro").or(CharMatcher.whitespace()))
+            .split(jacksons);
     assertThat(family)
         .containsExactly("(Marlon)", "(Michael)", "(Jackie)", "(Jemaine)", "(Tito)")
         .inOrder();
@@ -173,43 +172,42 @@ public class SplitterTest extends TestCase {
 
   public void testStringSimpleSplit() {
     String simple = "a,b,c";
-    Iterable<String> letters = Splitter.on(',').split(simple);
+    Iterable<String> letters = Splitter.on(",").split(simple);
     assertThat(letters).containsExactly("a", "b", "c").inOrder();
   }
 
   public void testStringSimpleSplitWithNoDelimiter() {
     String simple = "a,b,c";
-    Iterable<String> letters = Splitter.on('.').split(simple);
+    Iterable<String> letters = Splitter.on(".").split(simple);
     assertThat(letters).containsExactly("a,b,c").inOrder();
   }
 
   public void testStringSplitWithDoubleDelimiter() {
     String doubled = "a,,b,c";
-    Iterable<String> letters = Splitter.on(',').split(doubled);
+    Iterable<String> letters = Splitter.on(",").split(doubled);
     assertThat(letters).containsExactly("a", "", "b", "c").inOrder();
   }
 
   public void testStringSplitWithDoubleDelimiterAndSpace() {
     String doubled = "a,, b,c";
-    Iterable<String> letters = Splitter.on(',').split(doubled);
+    Iterable<String> letters = Splitter.on(",").split(doubled);
     assertThat(letters).containsExactly("a", "", " b", "c").inOrder();
   }
 
   public void testStringSplitWithTrailingDelimiter() {
     String trailing = "a,b,c,";
-    Iterable<String> letters = Splitter.on(',').split(trailing);
+    Iterable<String> letters = Splitter.on(",").split(trailing);
     assertThat(letters).containsExactly("a", "b", "c", "").inOrder();
   }
 
   public void testStringSplitWithLeadingDelimiter() {
     String leading = ",a,b,c";
-    Iterable<String> letters = Splitter.on(',').split(leading);
+    Iterable<String> letters = Splitter.on(",").split(leading);
     assertThat(letters).containsExactly("", "a", "b", "c").inOrder();
   }
 
   public void testStringSplitWithMultipleLetters() {
-    Iterable<String> testStringingMotto = Splitter.on('-').split(
-        "Testing-rocks-Debugging-sucks");
+    Iterable<String> testStringingMotto = Splitter.on("-").split("Testing-rocks-Debugging-sucks");
     assertThat(testStringingMotto)
         .containsExactly("Testing", "rocks", "Debugging", "sucks")
         .inOrder();
@@ -217,22 +215,19 @@ public class SplitterTest extends TestCase {
 
   public void testStringSplitWithDoubleDelimiterOmitEmptyStrings() {
     String doubled = "a..b.c";
-    Iterable<String> letters = Splitter.on('.')
-        .omitEmptyStrings().split(doubled);
+    Iterable<String> letters = Splitter.on(".").omitEmptyStrings().split(doubled);
     assertThat(letters).containsExactly("a", "b", "c").inOrder();
   }
 
   public void testStringSplitEmptyToken() {
     String emptyToken = "a. .c";
-    Iterable<String> letters = Splitter.on('.').trimResults()
-        .split(emptyToken);
+    Iterable<String> letters = Splitter.on(".").trimResults().split(emptyToken);
     assertThat(letters).containsExactly("a", "", "c").inOrder();
   }
 
   public void testStringSplitEmptyTokenOmitEmptyStrings() {
     String emptyToken = "a. .c";
-    Iterable<String> letters = Splitter.on('.')
-        .omitEmptyStrings().trimResults().split(emptyToken);
+    Iterable<String> letters = Splitter.on(".").omitEmptyStrings().trimResults().split(emptyToken);
     assertThat(letters).containsExactly("a", "c").inOrder();
   }
 
@@ -256,12 +251,10 @@ public class SplitterTest extends TestCase {
 
   public void testStringSplitWithDelimiterSubstringInValue() {
     String fourCommasAndFourSpaces = ",,,,    ";
-    Iterable<String> threeCommasThenThreeSpaces = Splitter.on(", ").split(
-        fourCommasAndFourSpaces);
+    Iterable<String> threeCommasThenThreeSpaces = Splitter.on(", ").split(fourCommasAndFourSpaces);
     assertThat(threeCommasThenThreeSpaces).containsExactly(",,,", "   ").inOrder();
   }
 
-  @SuppressWarnings("CheckReturnValue")
   public void testStringSplitWithEmptyString() {
     try {
       Splitter.on("");
@@ -271,71 +264,72 @@ public class SplitterTest extends TestCase {
   }
 
   public void testStringSplitOnEmptyString() {
-    Iterable<String> notMuch = Splitter.on('.').split("");
+    Iterable<String> notMuch = Splitter.on(".").split("");
     assertThat(notMuch).containsExactly("").inOrder();
   }
 
   public void testStringSplitOnEmptyStringOmitEmptyString() {
-    assertThat(Splitter.on('.').omitEmptyStrings().split("")).isEmpty();
+    assertThat(Splitter.on(".").omitEmptyStrings().split("")).isEmpty();
   }
 
   public void testStringSplitOnOnlyDelimiter() {
-    Iterable<String> blankblank = Splitter.on('.').split(".");
+    Iterable<String> blankblank = Splitter.on(".").split(".");
     assertThat(blankblank).containsExactly("", "").inOrder();
   }
 
   public void testStringSplitOnOnlyDelimitersOmitEmptyStrings() {
-    Iterable<String> empty = Splitter.on('.').omitEmptyStrings().split("...");
+    Iterable<String> empty = Splitter.on(".").omitEmptyStrings().split("...");
     assertThat(empty).isEmpty();
   }
 
   public void testStringSplitWithTrim() {
-    String jacksons = "arfo(Marlon)aorf, (Michael)orfa, afro(Jackie)orfa, "
-        + "ofar(Jemaine), aff(Tito)";
-    Iterable<String> family = Splitter.on(',')
-        .trimResults(CharMatcher.anyOf("afro").or(CharMatcher.WHITESPACE))
-        .split(jacksons);
+    String jacksons =
+        "arfo(Marlon)aorf, (Michael)orfa, afro(Jackie)orfa, " + "ofar(Jemaine), aff(Tito)";
+    Iterable<String> family =
+        Splitter.on(",")
+            .trimResults(CharMatcher.anyOf("afro").or(CharMatcher.whitespace()))
+            .split(jacksons);
     assertThat(family)
         .containsExactly("(Marlon)", "(Michael)", "(Jackie)", "(Jemaine)", "(Tito)")
         .inOrder();
   }
 
-  @GwtIncompatible("Splitter.onPattern")
+  @GwtIncompatible // Splitter.onPattern
   public void testPatternSimpleSplit() {
     String simple = "a,b,c";
     Iterable<String> letters = Splitter.onPattern(",").split(simple);
     assertThat(letters).containsExactly("a", "b", "c").inOrder();
   }
 
-  @GwtIncompatible("Splitter.onPattern")
+  @GwtIncompatible // Splitter.onPattern
   public void testPatternSimpleSplitWithNoDelimiter() {
     String simple = "a,b,c";
     Iterable<String> letters = Splitter.onPattern("foo").split(simple);
     assertThat(letters).containsExactly("a,b,c").inOrder();
   }
 
-  @GwtIncompatible("Splitter.onPattern")
+  @GwtIncompatible // Splitter.onPattern
   public void testPatternSplitWithDoubleDelimiter() {
     String doubled = "a,,b,c";
     Iterable<String> letters = Splitter.onPattern(",").split(doubled);
     assertThat(letters).containsExactly("a", "", "b", "c").inOrder();
   }
 
-  @GwtIncompatible("Splitter.onPattern")
+  @GwtIncompatible // Splitter.onPattern
   public void testPatternSplitWithDoubleDelimiterAndSpace() {
     String doubled = "a,, b,c";
     Iterable<String> letters = Splitter.onPattern(",").split(doubled);
     assertThat(letters).containsExactly("a", "", " b", "c").inOrder();
   }
 
-  @GwtIncompatible("Splitter.onPattern")
+  @GwtIncompatible // Splitter.onPattern
   public void testPatternSplitWithTrailingDelimiter() {
     String trailing = "a,b,c,";
     Iterable<String> letters = Splitter.onPattern(",").split(trailing);
     assertThat(letters).containsExactly("a", "b", "c", "").inOrder();
   }
 
-  @GwtIncompatible("Splitter.onPattern")
+  @GwtIncompatible // Splitter.onPattern
   public void testPatternSplitWithLeadingDelimiter() {
     String leading = ",a,b,c";
     Iterable<String> letters = Splitter.onPattern(",").split(leading);
@@ -344,30 +338,33 @@ public class SplitterTest extends TestCase {
 
   // TODO(kevinb): the name of this method suggests it might not actually be testing what it
   // intends to be testing?
-  @GwtIncompatible("Splitter.onPattern")
+  @GwtIncompatible // Splitter.onPattern
   public void testPatternSplitWithMultipleLetters() {
-    Iterable<String> testPatterningMotto = Splitter.onPattern("-").split(
-        "Testing-rocks-Debugging-sucks");
-    assertThat(testPatterningMotto).containsExactly("Testing", "rocks", "Debugging", "sucks")
+    Iterable<String> testPatterningMotto =
+        Splitter.onPattern("-").split("Testing-rocks-Debugging-sucks");
+    assertThat(testPatterningMotto)
+        .containsExactly("Testing", "rocks", "Debugging", "sucks")
         .inOrder();
   }
 
-  @GwtIncompatible("java.util.regex.Pattern")
+  @GwtIncompatible // java.util.regex.Pattern
   private static Pattern literalDotPattern() {
     return Pattern.compile("\\.");
   }
 
-  @GwtIncompatible("java.util.regex.Pattern")
+  @GwtIncompatible // java.util.regex.Pattern
   public void testPatternSplitWithDoubleDelimiterOmitEmptyStrings() {
     String doubled = "a..b.c";
-    Iterable<String> letters = Splitter.on(literalDotPattern())
-        .omitEmptyStrings().split(doubled);
+    Iterable<String> letters = Splitter.on(literalDotPattern()).omitEmptyStrings().split(doubled);
     assertThat(letters).containsExactly("a", "b", "c").inOrder();
   }
 
-  @GwtIncompatible("java.util.regex.Pattern")
-  @SuppressUnderAndroid // TODO(cpovirk): File Android bug.
+  @GwtIncompatible // java.util.regex.Pattern
+  @AndroidIncompatible // Bug in older versions of Android we test against, since fixed.
   public void testPatternSplitLookBehind() {
+    if (!CommonPattern.isPcreLike()) {
+      return;
+    }
     String toSplit = ":foo::barbaz:";
     String regexPattern = "(?<=:)";
     Iterable<String> split = Splitter.onPattern(regexPattern).split(toSplit);
@@ -375,69 +372,79 @@ public class SplitterTest extends TestCase {
     // splits into chunks ending in :
   }
 
-  @GwtIncompatible("java.util.regex.Pattern")
-  @SuppressUnderAndroid // TODO(cpovirk): File Android bug.
+  @GwtIncompatible // java.util.regex.Pattern
+  @AndroidIncompatible // Bug in older versions of Android we test against, since fixed.
   public void testPatternSplitWordBoundary() {
     String string = "foo<bar>bletch";
     Iterable<String> words = Splitter.on(Pattern.compile("\\b")).split(string);
     assertThat(words).containsExactly("foo", "<", "bar", ">", "bletch").inOrder();
   }
 
-  @GwtIncompatible("java.util.regex.Pattern")
+  @GwtIncompatible // java.util.regex.Pattern
+  public void testPatternSplitWordBoundary_singleCharInput() {
+    String string = "f";
+    Iterable<String> words = Splitter.on(Pattern.compile("\\b")).split(string);
+    assertThat(words).containsExactly("f").inOrder();
+  }
+
+  @AndroidIncompatible // Apparently Gingerbread's regex API is buggy.
+  @GwtIncompatible // java.util.regex.Pattern
+  public void testPatternSplitWordBoundary_singleWordInput() {
+    String string = "foo";
+    Iterable<String> words = Splitter.on(Pattern.compile("\\b")).split(string);
+    assertThat(words).containsExactly("foo").inOrder();
+  }
+
+  @GwtIncompatible // java.util.regex.Pattern
   public void testPatternSplitEmptyToken() {
     String emptyToken = "a. .c";
     Iterable<String> letters = Splitter.on(literalDotPattern()).trimResults().split(emptyToken);
     assertThat(letters).containsExactly("a", "", "c").inOrder();
   }
 
-  @GwtIncompatible("java.util.regex.Pattern")
+  @GwtIncompatible // java.util.regex.Pattern
   public void testPatternSplitEmptyTokenOmitEmptyStrings() {
     String emptyToken = "a. .c";
-    Iterable<String> letters = Splitter.on(literalDotPattern())
-        .omitEmptyStrings().trimResults().split(emptyToken);
+    Iterable<String> letters =
+        Splitter.on(literalDotPattern()).omitEmptyStrings().trimResults().split(emptyToken);
     assertThat(letters).containsExactly("a", "c").inOrder();
   }
 
-  @GwtIncompatible("java.util.regex.Pattern")
+  @GwtIncompatible // java.util.regex.Pattern
   public void testPatternSplitOnOnlyDelimiter() {
     Iterable<String> blankblank = Splitter.on(literalDotPattern()).split(".");
 
     assertThat(blankblank).containsExactly("", "").inOrder();
   }
 
-  @GwtIncompatible("java.util.regex.Pattern")
+  @GwtIncompatible // java.util.regex.Pattern
   public void testPatternSplitOnOnlyDelimitersOmitEmptyStrings() {
-    Iterable<String> empty = Splitter.on(literalDotPattern()).omitEmptyStrings()
-        .split("...");
+    Iterable<String> empty = Splitter.on(literalDotPattern()).omitEmptyStrings().split("...");
     assertThat(empty).isEmpty();
   }
 
-  @GwtIncompatible("java.util.regex.Pattern")
+  @GwtIncompatible // java.util.regex.Pattern
   public void testPatternSplitMatchingIsGreedy() {
     String longDelimiter = "a, b,   c";
-    Iterable<String> letters = Splitter.on(Pattern.compile(",\\s*"))
-        .split(longDelimiter);
+    Iterable<String> letters = Splitter.on(Pattern.compile(",\\s*")).split(longDelimiter);
     assertThat(letters).containsExactly("a", "b", "c").inOrder();
   }
 
-  @GwtIncompatible("java.util.regex.Pattern")
+  @GwtIncompatible // java.util.regex.Pattern
   public void testPatternSplitWithLongLeadingDelimiter() {
     String longDelimiter = ", a, b, c";
-    Iterable<String> letters = Splitter.on(Pattern.compile(", "))
-        .split(longDelimiter);
+    Iterable<String> letters = Splitter.on(Pattern.compile(", ")).split(longDelimiter);
     assertThat(letters).containsExactly("", "a", "b", "c").inOrder();
   }
 
-  @GwtIncompatible("java.util.regex.Pattern")
+  @GwtIncompatible // java.util.regex.Pattern
   public void testPatternSplitWithLongTrailingDelimiter() {
     String longDelimiter = "a, b, c/ ";
-    Iterable<String> letters = Splitter.on(Pattern.compile("[,/]\\s"))
-        .split(longDelimiter);
+    Iterable<String> letters = Splitter.on(Pattern.compile("[,/]\\s")).split(longDelimiter);
     assertThat(letters).containsExactly("a", "b", "c", "").inOrder();
   }
 
-  @SuppressWarnings("CheckReturnValue")
-  @GwtIncompatible("java.util.regex.Pattern")
+  @GwtIncompatible // java.util.regex.Pattern
   public void testPatternSplitInvalidPattern() {
     try {
       Splitter.on(Pattern.compile("a*"));
@@ -446,13 +453,14 @@ public class SplitterTest extends TestCase {
     }
   }
 
-  @GwtIncompatible("java.util.regex.Pattern")
+  @GwtIncompatible // java.util.regex.Pattern
   public void testPatternSplitWithTrim() {
-    String jacksons = "arfo(Marlon)aorf, (Michael)orfa, afro(Jackie)orfa, "
-        + "ofar(Jemaine), aff(Tito)";
-    Iterable<String> family = Splitter.on(Pattern.compile(","))
-        .trimResults(CharMatcher.anyOf("afro").or(CharMatcher.WHITESPACE))
-        .split(jacksons);
+    String jacksons =
+        "arfo(Marlon)aorf, (Michael)orfa, afro(Jackie)orfa, " + "ofar(Jemaine), aff(Tito)";
+    Iterable<String> family =
+        Splitter.on(Pattern.compile(","))
+            .trimResults(CharMatcher.anyOf("afro").or(CharMatcher.whitespace()))
+            .split(jacksons);
     assertThat(family)
         .containsExactly("(Marlon)", "(Michael)", "(Jackie)", "(Jemaine)", "(Tito)")
         .inOrder();
@@ -463,13 +471,12 @@ public class SplitterTest extends TestCase {
   }
 
   public void testSplitterIterableIsUnmodifiable_string() {
-    assertIteratorIsUnmodifiable(Splitter.on(',').split("a,b").iterator());
+    assertIteratorIsUnmodifiable(Splitter.on(",").split("a,b").iterator());
   }
 
-  @GwtIncompatible("java.util.regex.Pattern")
+  @GwtIncompatible // java.util.regex.Pattern
   public void testSplitterIterableIsUnmodifiable_pattern() {
-    assertIteratorIsUnmodifiable(
-        Splitter.on(Pattern.compile(",")).split("a,b").iterator());
+    assertIteratorIsUnmodifiable(Splitter.on(Pattern.compile(",")).split("a,b").iterator());
   }
 
   private void assertIteratorIsUnmodifiable(Iterator<?> iterator) {
@@ -486,19 +493,21 @@ public class SplitterTest extends TestCase {
   }
 
   public void testSplitterIterableIsLazy_string() {
-    assertSplitterIterableIsLazy(Splitter.on(','));
+    assertSplitterIterableIsLazy(Splitter.on(","));
   }
 
-  @GwtIncompatible("java.util.regex.Pattern")
-  @SuppressUnderAndroid // TODO(cpovirk): File Android bug (different from the two above).
+  @GwtIncompatible // java.util.regex.Pattern
+  @AndroidIncompatible // not clear that j.u.r.Matcher promises to handle mutations during use
   public void testSplitterIterableIsLazy_pattern() {
+    if (!CommonPattern.isPcreLike()) {
+      return;
+    }
     assertSplitterIterableIsLazy(Splitter.onPattern(","));
   }
 
   /**
-   * This test really pushes the boundaries of what we support. In general the
-   * splitter's behaviour is not well defined if the char sequence it's
-   * splitting is mutated during iteration.
+   * This test really pushes the boundaries of what we support. In general the splitter's behaviour
+   * is not well defined if the char sequence it's splitting is mutated during iteration.
    */
   private void assertSplitterIterableIsLazy(Splitter splitter) {
     StringBuilder builder = new StringBuilder();
@@ -553,7 +562,6 @@ public class SplitterTest extends TestCase {
     assertThat(letters).containsExactly("a", "b", "c", "d").inOrder();
   }
 
-  @SuppressWarnings("CheckReturnValue")
   public void testFixedLengthSplitZeroChunkLen() {
     try {
       Splitter.fixedLength(0);
@@ -562,7 +570,6 @@ public class SplitterTest extends TestCase {
     }
   }
 
-  @SuppressWarnings("CheckReturnValue")
   public void testFixedLengthSplitNegativeChunkLen() {
     try {
       Splitter.fixedLength(-1);
@@ -589,6 +596,12 @@ public class SplitterTest extends TestCase {
     assertThat(letters).containsExactly("a", "bcd").inOrder();
   }
 
+  public void testLimit1Separator() {
+    String simple = "a,b,c,d";
+    Iterable<String> items = COMMA_SPLITTER.limit(1).split(simple);
+    assertThat(items).containsExactly("a,b,c,d").inOrder();
+  }
+
   public void testLimitSeparator() {
     String simple = "a,b,c,d";
     Iterable<String> items = COMMA_SPLITTER.limit(2).split(simple);
@@ -649,7 +662,6 @@ public class SplitterTest extends TestCase {
     assertThat(items).isEmpty();
   }
 
-  @SuppressWarnings("ReturnValueIgnored") // testing for exception
   public void testInvalidZeroLimit() {
     try {
       COMMA_SPLITTER.limit(0);
@@ -658,19 +670,20 @@ public class SplitterTest extends TestCase {
     }
   }
 
-  @GwtIncompatible("NullPointerTester")
+  @GwtIncompatible // NullPointerTester
   public void testNullPointers() {
     NullPointerTester tester = new NullPointerTester();
     tester.testAllPublicStaticMethods(Splitter.class);
-    tester.testAllPublicInstanceMethods(Splitter.on(','));
-    tester.testAllPublicInstanceMethods(Splitter.on(',').trimResults());
+    tester.testAllPublicInstanceMethods(COMMA_SPLITTER);
+    tester.testAllPublicInstanceMethods(COMMA_SPLITTER.trimResults());
   }
 
   public void testMapSplitter_trimmedBoth() {
-    Map<String, String> m = COMMA_SPLITTER
-        .trimResults()
-        .withKeyValueSeparator(Splitter.on(':').trimResults())
-        .split("boy  : tom , girl: tina , cat  : kitty , dog: tommy ");
+    Map<String, String> m =
+        COMMA_SPLITTER
+            .trimResults()
+            .withKeyValueSeparator(Splitter.on(':').trimResults())
+            .split("boy  : tom , girl: tina , cat  : kitty , dog: tommy ");
     ImmutableMap<String, String> expected =
         ImmutableMap.of("boy", "tom", "girl", "tina", "cat", "kitty", "dog", "tommy");
     assertThat(m).isEqualTo(expected);
@@ -678,10 +691,11 @@ public class SplitterTest extends TestCase {
   }
 
   public void testMapSplitter_trimmedEntries() {
-    Map<String, String> m = COMMA_SPLITTER
-        .trimResults()
-        .withKeyValueSeparator(":")
-        .split("boy  : tom , girl: tina , cat  : kitty , dog: tommy ");
+    Map<String, String> m =
+        COMMA_SPLITTER
+            .trimResults()
+            .withKeyValueSeparator(":")
+            .split("boy  : tom , girl: tina , cat  : kitty , dog: tommy ");
     ImmutableMap<String, String> expected =
         ImmutableMap.of("boy  ", " tom", "girl", " tina", "cat  ", " kitty", "dog", " tommy");
 
@@ -691,8 +705,9 @@ public class SplitterTest extends TestCase {
 
   public void testMapSplitter_trimmedKeyValue() {
     Map<String, String> m =
-        COMMA_SPLITTER.withKeyValueSeparator(Splitter.on(':').trimResults()).split(
-            "boy  : tom , girl: tina , cat  : kitty , dog: tommy ");
+        COMMA_SPLITTER
+            .withKeyValueSeparator(Splitter.on(':').trimResults())
+            .split("boy  : tom , girl: tina , cat  : kitty , dog: tommy ");
     ImmutableMap<String, String> expected =
         ImmutableMap.of("boy", "tom", "girl", "tina", "cat", "kitty", "dog", "tommy");
     assertThat(m).isEqualTo(expected);
@@ -700,8 +715,10 @@ public class SplitterTest extends TestCase {
   }
 
   public void testMapSplitter_notTrimmed() {
-    Map<String, String> m = COMMA_SPLITTER.withKeyValueSeparator(":").split(
-        " boy:tom , girl: tina , cat :kitty , dog:  tommy ");
+    Map<String, String> m =
+        COMMA_SPLITTER
+            .withKeyValueSeparator(":")
+            .split(" boy:tom , girl: tina , cat :kitty , dog:  tommy ");
     ImmutableMap<String, String> expected =
         ImmutableMap.of(" boy", "tom ", " girl", " tina ", " cat ", "kitty ", " dog", "  tommy ");
     assertThat(m).isEqualTo(expected);
@@ -710,10 +727,8 @@ public class SplitterTest extends TestCase {
 
   public void testMapSplitter_CharacterSeparator() {
     // try different delimiters.
-    Map<String, String> m = Splitter
-        .on(",")
-        .withKeyValueSeparator(':')
-        .split("boy:tom,girl:tina,cat:kitty,dog:tommy");
+    Map<String, String> m =
+        Splitter.on(",").withKeyValueSeparator(':').split("boy:tom,girl:tina,cat:kitty,dog:tommy");
     ImmutableMap<String, String> expected =
         ImmutableMap.of("boy", "tom", "girl", "tina", "cat", "kitty", "dog", "tommy");
 
@@ -723,10 +738,10 @@ public class SplitterTest extends TestCase {
 
   public void testMapSplitter_multiCharacterSeparator() {
     // try different delimiters.
-    Map<String, String> m = Splitter
-        .on(",")
-        .withKeyValueSeparator(":^&")
-        .split("boy:^&tom,girl:^&tina,cat:^&kitty,dog:^&tommy");
+    Map<String, String> m =
+        Splitter.on(",")
+            .withKeyValueSeparator(":^&")
+            .split("boy:^&tom,girl:^&tina,cat:^&kitty,dog:^&tommy");
     ImmutableMap<String, String> expected =
         ImmutableMap.of("boy", "tom", "girl", "tina", "cat", "kitty", "dog", "tommy");
 
@@ -734,7 +749,6 @@ public class SplitterTest extends TestCase {
     assertThat(m.entrySet()).containsExactlyElementsIn(expected.entrySet()).inOrder();
   }
 
-  @SuppressWarnings("CheckReturnValue")
   public void testMapSplitter_emptySeparator() {
     try {
       COMMA_SPLITTER.withKeyValueSeparator("");
@@ -743,7 +757,6 @@ public class SplitterTest extends TestCase {
     }
   }
 
-  @SuppressWarnings("CheckReturnValue")
   public void testMapSplitter_malformedEntry() {
     try {
       COMMA_SPLITTER.withKeyValueSeparator("=").split("a=1,b,c=2");
@@ -752,31 +765,46 @@ public class SplitterTest extends TestCase {
     }
   }
 
-  public void testMapSplitter_orderedResults() {
-    Map<String, String> m = Splitter.on(',')
-        .withKeyValueSeparator(":")
-        .split("boy:tom,girl:tina,cat:kitty,dog:tommy");
-
-    assertThat(m.keySet()).containsExactly("boy", "girl", "cat", "dog").inOrder();
-    assertThat(m).isEqualTo(
-        ImmutableMap.of("boy", "tom", "girl", "tina", "cat", "kitty", "dog", "tommy"));
-
-    // try in a different order
-    m = Splitter.on(',')
-        .withKeyValueSeparator(":")
-        .split("girl:tina,boy:tom,dog:tommy,cat:kitty");
-
-    assertThat(m.keySet()).containsExactly("girl", "boy", "dog", "cat").inOrder();
-    assertThat(m).isEqualTo(
-        ImmutableMap.of("boy", "tom", "girl", "tina", "cat", "kitty", "dog", "tommy"));
-  }
-
-  @SuppressWarnings("CheckReturnValue")
-  public void testMapSplitter_duplicateKeys() {
+  /**
+   * Testing the behavior in https://github.com/google/guava/issues/1900 - this behavior may want to
+   * be changed?
+   */
+  public void testMapSplitter_extraValueDelimiter() {
     try {
-      Splitter.on(',').withKeyValueSeparator(":").split("a:1,b:2,a:3");
+      COMMA_SPLITTER.withKeyValueSeparator("=").split("a=1,c=2=");
       fail();
     } catch (IllegalArgumentException expected) {
     }
   }
+
+  public void testMapSplitter_orderedResults() {
+    Map<String, String> m =
+        COMMA_SPLITTER.withKeyValueSeparator(":").split("boy:tom,girl:tina,cat:kitty,dog:tommy");
+
+    assertThat(m.keySet()).containsExactly("boy", "girl", "cat", "dog").inOrder();
+    assertThat(m)
+        .isEqualTo(ImmutableMap.of("boy", "tom", "girl", "tina", "cat", "kitty", "dog", "tommy"));
+
+    // try in a different order
+    m = COMMA_SPLITTER.withKeyValueSeparator(":").split("girl:tina,boy:tom,dog:tommy,cat:kitty");
+
+    assertThat(m.keySet()).containsExactly("girl", "boy", "dog", "cat").inOrder();
+    assertThat(m)
+        .isEqualTo(ImmutableMap.of("boy", "tom", "girl", "tina", "cat", "kitty", "dog", "tommy"));
+  }
+
+  public void testMapSplitter_duplicateKeys() {
+    try {
+      COMMA_SPLITTER.withKeyValueSeparator(":").split("a:1,b:2,a:3");
+      fail();
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  public void testMapSplitter_varyingTrimLevels() {
+    MapSplitter splitter = COMMA_SPLITTER.trimResults().withKeyValueSeparator(Splitter.on("->"));
+    Map<String, String> split = splitter.split(" x -> y, z-> a ");
+    assertThat(split).containsEntry("x ", " y");
+    assertThat(split).containsEntry("z", " a");
+  }
 }
