@@ -19,7 +19,8 @@ import static com.google.common.collect.Range.range;
 import static com.google.common.truth.Truth.assertThat;
 
 import com.google.common.annotations.GwtIncompatible;
-
+import com.google.common.testing.SerializableTester;
+import java.util.Arrays;
 import java.util.List;
 import java.util.NavigableMap;
 
@@ -29,7 +30,7 @@ import java.util.NavigableMap;
  * @author Louis Wasserman
  * @author Chris Povirk
  */
-@GwtIncompatible("TreeRangeSet")
+@GwtIncompatible // TreeRangeSet
 public class TreeRangeSetTest extends AbstractRangeSetTest {
   // TODO(cpovirk): test all of these with the ranges added in the reverse order
 
@@ -130,7 +131,24 @@ public class TreeRangeSetTest extends AbstractRangeSetTest {
     }
   }
 
+  public void testIntersects(RangeSet<Integer> rangeSet) {
+    for (Range<Integer> query : QUERY_RANGES) {
+      boolean expectIntersect = false;
+      for (Range<Integer> expectedRange : rangeSet.asRanges()) {
+        if (expectedRange.isConnected(query) && !expectedRange.intersection(query).isEmpty()) {
+          expectIntersect = true;
+          break;
+        }
+      }
+      assertEquals(
+          rangeSet + " was incorrect on intersects(" + query + ")",
+          expectIntersect,
+          rangeSet.intersects(query));
+    }
+  }
+
   public void testEnclosing(RangeSet<Integer> rangeSet) {
+    assertTrue(rangeSet.enclosesAll(ImmutableList.<Range<Integer>>of()));
     for (Range<Integer> query : QUERY_RANGES) {
       boolean expectEnclose = false;
       for (Range<Integer> expectedRange : rangeSet.asRanges()) {
@@ -140,8 +158,14 @@ public class TreeRangeSetTest extends AbstractRangeSetTest {
         }
       }
 
-      assertEquals(rangeSet + " was incorrect on encloses(" + query + ")", expectEnclose,
+      assertEquals(
+          rangeSet + " was incorrect on encloses(" + query + ")",
+          expectEnclose,
           rangeSet.encloses(query));
+      assertEquals(
+          rangeSet + " was incorrect on enclosesAll([" + query + "])",
+          expectEnclose,
+          rangeSet.enclosesAll(ImmutableList.of(query)));
     }
   }
 
@@ -165,6 +189,37 @@ public class TreeRangeSetTest extends AbstractRangeSetTest {
     testInvariants(TreeRangeSet.create());
   }
 
+  public void testEmptyIntersecting() {
+    testIntersects(TreeRangeSet.<Integer>create());
+    testIntersects(TreeRangeSet.<Integer>create().complement());
+  }
+
+  public void testAllSingleRangesIntersecting() {
+    for (Range<Integer> range : QUERY_RANGES) {
+      TreeRangeSet<Integer> rangeSet = TreeRangeSet.create();
+      rangeSet.add(range);
+      testIntersects(rangeSet);
+      testIntersects(rangeSet.complement());
+    }
+  }
+
+  public void testAllTwoRangesIntersecting() {
+    for (Range<Integer> range1 : QUERY_RANGES) {
+      for (Range<Integer> range2 : QUERY_RANGES) {
+        TreeRangeSet<Integer> rangeSet = TreeRangeSet.create();
+        rangeSet.add(range1);
+        rangeSet.add(range2);
+        testIntersects(rangeSet);
+        testIntersects(rangeSet.complement());
+      }
+    }
+  }
+
+  public void testEmptyEnclosing() {
+    testEnclosing(TreeRangeSet.<Integer>create());
+    testEnclosing(TreeRangeSet.<Integer>create().complement());
+  }
+
   public void testAllSingleRangesEnclosing() {
     for (Range<Integer> range : QUERY_RANGES) {
       TreeRangeSet<Integer> rangeSet = TreeRangeSet.create();
@@ -216,6 +271,7 @@ public class TreeRangeSetTest extends AbstractRangeSetTest {
     return expected;
   }
 
+
   public void testSubRangeSet() {
     for (Range<Integer> range1 : QUERY_RANGES) {
       for (Range<Integer> range2 : QUERY_RANGES) {
@@ -230,6 +286,19 @@ public class TreeRangeSetTest extends AbstractRangeSetTest {
     }
   }
 
+  public void testSubRangeSetAdd() {
+    TreeRangeSet<Integer> set = TreeRangeSet.create();
+    Range<Integer> range = Range.closedOpen(0, 5);
+    set.subRangeSet(range).add(range);
+  }
+
+  public void testSubRangeSetReplaceAdd() {
+    TreeRangeSet<Integer> set = TreeRangeSet.create();
+    Range<Integer> range = Range.closedOpen(0, 5);
+    set.add(range);
+    set.subRangeSet(range).add(range);
+  }
+
   public void testComplement() {
     for (Range<Integer> range1 : QUERY_RANGES) {
       for (Range<Integer> range2 : QUERY_RANGES) {
@@ -241,6 +310,7 @@ public class TreeRangeSetTest extends AbstractRangeSetTest {
     }
   }
 
+
   public void testSubRangeSetOfComplement() {
     for (Range<Integer> range1 : QUERY_RANGES) {
       for (Range<Integer> range2 : QUERY_RANGES) {
@@ -256,6 +326,7 @@ public class TreeRangeSetTest extends AbstractRangeSetTest {
     }
   }
 
+
   public void testComplementOfSubRangeSet() {
     for (Range<Integer> range1 : QUERY_RANGES) {
       for (Range<Integer> range2 : QUERY_RANGES) {
@@ -282,7 +353,8 @@ public class TreeRangeSetTest extends AbstractRangeSetTest {
         for (Range<Integer> range : rangeSet.asRanges()) {
           expectedRangesByUpperBound.put(range.upperBound, range);
         }
-        testNavigationAgainstExpected(expectedRangesByUpperBound,
+        testNavigationAgainstExpected(
+            expectedRangesByUpperBound,
             new TreeRangeSet.RangesByUpperBound<Integer>(rangeSet.rangesByLowerBound),
             CUTS_TO_TEST);
       }
@@ -435,7 +507,8 @@ public class TreeRangeSetTest extends AbstractRangeSetTest {
                     if ((bLow == bHigh && bLowType == OPEN && bHighType == OPEN) || bLow > bHigh) {
                       continue;
                     }
-                    doPairTest(range(aLow, aLowType, aHigh, aHighType),
+                    doPairTest(
+                        range(aLow, aLowType, aHigh, aHighType),
                         range(bLow, bLowType, bHigh, bHighType));
                   }
                 }
@@ -507,9 +580,11 @@ public class TreeRangeSetTest extends AbstractRangeSetTest {
     rangeSet.remove(Range.closedOpen(3, 4));
     testInvariants(rangeSet);
     assertThat(rangeSet.asRanges())
-        .containsExactly(Range.lessThan(3), Range.closed(4, 6)).inOrder();
+        .containsExactly(Range.lessThan(3), Range.closed(4, 6))
+        .inOrder();
     assertThat(rangeSet.complement().asRanges())
-        .containsExactly(Range.closedOpen(3, 4), Range.greaterThan(6)).inOrder();
+        .containsExactly(Range.closedOpen(3, 4), Range.greaterThan(6))
+        .inOrder();
   }
 
   public void testRemoveNoOverlap() {
@@ -588,4 +663,28 @@ public class TreeRangeSetTest extends AbstractRangeSetTest {
     assertNull(rangeSet.rangeContaining(6));
     assertFalse(rangeSet.contains(6));
   }
+
+  public void testAddAll() {
+    RangeSet<Integer> rangeSet = TreeRangeSet.create();
+    rangeSet.add(Range.closed(3, 10));
+    rangeSet.addAll(Arrays.asList(Range.open(1, 3), Range.closed(5, 8), Range.closed(9, 11)));
+    assertThat(rangeSet.asRanges()).containsExactly(Range.openClosed(1, 11)).inOrder();
+  }
+
+  public void testRemoveAll() {
+    RangeSet<Integer> rangeSet = TreeRangeSet.create();
+    rangeSet.add(Range.closed(3, 10));
+    rangeSet.removeAll(Arrays.asList(Range.open(1, 3), Range.closed(5, 8), Range.closed(9, 11)));
+    assertThat(rangeSet.asRanges())
+        .containsExactly(Range.closedOpen(3, 5), Range.open(8, 9))
+        .inOrder();
+  }
+
+  @GwtIncompatible // SerializableTester
+  public void testSerialization() {
+    RangeSet<Integer> rangeSet = TreeRangeSet.create();
+    rangeSet.add(Range.closed(3, 10));
+    rangeSet.remove(Range.open(5, 7));
+    SerializableTester.reserializeAndAssert(rangeSet);
+  }
 }
