@@ -25,6 +25,8 @@ import static com.google.common.math.MathTesting.NEGATIVE_LONG_CANDIDATES;
 import static com.google.common.math.MathTesting.NONZERO_LONG_CANDIDATES;
 import static com.google.common.math.MathTesting.POSITIVE_INTEGER_CANDIDATES;
 import static com.google.common.math.MathTesting.POSITIVE_LONG_CANDIDATES;
+import static com.google.common.truth.Truth.assertThat;
+import static com.google.common.truth.Truth.assertWithMessage;
 import static java.math.BigInteger.valueOf;
 import static java.math.RoundingMode.FLOOR;
 import static java.math.RoundingMode.UNNECESSARY;
@@ -32,12 +34,12 @@ import static java.math.RoundingMode.UNNECESSARY;
 import com.google.common.annotations.GwtCompatible;
 import com.google.common.annotations.GwtIncompatible;
 import com.google.common.testing.NullPointerTester;
-
-import junit.framework.TestCase;
-
 import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.math.RoundingMode;
+import java.util.EnumSet;
+import java.util.Random;
+import junit.framework.TestCase;
 
 /**
  * Tests for LongMath.
@@ -46,13 +48,79 @@ import java.math.RoundingMode;
  */
 @GwtCompatible(emulated = true)
 public class LongMathTest extends TestCase {
-  @GwtIncompatible("TODO")
-  public void testConstantMaxPowerOfSqrt2Unsigned() {
-    assertEquals(BigIntegerMath.sqrt(BigInteger.ZERO.setBit(2 * Long.SIZE - 1), FLOOR).longValue(),
-        LongMath.MAX_POWER_OF_SQRT2_UNSIGNED);
+  @SuppressWarnings("ConstantOverflow")
+  public void testMaxSignedPowerOfTwo() {
+    assertTrue(LongMath.isPowerOfTwo(LongMath.MAX_SIGNED_POWER_OF_TWO));
+    assertFalse(LongMath.isPowerOfTwo(LongMath.MAX_SIGNED_POWER_OF_TWO * 2));
   }
 
-  @GwtIncompatible("BigIntegerMath") // TODO(cpovirk): GWT-enable BigIntegerMath
+  public void testCeilingPowerOfTwo() {
+    for (long x : POSITIVE_LONG_CANDIDATES) {
+      BigInteger expectedResult = BigIntegerMath.ceilingPowerOfTwo(BigInteger.valueOf(x));
+      if (fitsInLong(expectedResult)) {
+        assertEquals(expectedResult.longValue(), LongMath.ceilingPowerOfTwo(x));
+      } else {
+        try {
+          LongMath.ceilingPowerOfTwo(x);
+          fail("Expected ArithmeticException");
+        } catch (ArithmeticException expected) {
+        }
+      }
+    }
+  }
+
+  public void testFloorPowerOfTwo() {
+    for (long x : POSITIVE_LONG_CANDIDATES) {
+      BigInteger expectedResult = BigIntegerMath.floorPowerOfTwo(BigInteger.valueOf(x));
+      assertEquals(expectedResult.longValue(), LongMath.floorPowerOfTwo(x));
+    }
+  }
+
+  public void testCeilingPowerOfTwoNegative() {
+    for (long x : NEGATIVE_LONG_CANDIDATES) {
+      try {
+        LongMath.ceilingPowerOfTwo(x);
+        fail("Expected IllegalArgumentException");
+      } catch (IllegalArgumentException expected) {
+      }
+    }
+  }
+
+  public void testFloorPowerOfTwoNegative() {
+    for (long x : NEGATIVE_LONG_CANDIDATES) {
+      try {
+        LongMath.floorPowerOfTwo(x);
+        fail("Expected IllegalArgumentException");
+      } catch (IllegalArgumentException expected) {
+      }
+    }
+  }
+
+  public void testCeilingPowerOfTwoZero() {
+    try {
+      LongMath.ceilingPowerOfTwo(0L);
+      fail("Expected IllegalArgumentException");
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  public void testFloorPowerOfTwoZero() {
+    try {
+      LongMath.floorPowerOfTwo(0L);
+      fail("Expected IllegalArgumentException");
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  @GwtIncompatible // TODO
+  public void testConstantMaxPowerOfSqrt2Unsigned() {
+    assertEquals(
+        /*expected=*/ BigIntegerMath.sqrt(BigInteger.ZERO.setBit(2 * Long.SIZE - 1), FLOOR)
+            .longValue(),
+        /*actual=*/ LongMath.MAX_POWER_OF_SQRT2_UNSIGNED);
+  }
+
+  @GwtIncompatible // BigIntegerMath // TODO(cpovirk): GWT-enable BigIntegerMath
   public void testMaxLog10ForLeadingZeros() {
     for (int i = 0; i < Long.SIZE; i++) {
       assertEquals(
@@ -61,7 +129,7 @@ public class LongMathTest extends TestCase {
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testConstantsPowersOf10() {
     for (int i = 0; i < LongMath.powersOf10.length; i++) {
       assertEquals(LongMath.checkedPow(10, i), LongMath.powersOf10[i]);
@@ -69,13 +137,15 @@ public class LongMathTest extends TestCase {
     try {
       LongMath.checkedPow(10, LongMath.powersOf10.length);
       fail("Expected ArithmeticException");
-    } catch (ArithmeticException expected) {}
+    } catch (ArithmeticException expected) {
+    }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testConstantsHalfPowersOf10() {
     for (int i = 0; i < LongMath.halfPowersOf10.length; i++) {
-      assertEquals(BigIntegerMath.sqrt(BigInteger.TEN.pow(2 * i + 1), FLOOR),
+      assertEquals(
+          BigIntegerMath.sqrt(BigInteger.TEN.pow(2 * i + 1), FLOOR),
           BigInteger.valueOf(LongMath.halfPowersOf10[i]));
     }
     BigInteger nextBigger =
@@ -83,12 +153,14 @@ public class LongMathTest extends TestCase {
     assertTrue(nextBigger.compareTo(BigInteger.valueOf(Long.MAX_VALUE)) > 0);
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testConstantsSqrtMaxLong() {
-    assertEquals(LongMath.sqrt(Long.MAX_VALUE, FLOOR), LongMath.FLOOR_SQRT_MAX_LONG);
+    assertEquals(
+        /*expected=*/ LongMath.sqrt(Long.MAX_VALUE, FLOOR),
+        /*actual=*/ LongMath.FLOOR_SQRT_MAX_LONG);
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testConstantsFactorials() {
     long expected = 1;
     for (int i = 0; i < LongMath.factorials.length; i++, expected *= i) {
@@ -98,15 +170,17 @@ public class LongMathTest extends TestCase {
       LongMath.checkedMultiply(
           LongMath.factorials[LongMath.factorials.length - 1], LongMath.factorials.length);
       fail("Expected ArithmeticException");
-    } catch (ArithmeticException expect) {}
+    } catch (ArithmeticException expect) {
+    }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testConstantsBiggestBinomials() {
     for (int k = 0; k < LongMath.biggestBinomials.length; k++) {
       assertTrue(fitsInLong(BigIntegerMath.binomial(LongMath.biggestBinomials[k], k)));
-      assertTrue(LongMath.biggestBinomials[k] == Integer.MAX_VALUE
-          || !fitsInLong(BigIntegerMath.binomial(LongMath.biggestBinomials[k] + 1, k)));
+      assertTrue(
+          LongMath.biggestBinomials[k] == Integer.MAX_VALUE
+              || !fitsInLong(BigIntegerMath.binomial(LongMath.biggestBinomials[k] + 1, k)));
       // In the first case, any long is valid; in the second, we want to test that the next-bigger
       // long overflows.
     }
@@ -115,17 +189,18 @@ public class LongMathTest extends TestCase {
     // 2 * k is the smallest value for which we don't replace k with (n-k).
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testConstantsBiggestSimpleBinomials() {
     for (int k = 0; k < LongMath.biggestSimpleBinomials.length; k++) {
       assertTrue(LongMath.biggestSimpleBinomials[k] <= LongMath.biggestBinomials[k]);
-      simpleBinomial(LongMath.biggestSimpleBinomials[k], k); // mustn't throw
+      long unused = simpleBinomial(LongMath.biggestSimpleBinomials[k], k); // mustn't throw
       if (LongMath.biggestSimpleBinomials[k] < Integer.MAX_VALUE) {
         // unless all n are fair game with this k
         try {
           simpleBinomial(LongMath.biggestSimpleBinomials[k] + 1, k);
           fail("Expected ArithmeticException");
-        } catch (ArithmeticException expected) {}
+        } catch (ArithmeticException expected) {
+        }
       }
     }
     try {
@@ -133,9 +208,11 @@ public class LongMathTest extends TestCase {
       simpleBinomial(2 * k, k);
       // 2 * k is the smallest value for which we don't replace k with (n-k).
       fail("Expected ArithmeticException");
-    } catch (ArithmeticException expected) {}
+    } catch (ArithmeticException expected) {
+    }
   }
 
+  @AndroidIncompatible // slow
   public void testLessThanBranchFree() {
     for (long x : ALL_LONG_CANDIDATES) {
       for (long y : ALL_LONG_CANDIDATES) {
@@ -150,7 +227,7 @@ public class LongMathTest extends TestCase {
   }
 
   // Throws an ArithmeticException if "the simple implementation" of binomial coefficients overflows
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   private long simpleBinomial(int n, int k) {
     long accum = 1;
     for (int i = 0; i < k; i++) {
@@ -160,7 +237,7 @@ public class LongMathTest extends TestCase {
     return accum;
   }
 
-  @GwtIncompatible("java.math.BigInteger")
+  @GwtIncompatible // java.math.BigInteger
   public void testIsPowerOfTwo() {
     for (long x : ALL_LONG_CANDIDATES) {
       // Checks for a single bit set.
@@ -175,7 +252,8 @@ public class LongMathTest extends TestCase {
       try {
         LongMath.log2(0L, mode);
         fail("Expected IllegalArgumentException");
-      } catch (IllegalArgumentException expected) {}
+      } catch (IllegalArgumentException expected) {
+      }
     }
   }
 
@@ -185,7 +263,8 @@ public class LongMathTest extends TestCase {
         try {
           LongMath.log2(x, mode);
           fail("Expected IllegalArgumentException");
-        } catch (IllegalArgumentException expected) {}
+        } catch (IllegalArgumentException expected) {
+        }
       }
     }
   }
@@ -214,30 +293,32 @@ public class LongMathTest extends TestCase {
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testLog10ZeroAlwaysThrows() {
     for (RoundingMode mode : ALL_ROUNDING_MODES) {
       try {
         LongMath.log10(0L, mode);
         fail("Expected IllegalArgumentException");
-      } catch (IllegalArgumentException expected) {}
+      } catch (IllegalArgumentException expected) {
+      }
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testLog10NegativeAlwaysThrows() {
     for (long x : NEGATIVE_LONG_CANDIDATES) {
       for (RoundingMode mode : ALL_ROUNDING_MODES) {
         try {
           LongMath.log10(x, mode);
           fail("Expected IllegalArgumentException");
-        } catch (IllegalArgumentException expected) {}
+        } catch (IllegalArgumentException expected) {
+        }
       }
     }
   }
 
   // Relies on the correctness of BigIntegerMath.log10 for all modes except UNNECESSARY.
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testLog10MatchesBigInteger() {
     for (long x : POSITIVE_LONG_CANDIDATES) {
       for (RoundingMode mode : ALL_SAFE_ROUNDING_MODES) {
@@ -247,7 +328,7 @@ public class LongMathTest extends TestCase {
   }
 
   // Relies on the correctness of log10(long, FLOOR) and of pow(long, int).
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testLog10Exact() {
     for (long x : POSITIVE_LONG_CANDIDATES) {
       int floor = LongMath.log10(x, FLOOR);
@@ -263,7 +344,7 @@ public class LongMathTest extends TestCase {
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testLog10TrivialOnPowerOf10() {
     long x = 1000000000000L;
     for (RoundingMode mode : ALL_ROUNDING_MODES) {
@@ -271,20 +352,21 @@ public class LongMathTest extends TestCase {
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testSqrtNegativeAlwaysThrows() {
     for (long x : NEGATIVE_LONG_CANDIDATES) {
       for (RoundingMode mode : ALL_ROUNDING_MODES) {
         try {
           LongMath.sqrt(x, mode);
           fail("Expected IllegalArgumentException");
-        } catch (IllegalArgumentException expected) {}
+        } catch (IllegalArgumentException expected) {
+        }
       }
     }
   }
 
   // Relies on the correctness of BigIntegerMath.sqrt for all modes except UNNECESSARY.
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testSqrtMatchesBigInteger() {
     for (long x : POSITIVE_LONG_CANDIDATES) {
       for (RoundingMode mode : ALL_SAFE_ROUNDING_MODES) {
@@ -296,7 +378,7 @@ public class LongMathTest extends TestCase {
   }
 
   /* Relies on the correctness of sqrt(long, FLOOR). */
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testSqrtExactMatchesFloorOrThrows() {
     for (long x : POSITIVE_LONG_CANDIDATES) {
       long sqrtFloor = LongMath.sqrt(x, FLOOR);
@@ -311,19 +393,17 @@ public class LongMathTest extends TestCase {
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testPow() {
     for (long i : ALL_LONG_CANDIDATES) {
       for (int exp : EXPONENTS) {
-        assertEquals(LongMath.pow(i, exp), valueOf(i)
-            .pow(exp)
-            .longValue());
+        assertEquals(LongMath.pow(i, exp), valueOf(i).pow(exp).longValue());
       }
     }
   }
 
-  @GwtIncompatible("TODO")
-  @SuppressUnderAndroid // TODO(cpovirk): File BigDecimal.divide() rounding bug.
+  @GwtIncompatible // TODO
+  @AndroidIncompatible // TODO(cpovirk): File BigDecimal.divide() rounding bug.
   public void testDivNonZero() {
     for (long p : NONZERO_LONG_CANDIDATES) {
       for (long q : NONZERO_LONG_CANDIDATES) {
@@ -339,8 +419,8 @@ public class LongMathTest extends TestCase {
     }
   }
 
-  @GwtIncompatible("TODO")
-  @SuppressUnderAndroid // TODO(cpovirk): Problem with LongMath.divide on Android?
+  @GwtIncompatible // TODO
+  @AndroidIncompatible // Bug in older versions of Android we test against, since fixed.
   public void testDivNonZeroExact() {
     for (long p : NONZERO_LONG_CANDIDATES) {
       for (long q : NONZERO_LONG_CANDIDATES) {
@@ -359,7 +439,7 @@ public class LongMathTest extends TestCase {
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testZeroDivIsAlwaysZero() {
     for (long q : NONZERO_LONG_CANDIDATES) {
       for (RoundingMode mode : ALL_ROUNDING_MODES) {
@@ -368,70 +448,71 @@ public class LongMathTest extends TestCase {
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testDivByZeroAlwaysFails() {
     for (long p : ALL_LONG_CANDIDATES) {
       for (RoundingMode mode : ALL_ROUNDING_MODES) {
         try {
           LongMath.divide(p, 0L, mode);
           fail("Expected ArithmeticException");
-        } catch (ArithmeticException expected) {}
+        } catch (ArithmeticException expected) {
+        }
       }
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testIntMod() {
     for (long x : ALL_LONG_CANDIDATES) {
       for (int m : POSITIVE_INTEGER_CANDIDATES) {
-        assertEquals(valueOf(x)
-            .mod(valueOf(m))
-            .intValue(), LongMath.mod(x, m));
+        assertEquals(valueOf(x).mod(valueOf(m)).intValue(), LongMath.mod(x, m));
       }
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testIntModNegativeModulusFails() {
     for (long x : ALL_LONG_CANDIDATES) {
       for (int m : NEGATIVE_INTEGER_CANDIDATES) {
         try {
           LongMath.mod(x, m);
           fail("Expected ArithmeticException");
-        } catch (ArithmeticException expected) {}
+        } catch (ArithmeticException expected) {
+        }
       }
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testIntModZeroModulusFails() {
     for (long x : ALL_LONG_CANDIDATES) {
       try {
         LongMath.mod(x, 0);
         fail("Expected AE");
-      } catch (ArithmeticException expected) {}
-    }
-  }
-
-  @GwtIncompatible("TODO")
-  public void testMod() {
-    for (long x : ALL_LONG_CANDIDATES) {
-      for (long m : POSITIVE_LONG_CANDIDATES) {
-        assertEquals(valueOf(x)
-            .mod(valueOf(m))
-            .longValue(), LongMath.mod(x, m));
+      } catch (ArithmeticException expected) {
       }
     }
   }
 
-  @GwtIncompatible("TODO")
+  @AndroidIncompatible // slow
+  @GwtIncompatible // TODO
+  public void testMod() {
+    for (long x : ALL_LONG_CANDIDATES) {
+      for (long m : POSITIVE_LONG_CANDIDATES) {
+        assertEquals(valueOf(x).mod(valueOf(m)).longValue(), LongMath.mod(x, m));
+      }
+    }
+  }
+
+  @GwtIncompatible // TODO
   public void testModNegativeModulusFails() {
     for (long x : ALL_LONG_CANDIDATES) {
       for (long m : NEGATIVE_LONG_CANDIDATES) {
         try {
           LongMath.mod(x, m);
           fail("Expected ArithmeticException");
-        } catch (ArithmeticException expected) {}
+        } catch (ArithmeticException expected) {
+        }
       }
     }
   }
@@ -444,7 +525,7 @@ public class LongMathTest extends TestCase {
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testGCDZero() {
     for (long a : POSITIVE_LONG_CANDIDATES) {
       assertEquals(a, LongMath.gcd(a, 0));
@@ -453,35 +534,40 @@ public class LongMathTest extends TestCase {
     assertEquals(0, LongMath.gcd(0, 0));
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testGCDNegativePositiveThrows() {
     for (long a : NEGATIVE_LONG_CANDIDATES) {
       try {
         LongMath.gcd(a, 3);
         fail("Expected IllegalArgumentException");
-      } catch (IllegalArgumentException expected) {}
+      } catch (IllegalArgumentException expected) {
+      }
       try {
         LongMath.gcd(3, a);
         fail("Expected IllegalArgumentException");
-      } catch (IllegalArgumentException expected) {}
+      } catch (IllegalArgumentException expected) {
+      }
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testGCDNegativeZeroThrows() {
     for (long a : NEGATIVE_LONG_CANDIDATES) {
       try {
         LongMath.gcd(a, 0);
         fail("Expected IllegalArgumentException");
-      } catch (IllegalArgumentException expected) {}
+      } catch (IllegalArgumentException expected) {
+      }
       try {
         LongMath.gcd(0, a);
         fail("Expected IllegalArgumentException");
-      } catch (IllegalArgumentException expected) {}
+      } catch (IllegalArgumentException expected) {
+      }
     }
   }
 
-  @GwtIncompatible("TODO")
+  @AndroidIncompatible // slow
+  @GwtIncompatible // TODO
   public void testCheckedAdd() {
     for (long a : ALL_LONG_CANDIDATES) {
       for (long b : ALL_LONG_CANDIDATES) {
@@ -500,7 +586,8 @@ public class LongMathTest extends TestCase {
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
+  @AndroidIncompatible // slow
   public void testCheckedSubtract() {
     for (long a : ALL_LONG_CANDIDATES) {
       for (long b : ALL_LONG_CANDIDATES) {
@@ -513,20 +600,28 @@ public class LongMathTest extends TestCase {
           if (expectedSuccess) {
             failFormat(
                 "expected checkedSubtract(%s, %s) = %s; got ArithmeticException",
-                a,
-                b,
-                expectedResult);
+                a, b, expectedResult);
           }
         }
       }
     }
   }
 
-  @GwtIncompatible("TODO")
-  @SuppressUnderAndroid // TODO(cpovirk): Problem with LongMath.checkedMultiply on Android?
+  @AndroidIncompatible // slow
   public void testCheckedMultiply() {
+    boolean isAndroid = TestPlatform.isAndroid();
     for (long a : ALL_LONG_CANDIDATES) {
       for (long b : ALL_LONG_CANDIDATES) {
+        if (isAndroid && a == -4294967296L && b == 2147483648L) {
+          /*
+           * Bug in older versions of Android we test against, since fixed: -9223372036854775808L /
+           * -4294967296L = -9223372036854775808L!
+           *
+           * To be clear, this bug affects not the test's computation of the expected result but the
+           * _actual prod code_. But it probably affects only unusual cases.
+           */
+          continue;
+        }
         BigInteger expectedResult = valueOf(a).multiply(valueOf(b));
         boolean expectedSuccess = fitsInLong(expectedResult);
         try {
@@ -536,16 +631,14 @@ public class LongMathTest extends TestCase {
           if (expectedSuccess) {
             failFormat(
                 "expected checkedMultiply(%s, %s) = %s; got ArithmeticException",
-                a,
-                b,
-                expectedResult);
+                a, b, expectedResult);
           }
         }
       }
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testCheckedPow() {
     for (long b : ALL_LONG_CANDIDATES) {
       for (int exp : EXPONENTS) {
@@ -558,17 +651,72 @@ public class LongMathTest extends TestCase {
           if (expectedSuccess) {
             failFormat(
                 "expected checkedPow(%s, %s) = %s; got ArithmeticException",
-                b,
-                exp,
-                expectedResult);
+                b, exp, expectedResult);
           }
         }
       }
     }
   }
 
+  @AndroidIncompatible // slow
+  @GwtIncompatible // TODO
+  public void testSaturatedAdd() {
+    for (long a : ALL_LONG_CANDIDATES) {
+      for (long b : ALL_LONG_CANDIDATES) {
+        assertOperationEquals(
+            a, b, "s+", saturatedCast(valueOf(a).add(valueOf(b))), LongMath.saturatedAdd(a, b));
+      }
+    }
+  }
+
+  @AndroidIncompatible // slow
+  @GwtIncompatible // TODO
+  public void testSaturatedSubtract() {
+    for (long a : ALL_LONG_CANDIDATES) {
+      for (long b : ALL_LONG_CANDIDATES) {
+        assertOperationEquals(
+            a,
+            b,
+            "s-",
+            saturatedCast(valueOf(a).subtract(valueOf(b))),
+            LongMath.saturatedSubtract(a, b));
+      }
+    }
+  }
+
+  @AndroidIncompatible // slow
+  @GwtIncompatible // TODO
+  public void testSaturatedMultiply() {
+    for (long a : ALL_LONG_CANDIDATES) {
+      for (long b : ALL_LONG_CANDIDATES) {
+        assertOperationEquals(
+            a,
+            b,
+            "s*",
+            saturatedCast(valueOf(a).multiply(valueOf(b))),
+            LongMath.saturatedMultiply(a, b));
+      }
+    }
+  }
+
+  @GwtIncompatible // TODO
+  public void testSaturatedPow() {
+    for (long a : ALL_LONG_CANDIDATES) {
+      for (int b : EXPONENTS) {
+        assertOperationEquals(
+            a, b, "s^", saturatedCast(valueOf(a).pow(b)), LongMath.saturatedPow(a, b));
+      }
+    }
+  }
+
+  private void assertOperationEquals(long a, long b, String op, long expected, long actual) {
+    if (expected != actual) {
+      fail("Expected for " + a + " " + op + " " + b + " = " + expected + ", but got " + actual);
+    }
+  }
+
   // Depends on the correctness of BigIntegerMath.factorial.
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testFactorial() {
     for (int n = 0; n <= 50; n++) {
       BigInteger expectedBig = BigIntegerMath.factorial(n);
@@ -577,13 +725,14 @@ public class LongMathTest extends TestCase {
     }
   }
 
-  @GwtIncompatible("TODO")
+  @GwtIncompatible // TODO
   public void testFactorialNegative() {
     for (int n : NEGATIVE_INTEGER_CANDIDATES) {
       try {
         LongMath.factorial(n);
         fail("Expected IllegalArgumentException");
-      } catch (IllegalArgumentException expected) {}
+      } catch (IllegalArgumentException expected) {
+      }
     }
   }
 
@@ -598,7 +747,8 @@ public class LongMathTest extends TestCase {
     }
   }
 
-  @GwtIncompatible("Slow")
+
+  @GwtIncompatible // Slow
   public void testBinomial_exhaustiveNotOverflowing() {
     // Tests all of the inputs to LongMath.binomial that won't cause it to overflow, that weren't
     // tested in the previous method, for k >= 3.
@@ -614,11 +764,13 @@ public class LongMathTest extends TestCase {
       try {
         LongMath.binomial(n, -1);
         fail("Expected IllegalArgumentException");
-      } catch (IllegalArgumentException expected) {}
+      } catch (IllegalArgumentException expected) {
+      }
       try {
         LongMath.binomial(n, n + 1);
         fail("Expected IllegalArgumentException");
-      } catch (IllegalArgumentException expected) {}
+      } catch (IllegalArgumentException expected) {
+      }
     }
   }
 
@@ -627,11 +779,13 @@ public class LongMathTest extends TestCase {
       try {
         LongMath.binomial(n, 0);
         fail("Expected IllegalArgumentException");
-      } catch (IllegalArgumentException expected) {}
+      } catch (IllegalArgumentException expected) {
+      }
     }
   }
 
-  @GwtIncompatible("far too slow")
+
+  @GwtIncompatible // far too slow
   public void testSqrtOfPerfectSquareAsDoubleIsPerfect() {
     // This takes just over a minute on my machine.
     for (long n = 0; n <= LongMath.FLOOR_SQRT_MAX_LONG; n++) {
@@ -645,7 +799,8 @@ public class LongMathTest extends TestCase {
     assertTrue(sqrtMaxLong <= LongMath.FLOOR_SQRT_MAX_LONG);
   }
 
-  @GwtIncompatible("java.math.BigInteger")
+  @AndroidIncompatible // slow
+  @GwtIncompatible // java.math.BigInteger
   public void testMean() {
     // Odd-sized ranges have an obvious mean
     assertMean(2, 1, 3);
@@ -679,36 +834,35 @@ public class LongMathTest extends TestCase {
     }
   }
 
-  /**
-   * Helper method that asserts the arithmetic mean of x and y is equal
-   * to the expectedMean.
-   */
+  /** Helper method that asserts the arithmetic mean of x and y is equal to the expectedMean. */
   private static void assertMean(long expectedMean, long x, long y) {
-    assertEquals("The expectedMean should be the same as computeMeanSafely",
-        expectedMean, computeMeanSafely(x, y));
+    assertEquals(
+        "The expectedMean should be the same as computeMeanSafely",
+        expectedMean,
+        computeMeanSafely(x, y));
     assertMean(x, y);
   }
 
   /**
-   * Helper method that asserts the arithmetic mean of x and y is equal
-   *to the result of computeMeanSafely.
+   * Helper method that asserts the arithmetic mean of x and y is equal to the result of
+   * computeMeanSafely.
    */
   private static void assertMean(long x, long y) {
     long expectedMean = computeMeanSafely(x, y);
     assertEquals(expectedMean, LongMath.mean(x, y));
-    assertEquals("The mean of x and y should equal the mean of y and x",
-        expectedMean, LongMath.mean(y, x));
+    assertEquals(
+        "The mean of x and y should equal the mean of y and x", expectedMean, LongMath.mean(y, x));
   }
 
   /**
-   * Computes the mean in a way that is obvious and resilient to
-   * overflow by using BigInteger arithmetic.
+   * Computes the mean in a way that is obvious and resilient to overflow by using BigInteger
+   * arithmetic.
    */
   private static long computeMeanSafely(long x, long y) {
     BigInteger bigX = BigInteger.valueOf(x);
     BigInteger bigY = BigInteger.valueOf(y);
-    BigDecimal bigMean = new BigDecimal(bigX.add(bigY))
-        .divide(BigDecimal.valueOf(2), BigDecimal.ROUND_FLOOR);
+    BigDecimal bigMean =
+        new BigDecimal(bigX.add(bigY)).divide(BigDecimal.valueOf(2), BigDecimal.ROUND_FLOOR);
     // parseInt blows up on overflow as opposed to intValue() which does not.
     return Long.parseLong(bigMean.toString());
   }
@@ -717,7 +871,20 @@ public class LongMathTest extends TestCase {
     return big.bitLength() <= 63;
   }
 
-  @GwtIncompatible("NullPointerTester")
+  private static final BigInteger MAX_LONG = BigInteger.valueOf(Long.MAX_VALUE);
+  private static final BigInteger MIN_LONG = BigInteger.valueOf(Long.MIN_VALUE);
+
+  private static long saturatedCast(BigInteger big) {
+    if (big.compareTo(MAX_LONG) > 0) {
+      return Long.MAX_VALUE;
+    }
+    if (big.compareTo(MIN_LONG) < 0) {
+      return Long.MIN_VALUE;
+    }
+    return big.longValue();
+  }
+
+  @GwtIncompatible // NullPointerTester
   public void testNullPointers() {
     NullPointerTester tester = new NullPointerTester();
     tester.setDefault(int.class, 1);
@@ -725,8 +892,137 @@ public class LongMathTest extends TestCase {
     tester.testAllPublicStaticMethods(LongMath.class);
   }
 
-  @GwtIncompatible("String.format")
+  @GwtIncompatible // isPrime is GWT-incompatible
+  public void testIsPrimeSmall() {
+    // Check the first 1000 integers
+    for (int i = 2; i < 1000; i++) {
+      assertEquals(BigInteger.valueOf(i).isProbablePrime(100), LongMath.isPrime(i));
+    }
+  }
+
+  @GwtIncompatible // isPrime is GWT-incompatible
+  public void testIsPrimeManyConstants() {
+    // Test the thorough test inputs, which also includes special constants in the Miller-Rabin
+    // tests.
+    for (long l : POSITIVE_LONG_CANDIDATES) {
+      assertEquals(BigInteger.valueOf(l).isProbablePrime(100), LongMath.isPrime(l));
+    }
+  }
+
+  @GwtIncompatible // isPrime is GWT-incompatible
+  public void testIsPrimeOnUniformRandom() {
+    Random rand = new Random(1);
+    for (int bits = 10; bits < 63; bits++) {
+      for (int i = 0; i < 2000; i++) {
+        // A random long between 0 and Long.MAX_VALUE, inclusive.
+        long l = rand.nextLong() & ((1L << bits) - 1);
+        assertEquals(BigInteger.valueOf(l).isProbablePrime(100), LongMath.isPrime(l));
+      }
+    }
+  }
+
+  @GwtIncompatible // isPrime is GWT-incompatible
+  public void testIsPrimeOnRandomPrimes() {
+    Random rand = new Random(1);
+    for (int bits = 10; bits < 63; bits++) {
+      for (int i = 0; i < 100; i++) {
+        long p = BigInteger.probablePrime(bits, rand).longValue();
+        assertTrue(LongMath.isPrime(p));
+      }
+    }
+  }
+
+  @GwtIncompatible // isPrime is GWT-incompatible
+  public void testIsPrimeOnRandomComposites() {
+    Random rand = new Random(1);
+    for (int bits = 5; bits < 32; bits++) {
+      for (int i = 0; i < 100; i++) {
+        long p = BigInteger.probablePrime(bits, rand).longValue();
+        long q = BigInteger.probablePrime(bits, rand).longValue();
+        assertFalse(LongMath.isPrime(p * q));
+      }
+    }
+  }
+
+  @GwtIncompatible // isPrime is GWT-incompatible
+  public void testIsPrimeThrowsOnNegative() {
+    try {
+      LongMath.isPrime(-1);
+      fail("Expected IllegalArgumentException");
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  private static final long[] roundToDoubleTestCandidates = {
+    0,
+    16,
+    1L << 53,
+    (1L << 53) + 1,
+    (1L << 53) + 2,
+    (1L << 53) + 3,
+    (1L << 53) + 4,
+    1L << 54,
+    (1L << 54) + 1,
+    (1L << 54) + 2,
+    (1L << 54) + 3,
+    (1L << 54) + 4,
+    0x7ffffffffffffe00L, // halfway between 2^63 and next-lower double
+    0x7ffffffffffffe01L, // above + 1
+    0x7ffffffffffffdffL, // above - 1
+    Long.MAX_VALUE - (1L << 11) + 1,
+    Long.MAX_VALUE - 2,
+    Long.MAX_VALUE - 1,
+    Long.MAX_VALUE,
+    -16,
+    -1L << 53,
+    -(1L << 53) - 1,
+    -(1L << 53) - 2,
+    -(1L << 53) - 3,
+    -(1L << 53) - 4,
+    -1L << 54,
+    -(1L << 54) - 1,
+    -(1L << 54) - 2,
+    -(1L << 54) - 3,
+    -(1L << 54) - 4,
+    Long.MIN_VALUE + 2,
+    Long.MIN_VALUE + 1,
+    Long.MIN_VALUE
+  };
+
+  @GwtIncompatible
+  public void testRoundToDoubleAgainstBigInteger() {
+    for (RoundingMode roundingMode : EnumSet.complementOf(EnumSet.of(UNNECESSARY))) {
+      for (long candidate : roundToDoubleTestCandidates) {
+        assertThat(LongMath.roundToDouble(candidate, roundingMode))
+            .isEqualTo(BigIntegerMath.roundToDouble(BigInteger.valueOf(candidate), roundingMode));
+      }
+    }
+  }
+
+  @GwtIncompatible
+  public void testRoundToDoubleAgainstBigIntegerUnnecessary() {
+    for (long candidate : roundToDoubleTestCandidates) {
+      Double expectedDouble = null;
+      try {
+        expectedDouble = BigIntegerMath.roundToDouble(BigInteger.valueOf(candidate), UNNECESSARY);
+      } catch (ArithmeticException expected) {
+        // do nothing
+      }
+
+      if (expectedDouble != null) {
+        assertThat(LongMath.roundToDouble(candidate, UNNECESSARY)).isEqualTo(expectedDouble);
+      } else {
+        try {
+          LongMath.roundToDouble(candidate, UNNECESSARY);
+          fail("Expected ArithmeticException on roundToDouble(" + candidate + ", UNNECESSARY)");
+        } catch (ArithmeticException expected) {
+          // success
+        }
+      }
+    }
+  }
+
   private static void failFormat(String template, Object... args) {
-    fail(String.format(template, args));
+    assertWithMessage(template, args).fail();
   }
 }
