@@ -38,19 +38,17 @@ import com.google.common.annotations.GwtIncompatible;
 import com.google.common.base.Function;
 import com.google.common.base.Optional;
 import com.google.common.base.Predicate;
-import com.google.common.base.Throwables;
 import com.google.common.collect.FluentIterable;
 import com.google.common.collect.ImmutableListMultimap;
 import com.google.common.collect.ImmutableMultimap;
+import com.google.common.collect.ImmutableSet;
 import com.google.common.testing.EqualsTester;
 import com.google.common.testing.NullPointerTester;
-
-import junit.framework.TestCase;
-
 import java.lang.reflect.Field;
 import java.nio.charset.Charset;
-import java.nio.charset.IllegalCharsetNameException;
 import java.nio.charset.UnsupportedCharsetException;
+import java.util.Arrays;
+import junit.framework.TestCase;
 
 /**
  * Tests for {@link MediaType}.
@@ -59,20 +57,25 @@ import java.nio.charset.UnsupportedCharsetException;
  */
 @GwtCompatible(emulated = true)
 public class MediaTypeTest extends TestCase {
-  @GwtIncompatible("reflection") public void testParse_useConstants() throws Exception {
+  @GwtIncompatible // reflection
+  public void testParse_useConstants() throws Exception {
     for (MediaType constant : getConstants()) {
       assertSame(constant, MediaType.parse(constant.toString()));
     }
   }
 
-  @GwtIncompatible("reflection") public void testCreate_useConstants() throws Exception {
+  @GwtIncompatible // reflection
+  public void testCreate_useConstants() throws Exception {
     for (MediaType constant : getConstants()) {
-      assertSame(constant, MediaType.create(constant.type(), constant.subtype())
-          .withParameters(constant.parameters()));
+      assertSame(
+          constant,
+          MediaType.create(constant.type(), constant.subtype())
+              .withParameters(constant.parameters()));
     }
   }
 
-  @GwtIncompatible("reflection") public void testConstants_charset() throws Exception {
+  @GwtIncompatible // reflection
+  public void testConstants_charset() throws Exception {
     for (Field field : getConstantFields()) {
       Optional<Charset> charset = ((MediaType) field.get(null)).charset();
       if (field.getName().endsWith("_UTF_8")) {
@@ -83,49 +86,97 @@ public class MediaTypeTest extends TestCase {
     }
   }
 
-  @GwtIncompatible("reflection") private static FluentIterable<Field> getConstantFields() {
-    return FluentIterable.from(asList(MediaType.class.getDeclaredFields()))
-        .filter(new Predicate<Field>() {
-          @Override public boolean apply(Field input) {
-            int modifiers = input.getModifiers();
-            return isPublic(modifiers) && isStatic(modifiers) && isFinal(modifiers)
-                && MediaType.class.equals(input.getType());
-          }
-        });
+  @GwtIncompatible // reflection
+  public void testConstants_areUnique() {
+    assertThat(getConstants()).containsNoDuplicates();
   }
 
-  @GwtIncompatible("reflection") private static FluentIterable<MediaType> getConstants() {
+  @GwtIncompatible // reflection
+  private static FluentIterable<Field> getConstantFields() {
+    return FluentIterable.from(asList(MediaType.class.getDeclaredFields()))
+        .filter(
+            new Predicate<Field>() {
+              @Override
+              public boolean apply(Field input) {
+                int modifiers = input.getModifiers();
+                return isPublic(modifiers)
+                    && isStatic(modifiers)
+                    && isFinal(modifiers)
+                    && MediaType.class.equals(input.getType());
+              }
+            });
+  }
+
+  @GwtIncompatible // reflection
+  private static FluentIterable<MediaType> getConstants() {
     return getConstantFields()
-        .transform(new Function<Field, MediaType>() {
-          @Override public MediaType apply(Field input) {
-            try {
-              return (MediaType) input.get(null);
-            } catch (Exception e) {
-              throw Throwables.propagate(e);
-            }
-          }
-        });
+        .transform(
+            new Function<Field, MediaType>() {
+              @Override
+              public MediaType apply(Field input) {
+                try {
+                  return (MediaType) input.get(null);
+                } catch (Exception e) {
+                  throw new RuntimeException(e);
+                }
+              }
+            });
   }
 
   public void testCreate_invalidType() {
     try {
       MediaType.create("te><t", "plaintext");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
   }
 
   public void testCreate_invalidSubtype() {
     try {
       MediaType.create("text", "pl@intext");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
   }
 
   public void testCreate_wildcardTypeDeclaredSubtype() {
     try {
       MediaType.create("*", "text");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  public void testCreate_nonAsciiType() {
+    try {
+      MediaType.create("…", "a");
+      fail();
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  public void testCreate_nonAsciiSubtype() {
+    try {
+      MediaType.create("a", "…");
+      fail();
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  public void testCreate_emptyType() {
+    try {
+      MediaType.create("", "a");
+      fail();
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  public void testCreate_emptySubtype() {
+    try {
+      MediaType.create("a", "");
+      fail();
+    } catch (IllegalArgumentException expected) {
+    }
   }
 
   public void testCreateApplicationType() {
@@ -140,6 +191,12 @@ public class MediaTypeTest extends TestCase {
     assertEquals("yams", newType.subtype());
   }
 
+  public void testCreateFontType() {
+    MediaType newType = MediaType.createFontType("yams");
+    assertEquals("font", newType.type());
+    assertEquals("yams", newType.subtype());
+  }
+
   public void testCreateImageType() {
     MediaType newType = MediaType.createImageType("yams");
     assertEquals("image", newType.type());
@@ -160,14 +217,12 @@ public class MediaTypeTest extends TestCase {
 
   public void testGetType() {
     assertEquals("text", MediaType.parse("text/plain").type());
-    assertEquals("application",
-        MediaType.parse("application/atom+xml; charset=utf-8").type());
+    assertEquals("application", MediaType.parse("application/atom+xml; charset=utf-8").type());
   }
 
   public void testGetSubtype() {
     assertEquals("plain", MediaType.parse("text/plain").subtype());
-    assertEquals("atom+xml",
-        MediaType.parse("application/atom+xml; charset=utf-8").subtype());
+    assertEquals("atom+xml", MediaType.parse("application/atom+xml; charset=utf-8").subtype());
   }
 
   private static final ImmutableListMultimap<String, String> PARAMETERS =
@@ -175,23 +230,24 @@ public class MediaTypeTest extends TestCase {
 
   public void testGetParameters() {
     assertEquals(ImmutableListMultimap.of(), MediaType.parse("text/plain").parameters());
-    assertEquals(ImmutableListMultimap.of("charset", "utf-8"),
+    assertEquals(
+        ImmutableListMultimap.of("charset", "utf-8"),
         MediaType.parse("application/atom+xml; charset=utf-8").parameters());
-    assertEquals(PARAMETERS,
-        MediaType.parse("application/atom+xml; a=1; a=2; b=3").parameters());
+    assertEquals(PARAMETERS, MediaType.parse("application/atom+xml; a=1; a=2; b=3").parameters());
   }
 
   public void testWithoutParameters() {
-    assertSame(MediaType.parse("image/gif"),
-        MediaType.parse("image/gif").withoutParameters());
-    assertEquals(MediaType.parse("image/gif"),
-        MediaType.parse("image/gif; foo=bar").withoutParameters());
+    assertSame(MediaType.parse("image/gif"), MediaType.parse("image/gif").withoutParameters());
+    assertEquals(
+        MediaType.parse("image/gif"), MediaType.parse("image/gif; foo=bar").withoutParameters());
   }
 
   public void testWithParameters() {
-    assertEquals(MediaType.parse("text/plain; a=1; a=2; b=3"),
+    assertEquals(
+        MediaType.parse("text/plain; a=1; a=2; b=3"),
         MediaType.parse("text/plain").withParameters(PARAMETERS));
-    assertEquals(MediaType.parse("text/plain; a=1; a=2; b=3"),
+    assertEquals(
+        MediaType.parse("text/plain; a=1; a=2; b=3"),
         MediaType.parse("text/plain; a=1; a=2; b=3").withParameters(PARAMETERS));
   }
 
@@ -202,17 +258,41 @@ public class MediaTypeTest extends TestCase {
     try {
       mediaType.withParameters(parameters);
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  public void testWithParameters_nonAsciiParameter() {
+    MediaType mediaType = MediaType.parse("text/plain");
+    ImmutableListMultimap<String, String> parameters = ImmutableListMultimap.of("…", "a");
+    try {
+      mediaType.withParameters(parameters);
+      fail();
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  public void testWithParameters_nonAsciiParameterValue() {
+    MediaType mediaType = MediaType.parse("text/plain");
+    ImmutableListMultimap<String, String> parameters = ImmutableListMultimap.of("a", "…");
+    try {
+      mediaType.withParameters(parameters);
+      fail();
+    } catch (IllegalArgumentException expected) {
+    }
   }
 
   public void testWithParameter() {
-    assertEquals(MediaType.parse("text/plain; a=1"),
-        MediaType.parse("text/plain").withParameter("a", "1"));
-    assertEquals(MediaType.parse("text/plain; a=1"),
+    assertEquals(
+        MediaType.parse("text/plain; a=1"), MediaType.parse("text/plain").withParameter("a", "1"));
+    assertEquals(
+        MediaType.parse("text/plain; a=1"),
         MediaType.parse("text/plain; a=1; a=2").withParameter("a", "1"));
-    assertEquals(MediaType.parse("text/plain; a=3"),
+    assertEquals(
+        MediaType.parse("text/plain; a=3"),
         MediaType.parse("text/plain; a=1; a=2").withParameter("a", "3"));
-    assertEquals(MediaType.parse("text/plain; a=1; a=2; b=3"),
+    assertEquals(
+        MediaType.parse("text/plain; a=1; a=2; b=3"),
         MediaType.parse("text/plain; a=1; a=2").withParameter("b", "3"));
   }
 
@@ -221,13 +301,97 @@ public class MediaTypeTest extends TestCase {
     try {
       mediaType.withParameter("@", "2");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  public void testWithParameter_nonAsciiParameter() {
+    MediaType mediaType = MediaType.parse("text/plain");
+    try {
+      mediaType.withParameter("…", "a");
+      fail();
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  public void testWithParameter_nonAsciiParameterValue() {
+    MediaType mediaType = MediaType.parse("text/plain");
+    try {
+      mediaType.withParameter("a", "…");
+      fail();
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  public void testWithParameter_emptyParameter() {
+    MediaType mediaType = MediaType.parse("text/plain");
+    try {
+      mediaType.withParameter("", "a");
+      fail();
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  public void testWithParametersIterable() {
+    assertEquals(
+        MediaType.parse("text/plain"),
+        MediaType.parse("text/plain; a=1; a=2").withParameters("a", ImmutableSet.<String>of()));
+    assertEquals(
+        MediaType.parse("text/plain; a=1"),
+        MediaType.parse("text/plain").withParameters("a", ImmutableSet.of("1")));
+    assertEquals(
+        MediaType.parse("text/plain; a=1"),
+        MediaType.parse("text/plain; a=1; a=2").withParameters("a", ImmutableSet.of("1")));
+    assertEquals(
+        MediaType.parse("text/plain; a=1; a=3"),
+        MediaType.parse("text/plain; a=1; a=2").withParameters("a", ImmutableSet.of("1", "3")));
+    assertEquals(
+        MediaType.parse("text/plain; a=1; a=2; b=3; b=4"),
+        MediaType.parse("text/plain; a=1; a=2").withParameters("b", ImmutableSet.of("3", "4")));
+  }
+
+  public void testWithParametersIterable_invalidAttribute() {
+    MediaType mediaType = MediaType.parse("text/plain");
+    try {
+      mediaType.withParameters("@", ImmutableSet.of("2"));
+      fail();
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  public void testWithParametersIterable_nonAsciiParameter() {
+    MediaType mediaType = MediaType.parse("text/plain");
+    try {
+      mediaType.withParameters("…", ImmutableSet.of("a"));
+      fail();
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  public void testWithParametersIterable_nonAsciiParameterValue() {
+    MediaType mediaType = MediaType.parse("text/plain");
+    try {
+      mediaType.withParameters("a", ImmutableSet.of("…"));
+      fail();
+    } catch (IllegalArgumentException expected) {
+    }
+  }
+
+  public void testWithParametersIterable_nullValue() {
+    MediaType mediaType = MediaType.parse("text/plain");
+    try {
+      mediaType.withParameters("a", Arrays.asList((String) null));
+      fail();
+    } catch (NullPointerException expected) {
+    }
   }
 
   public void testWithCharset() {
-    assertEquals(MediaType.parse("text/plain; charset=utf-8"),
+    assertEquals(
+        MediaType.parse("text/plain; charset=utf-8"),
         MediaType.parse("text/plain").withCharset(UTF_8));
-    assertEquals(MediaType.parse("text/plain; charset=utf-8"),
+    assertEquals(
+        MediaType.parse("text/plain; charset=utf-8"),
         MediaType.parse("text/plain; charset=utf-16").withCharset(UTF_8));
   }
 
@@ -261,74 +425,91 @@ public class MediaTypeTest extends TestCase {
     try {
       MediaType.parse("");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
   }
 
   public void testParse_badInput() {
     try {
       MediaType.parse("/");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
     try {
       MediaType.parse("text");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
     try {
       MediaType.parse("text/");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
     try {
       MediaType.parse("te<t/plain");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
     try {
       MediaType.parse("text/pl@in");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
     try {
       MediaType.parse("text/plain;");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
     try {
       MediaType.parse("text/plain; ");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
     try {
       MediaType.parse("text/plain; a");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
     try {
       MediaType.parse("text/plain; a=");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
     try {
       MediaType.parse("text/plain; a=@");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
     try {
       MediaType.parse("text/plain; a=\"@");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
     try {
       MediaType.parse("text/plain; a=1;");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
     try {
       MediaType.parse("text/plain; a=1; ");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
     try {
       MediaType.parse("text/plain; a=1; b");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
     try {
       MediaType.parse("text/plain; a=1; b=");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
     try {
       MediaType.parse("text/plain; a=\u2025");
       fail();
-    } catch (IllegalArgumentException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
   }
 
   public void testGetCharset() {
@@ -336,7 +517,7 @@ public class MediaTypeTest extends TestCase {
     assertThat(MediaType.parse("text/plain; charset=utf-8").charset()).hasValue(UTF_8);
   }
 
-  @GwtIncompatible("Non-UTF-8 Charset")
+  @GwtIncompatible // Non-UTF-8 Charset
   public void testGetCharset_utf16() {
     assertThat(MediaType.parse("text/plain; charset=utf-16").charset()).hasValue(UTF_16);
   }
@@ -346,30 +527,32 @@ public class MediaTypeTest extends TestCase {
     try {
       mediaType.charset();
       fail();
-    } catch (IllegalStateException expected) {}
+    } catch (IllegalStateException expected) {
+    }
   }
 
   public void testGetCharset_illegalCharset() {
-    MediaType mediaType = MediaType.parse(
-        "text/plain; charset=\"!@#$%^&*()\"");
+    MediaType mediaType = MediaType.parse("text/plain; charset=\"!@#$%^&*()\"");
     try {
       mediaType.charset();
       fail();
-    } catch (IllegalCharsetNameException expected) {}
+    } catch (IllegalArgumentException expected) {
+    }
   }
 
   public void testGetCharset_unsupportedCharset() {
-    MediaType mediaType = MediaType.parse(
-        "text/plain; charset=utf-wtf");
+    MediaType mediaType = MediaType.parse("text/plain; charset=utf-wtf");
     try {
       mediaType.charset();
       fail();
-    } catch (UnsupportedCharsetException expected) {}
+    } catch (UnsupportedCharsetException expected) {
+    }
   }
 
   public void testEquals() {
     new EqualsTester()
-        .addEqualityGroup(MediaType.create("text", "plain"),
+        .addEqualityGroup(
+            MediaType.create("text", "plain"),
             MediaType.create("TEXT", "PLAIN"),
             MediaType.parse("text/plain"),
             MediaType.parse("TEXT/PLAIN"),
@@ -377,8 +560,8 @@ public class MediaTypeTest extends TestCase {
         .addEqualityGroup(
             MediaType.create("text", "plain").withCharset(UTF_8),
             MediaType.create("text", "plain").withParameter("CHARSET", "UTF-8"),
-            MediaType.create("text", "plain").withParameters(
-                ImmutableMultimap.of("charset", "utf-8")),
+            MediaType.create("text", "plain")
+                .withParameters(ImmutableMultimap.of("charset", "utf-8")),
             MediaType.parse("text/plain;charset=utf-8"),
             MediaType.parse("text/plain; charset=utf-8"),
             MediaType.parse("text/plain;  charset=utf-8"),
@@ -387,11 +570,14 @@ public class MediaTypeTest extends TestCase {
             MediaType.parse("text/plain; CHARSET=utf-8"),
             MediaType.parse("text/plain; charset=\"utf-8\""),
             MediaType.parse("text/plain; charset=\"\\u\\tf-\\8\""),
-            MediaType.parse("text/plain; charset=UTF-8"))
+            MediaType.parse("text/plain; charset=UTF-8"),
+            MediaType.parse("text/plain ; charset=utf-8"))
         .addEqualityGroup(MediaType.parse("text/plain; charset=utf-8; charset=utf-8"))
-        .addEqualityGroup(MediaType.create("text", "plain").withParameter("a", "value"),
+        .addEqualityGroup(
+            MediaType.create("text", "plain").withParameter("a", "value"),
             MediaType.create("text", "plain").withParameter("A", "value"))
-        .addEqualityGroup(MediaType.create("text", "plain").withParameter("a", "VALUE"),
+        .addEqualityGroup(
+            MediaType.create("text", "plain").withParameter("a", "VALUE"),
             MediaType.create("text", "plain").withParameter("A", "VALUE"))
         .addEqualityGroup(
             MediaType.create("text", "plain")
@@ -403,7 +589,8 @@ public class MediaTypeTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("Non-UTF-8 Charset") public void testEquals_nonUtf8Charsets() {
+  @GwtIncompatible // Non-UTF-8 Charset
+  public void testEquals_nonUtf8Charsets() {
     new EqualsTester()
         .addEqualityGroup(MediaType.create("text", "plain"))
         .addEqualityGroup(MediaType.create("text", "plain").withCharset(UTF_8))
@@ -411,7 +598,7 @@ public class MediaTypeTest extends TestCase {
         .testEquals();
   }
 
-  @GwtIncompatible("com.google.common.testing.NullPointerTester")
+  @GwtIncompatible // com.google.common.testing.NullPointerTester
   public void testNullPointer() {
     NullPointerTester tester = new NullPointerTester();
     tester.testAllPublicConstructors(MediaType.class);
@@ -421,10 +608,14 @@ public class MediaTypeTest extends TestCase {
 
   public void testToString() {
     assertEquals("text/plain", MediaType.create("text", "plain").toString());
-    assertEquals("text/plain; something=\"cr@zy\"; something-else=\"crazy with spaces\"",
+    assertEquals(
+        "text/plain; something=\"cr@zy\"; something-else=\"crazy with spaces\";"
+            + " and-another-thing=\"\"; normal-thing=foo",
         MediaType.create("text", "plain")
             .withParameter("something", "cr@zy")
             .withParameter("something-else", "crazy with spaces")
+            .withParameter("and-another-thing", "")
+            .withParameter("normal-thing", "foo")
             .toString());
   }
 }
