@@ -21,18 +21,20 @@ import static com.google.common.util.concurrent.Uninterruptibles.awaitUninterrup
 import com.google.common.base.CaseFormat;
 import com.google.common.collect.ImmutableList;
 import com.google.common.primitives.Ints;
-
-import junit.framework.TestCase;
-import junit.framework.TestSuite;
-
+import com.google.errorprone.annotations.CanIgnoreReturnValue;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
+import java.time.Duration;
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Comparator;
+import java.util.List;
 import java.util.Locale;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.FutureTask;
 import java.util.concurrent.TimeUnit;
+import junit.framework.TestCase;
+import junit.framework.TestSuite;
 
 /**
  * Generated tests for {@link Monitor}.
@@ -44,7 +46,6 @@ import java.util.concurrent.TimeUnit;
  *
  * @author Justin T. Sampson
  */
-
 public class GeneratedMonitorTest extends TestCase {
 
   public static TestSuite suite() {
@@ -59,19 +60,15 @@ public class GeneratedMonitorTest extends TestCase {
       }
     }
 
-    assertEquals(548, suite.testCount());
+    assertEquals(980, suite.testCount());
 
     return suite;
   }
 
-  /**
-   * A typical timeout value we'll use in the tests.
-   */
+  /** A typical timeout value we'll use in the tests. */
   private static final long SMALL_TIMEOUT_MILLIS = 10;
 
-  /**
-   * How long to wait when determining that a thread is blocked if we expect it to be blocked.
-   */
+  /** How long to wait when determining that a thread is blocked if we expect it to be blocked. */
   private static final long EXPECTED_HANG_DELAY_MILLIS = 75;
 
   /**
@@ -85,7 +82,6 @@ public class GeneratedMonitorTest extends TestCase {
    * takes place in {@link #addTests(TestSuite, Method)}.
    */
   private enum Scenario {
-
     SATISFIED_AND_UNOCCUPIED_BEFORE_ENTERING,
     UNSATISFIED_AND_UNOCCUPIED_BEFORE_ENTERING,
     SATISFIED_AND_OCCUPIED_BEFORE_ENTERING,
@@ -101,14 +97,10 @@ public class GeneratedMonitorTest extends TestCase {
     public String toString() {
       return CaseFormat.UPPER_UNDERSCORE.to(CaseFormat.UPPER_CAMEL, name());
     }
-
   }
 
-  /**
-   * Timeout values to combine with each {@link Scenario}.
-   */
+  /** Timeout values to combine with each {@link Scenario}. */
   private enum Timeout {
-
     MIN(Long.MIN_VALUE, "-oo"),
     MINUS_SMALL(-SMALL_TIMEOUT_MILLIS, "-" + SMALL_TIMEOUT_MILLIS + "ms"),
     ZERO(0L, "0ms"),
@@ -119,7 +111,7 @@ public class GeneratedMonitorTest extends TestCase {
     final long millis;
     final String label;
 
-    private Timeout(long millis, String label) {
+    Timeout(long millis, String label) {
       this.millis = millis;
       this.label = label;
     }
@@ -128,14 +120,10 @@ public class GeneratedMonitorTest extends TestCase {
     public String toString() {
       return label;
     }
-
   }
 
-  /**
-   * Convenient subsets of the {@link Timeout} enumeration for specifying scenario outcomes.
-   */
+  /** Convenient subsets of the {@link Timeout} enumeration for specifying scenario outcomes. */
   private enum TimeoutsToUse {
-
     ANY(Timeout.values()),
     PAST(Timeout.MIN, Timeout.MINUS_SMALL, Timeout.ZERO),
     FUTURE(Timeout.SMALL, Timeout.MAX),
@@ -145,54 +133,38 @@ public class GeneratedMonitorTest extends TestCase {
 
     final ImmutableList<Timeout> timeouts;
 
-    private TimeoutsToUse(Timeout... timeouts) {
+    TimeoutsToUse(Timeout... timeouts) {
       this.timeouts = ImmutableList.copyOf(timeouts);
     }
-
   }
 
-  /**
-   * Possible outcomes of calling any of the methods under test.
-   */
+  /** Possible outcomes of calling any of the methods under test. */
   private enum Outcome {
 
-    /**
-     * The method returned normally and is either void or returned true.
-     */
+    /** The method returned normally and is either void or returned true. */
     SUCCESS,
 
-    /**
-     * The method returned false.
-     */
+    /** The method returned false. */
     FAILURE,
 
-    /**
-     * The method threw an InterruptedException.
-     */
+    /** The method threw an InterruptedException. */
     INTERRUPT,
 
-    /**
-     * The method did not return or throw anything.
-     */
+    /** The method did not return or throw anything. */
     HANG;
 
     @Override
     public String toString() {
       return CaseFormat.UPPER_UNDERSCORE.to(CaseFormat.UPPER_CAMEL, name());
     }
-
   }
 
-  /**
-   * Identifies all enterXxx and tryEnterXxx methods.
-   */
+  /** Identifies all enterXxx and tryEnterXxx methods. */
   private static boolean isAnyEnter(Method method) {
     return method.getName().startsWith("enter") || method.getName().startsWith("tryEnter");
   }
 
-  /**
-   * Identifies just tryEnterXxx methods (a subset of {@link #isAnyEnter}), which never block.
-   */
+  /** Identifies just tryEnterXxx methods (a subset of {@link #isAnyEnter}), which never block. */
   private static boolean isTryEnter(Method method) {
     return method.getName().startsWith("tryEnter");
   }
@@ -205,64 +177,65 @@ public class GeneratedMonitorTest extends TestCase {
     return method.getName().startsWith("enterIf");
   }
 
-  /**
-   * Identifies all waitForXxx methods, which must be called while occupying the monitor.
-   */
+  /** Identifies all waitForXxx methods, which must be called while occupying the monitor. */
   private static boolean isWaitFor(Method method) {
     return method.getName().startsWith("waitFor");
   }
 
-  /**
-   * Determines whether the given method takes a Guard as its first parameter.
-   */
+  /** Determines whether the given method takes a Guard as its first parameter. */
   private static boolean isGuarded(Method method) {
     Class<?>[] parameterTypes = method.getParameterTypes();
     return parameterTypes.length >= 1 && parameterTypes[0] == Monitor.Guard.class;
   }
 
-  /**
-   * Determines whether the given method takes a time and unit as its last two parameters.
-   */
+  /** Determines whether the given method is time-based. */
   private static boolean isTimed(Method method) {
+    return isLongTimeUnitBased(method) || isDurationBased(method);
+  }
+
+  /** Determines whether the given method takes a time and unit as its last two parameters. */
+  private static boolean isLongTimeUnitBased(Method method) {
     Class<?>[] parameterTypes = method.getParameterTypes();
     return parameterTypes.length >= 2
         && parameterTypes[parameterTypes.length - 2] == long.class
         && parameterTypes[parameterTypes.length - 1] == TimeUnit.class;
   }
 
-  /**
-   * Determines whether the given method returns a boolean value.
-   */
+  /** Determines whether the given method takes a Duration as its last parameter. */
+  private static boolean isDurationBased(Method method) {
+    Class<?>[] parameterTypes = method.getParameterTypes();
+    return parameterTypes.length >= 1
+        && parameterTypes[parameterTypes.length - 1] == Duration.class;
+  }
+
+  /** Determines whether the given method returns a boolean value. */
   private static boolean isBoolean(Method method) {
     return method.getReturnType() == boolean.class;
   }
 
-  /**
-   * Determines whether the given method can throw InterruptedException.
-   */
+  /** Determines whether the given method can throw InterruptedException. */
   private static boolean isInterruptible(Method method) {
     return Arrays.asList(method.getExceptionTypes()).contains(InterruptedException.class);
   }
 
-  /**
-   * Sorts the given methods primarily by name and secondarily by number of parameters.
-   */
+  /** Sorts the given methods primarily by name and secondarily by number of parameters. */
   private static void sortMethods(Method[] methods) {
-    Arrays.sort(methods, new Comparator<Method>() {
-      @Override public int compare(Method m1, Method m2) {
-        int nameComparison = m1.getName().compareTo(m2.getName());
-        if (nameComparison != 0) {
-          return nameComparison;
-        } else {
-          return Ints.compare(m1.getParameterTypes().length, m2.getParameterTypes().length);
-        }
-      }
-    });
+    Arrays.sort(
+        methods,
+        new Comparator<Method>() {
+          @Override
+          public int compare(Method m1, Method m2) {
+            int nameComparison = m1.getName().compareTo(m2.getName());
+            if (nameComparison != 0) {
+              return nameComparison;
+            } else {
+              return Ints.compare(m1.getParameterTypes().length, m2.getParameterTypes().length);
+            }
+          }
+        });
   }
 
-  /**
-   * Validates that the given method's signature meets all of our assumptions.
-   */
+  /** Validates that the given method's signature meets all of our assumptions. */
   private static void validateMethod(Method method) {
     String desc = method.toString();
 
@@ -274,11 +247,21 @@ public class GeneratedMonitorTest extends TestCase {
         assertFalse(desc, isTimed(method));
         break;
       case 1:
-        assertTrue(desc, isGuarded(method));
-        assertFalse(desc, isTimed(method));
+        if (isDurationBased(method)) {
+          assertFalse(desc, isGuarded(method));
+        } else {
+          assertTrue(desc, isGuarded(method));
+        }
+        // we can't make an assumption about isTimed() because now we have single-parameter methods
+        // that accept a java.time.Duration
+        assertFalse(desc, isLongTimeUnitBased(method));
         break;
       case 2:
-        assertFalse(desc, isGuarded(method));
+        if (isDurationBased(method)) {
+          assertTrue(desc, isGuarded(method));
+        } else {
+          assertFalse(desc, isGuarded(method));
+        }
         assertTrue(desc, isTimed(method));
         break;
       case 3:
@@ -321,85 +304,99 @@ public class GeneratedMonitorTest extends TestCase {
     }
   }
 
-  /**
-   * Generates all test cases appropriate for the given method.
-   */
+  /** Generates all test cases appropriate for the given method. */
   private static void addTests(TestSuite suite, Method method) {
     if (isGuarded(method)) {
-      for (boolean fair1 : new boolean[] { true, false }) {
-        for (boolean fair2 : new boolean[] { true, false }) {
+      for (boolean fair1 : new boolean[] {true, false}) {
+        for (boolean fair2 : new boolean[] {true, false}) {
           suite.addTest(generateGuardWithWrongMonitorTestCase(method, fair1, fair2));
         }
       }
     }
     if (isAnyEnter(method)) {
-      addTests(suite, method,
+      addTests(
+          suite,
+          method,
           Scenario.SATISFIED_AND_UNOCCUPIED_BEFORE_ENTERING,
           TimeoutsToUse.ANY,
           Outcome.SUCCESS);
-      addTests(suite, method,
+      addTests(
+          suite,
+          method,
           Scenario.UNSATISFIED_AND_UNOCCUPIED_BEFORE_ENTERING,
           TimeoutsToUse.FINITE,
           isGuarded(method)
               ? (isBoolean(method) ? Outcome.FAILURE : Outcome.HANG)
               : Outcome.SUCCESS);
-      addTests(suite, method,
+      addTests(
+          suite,
+          method,
           Scenario.UNSATISFIED_AND_UNOCCUPIED_BEFORE_ENTERING,
           TimeoutsToUse.INFINITE,
           isGuarded(method)
               ? (isTryEnter(method) || isEnterIf(method) ? Outcome.FAILURE : Outcome.HANG)
               : Outcome.SUCCESS);
-      addTests(suite, method,
+      addTests(
+          suite,
+          method,
           Scenario.SATISFIED_AND_OCCUPIED_BEFORE_ENTERING,
           TimeoutsToUse.FINITE,
           isBoolean(method) ? Outcome.FAILURE : Outcome.HANG);
-      addTests(suite, method,
+      addTests(
+          suite,
+          method,
           Scenario.SATISFIED_AND_OCCUPIED_BEFORE_ENTERING,
           TimeoutsToUse.INFINITE,
-          isGuarded(method)
-              ? Outcome.HANG
-              : (isTryEnter(method) ? Outcome.FAILURE : Outcome.HANG));
-      addTests(suite, method,
+          isGuarded(method) ? Outcome.HANG : (isTryEnter(method) ? Outcome.FAILURE : Outcome.HANG));
+      addTests(
+          suite,
+          method,
           Scenario.SATISFIED_UNOCCUPIED_AND_INTERRUPTED_BEFORE_ENTERING,
           TimeoutsToUse.ANY,
           isInterruptible(method) ? Outcome.INTERRUPT : Outcome.SUCCESS);
     } else { // any waitForXxx method
       suite.addTest(generateWaitForWhenNotOccupyingTestCase(method, true));
       suite.addTest(generateWaitForWhenNotOccupyingTestCase(method, false));
-      addTests(suite, method,
-          Scenario.SATISFIED_BEFORE_WAITING,
-          TimeoutsToUse.ANY,
-          Outcome.SUCCESS);
-      addTests(suite, method,
-          Scenario.SATISFIED_WHILE_WAITING,
-          TimeoutsToUse.INFINITE,
-          Outcome.SUCCESS);
-      addTests(suite, method,
-          Scenario.SATISFIED_WHILE_WAITING,
-          TimeoutsToUse.PAST,
-          Outcome.FAILURE);
-      addTests(suite, method,
+      addTests(
+          suite, method, Scenario.SATISFIED_BEFORE_WAITING, TimeoutsToUse.ANY, Outcome.SUCCESS);
+      addTests(
+          suite, method, Scenario.SATISFIED_WHILE_WAITING, TimeoutsToUse.INFINITE, Outcome.SUCCESS);
+      addTests(
+          suite, method, Scenario.SATISFIED_WHILE_WAITING, TimeoutsToUse.PAST, Outcome.FAILURE);
+      addTests(
+          suite,
+          method,
           Scenario.SATISFIED_AND_INTERRUPTED_BEFORE_WAITING,
           TimeoutsToUse.ANY,
           Outcome.SUCCESS);
-      addTests(suite, method,
+      addTests(
+          suite,
+          method,
           Scenario.UNSATISFIED_BEFORE_AND_WHILE_WAITING,
           TimeoutsToUse.FINITE,
           Outcome.FAILURE);
-      addTests(suite, method,
+      addTests(
+          suite,
+          method,
           Scenario.UNSATISFIED_BEFORE_AND_WHILE_WAITING,
           TimeoutsToUse.INFINITE,
           Outcome.HANG);
-      addTests(suite, method,
+      addTests(
+          suite,
+          method,
           Scenario.UNSATISFIED_AND_INTERRUPTED_BEFORE_WAITING,
           TimeoutsToUse.PAST,
           // prefer responding to interrupt over timing out
           isInterruptible(method) ? Outcome.INTERRUPT : Outcome.FAILURE);
-      addTests(suite, method,
+      addTests(
+          suite,
+          method,
           Scenario.UNSATISFIED_AND_INTERRUPTED_BEFORE_WAITING,
           TimeoutsToUse.SMALL,
           isInterruptible(method) ? Outcome.INTERRUPT : Outcome.FAILURE);
-      addTests(suite, method,
+      addTests(
+          suite,
+          method,
           Scenario.UNSATISFIED_AND_INTERRUPTED_BEFORE_WAITING,
           TimeoutsToUse.INFINITE,
           isInterruptible(method) ? Outcome.INTERRUPT : Outcome.HANG);
@@ -413,9 +410,13 @@ public class GeneratedMonitorTest extends TestCase {
    * implicit timeout of that method matches the given timeoutsToUse. For example, enter() is
    * treated like enter(MAX, MILLIS) and tryEnter() is treated like enter(0, MILLIS).
    */
-  private static void addTests(TestSuite suite, Method method, Scenario scenario,
-      TimeoutsToUse timeoutsToUse, Outcome expectedOutcome) {
-    for (boolean fair : new boolean[] { true, false }) {
+  private static void addTests(
+      TestSuite suite,
+      Method method,
+      Scenario scenario,
+      TimeoutsToUse timeoutsToUse,
+      Outcome expectedOutcome) {
+    for (boolean fair : new boolean[] {true, false}) {
       if (isTimed(method)) {
         for (Timeout timeout : timeoutsToUse.timeouts) {
           suite.addTest(new GeneratedMonitorTest(method, scenario, fair, timeout, expectedOutcome));
@@ -429,9 +430,7 @@ public class GeneratedMonitorTest extends TestCase {
     }
   }
 
-  /**
-   * A guard that encapsulates a simple, mutable boolean flag.
-   */
+  /** A guard that encapsulates a simple, mutable boolean flag. */
   static class FlagGuard extends Monitor.Guard {
 
     private boolean satisfied;
@@ -448,7 +447,6 @@ public class GeneratedMonitorTest extends TestCase {
     public void setSatisfied(boolean satisfied) {
       this.satisfied = satisfied;
     }
-
   }
 
   private final Method method;
@@ -477,7 +475,8 @@ public class GeneratedMonitorTest extends TestCase {
 
   private static String nameFor(
       Method method, Scenario scenario, boolean fair, Timeout timeout, Outcome expectedOutcome) {
-    return String.format(Locale.ROOT,
+    return String.format(
+        Locale.ROOT,
         "%s%s(%s)/%s->%s",
         method.getName(),
         fair ? "(fair)" : "(nonfair)",
@@ -488,19 +487,28 @@ public class GeneratedMonitorTest extends TestCase {
 
   @Override
   protected void runTest() throws Throwable {
-    final Runnable runChosenTest = new Runnable() {
-      @Override public void run() { runChosenTest(); }
-    };
-    final FutureTask<Void> task = new FutureTask<Void>(runChosenTest, null);
-    startThread(new Runnable() {
-        @Override public void run() { task.run(); }
-      });
+    final Runnable runChosenTest =
+        new Runnable() {
+          @Override
+          public void run() {
+            runChosenTest();
+          }
+        };
+    final FutureTask<Void> task = new FutureTask<>(runChosenTest, null);
+    startThread(
+        new Runnable() {
+          @Override
+          public void run() {
+            task.run();
+          }
+        });
     awaitUninterruptibly(doingCallLatch);
-    long hangDelayMillis = (expectedOutcome == Outcome.HANG)
-        ? EXPECTED_HANG_DELAY_MILLIS
-        : UNEXPECTED_HANG_DELAY_MILLIS;
-    boolean hung = !awaitUninterruptibly(
-        callCompletedLatch, hangDelayMillis, TimeUnit.MILLISECONDS);
+    long hangDelayMillis =
+        (expectedOutcome == Outcome.HANG)
+            ? EXPECTED_HANG_DELAY_MILLIS
+            : UNEXPECTED_HANG_DELAY_MILLIS;
+    boolean hung =
+        !awaitUninterruptibly(callCompletedLatch, hangDelayMillis, TimeUnit.MILLISECONDS);
     if (hung) {
       assertEquals(expectedOutcome, Outcome.HANG);
     } else {
@@ -523,7 +531,8 @@ public class GeneratedMonitorTest extends TestCase {
     // of any remaining waiting threads.
 
     tearDownLatch.countDown();
-    assertTrue("Monitor still occupied in tearDown()",
+    assertTrue(
+        "Monitor still occupied in tearDown()",
         monitor.enter(UNEXPECTED_HANG_DELAY_MILLIS, TimeUnit.MILLISECONDS));
     try {
       guard.setSatisfied(true);
@@ -559,8 +568,8 @@ public class GeneratedMonitorTest extends TestCase {
 
     assertEquals(expectedOutcome, actualOutcome);
     assertEquals(expectedOutcome == Outcome.SUCCESS, occupiedAfterCall);
-    assertEquals(interruptedBeforeCall && expectedOutcome != Outcome.INTERRUPT,
-        interruptedAfterCall);
+    assertEquals(
+        interruptedBeforeCall && expectedOutcome != Outcome.INTERRUPT, interruptedAfterCall);
   }
 
   private void doEnterScenarioSetUp() {
@@ -599,8 +608,8 @@ public class GeneratedMonitorTest extends TestCase {
 
       assertEquals(expectedOutcome, actualOutcome);
       assertTrue(occupiedAfterCall);
-      assertEquals(interruptedBeforeCall && expectedOutcome != Outcome.INTERRUPT,
-          interruptedAfterCall);
+      assertEquals(
+          interruptedBeforeCall && expectedOutcome != Outcome.INTERRUPT, interruptedAfterCall);
     } finally {
       guard.setSatisfied(true);
       monitor.leave();
@@ -634,21 +643,22 @@ public class GeneratedMonitorTest extends TestCase {
   }
 
   private Outcome doCall() {
-    boolean guarded = isGuarded(method);
-    boolean timed = isTimed(method);
-    Object[] arguments = new Object[(guarded ? 1 : 0) + (timed ? 2 : 0)];
-    if (guarded) {
-      arguments[0] = guard;
+    List<Object> arguments = new ArrayList<>();
+    if (isGuarded(method)) {
+      arguments.add(guard);
     }
-    if (timed) {
-      arguments[arguments.length - 2] = timeout.millis;
-      arguments[arguments.length - 1] = TimeUnit.MILLISECONDS;
+    if (isLongTimeUnitBased(method)) {
+      arguments.add(timeout.millis);
+      arguments.add(TimeUnit.MILLISECONDS);
+    }
+    if (isDurationBased(method)) {
+      arguments.add(Duration.ofMillis(timeout.millis));
     }
     try {
       Object result;
       doingCallLatch.countDown();
       try {
-        result = method.invoke(monitor, arguments);
+        result = method.invoke(monitor, arguments.toArray());
       } finally {
         callCompletedLatch.countDown();
       }
@@ -682,32 +692,37 @@ public class GeneratedMonitorTest extends TestCase {
 
   private void enterSatisfyGuardAndLeaveInAnotherThread() {
     final CountDownLatch startedLatch = new CountDownLatch(1);
-    startThread(new Runnable() {
-      @Override public void run() {
-        startedLatch.countDown();
-        enterSatisfyGuardAndLeaveInCurrentThread();
-      }
-    });
+    startThread(
+        new Runnable() {
+          @Override
+          public void run() {
+            startedLatch.countDown();
+            enterSatisfyGuardAndLeaveInCurrentThread();
+          }
+        });
     awaitUninterruptibly(startedLatch);
   }
 
   private void enterAndRemainOccupyingInAnotherThread() {
     final CountDownLatch enteredLatch = new CountDownLatch(1);
-    startThread(new Runnable() {
-      @Override public void run() {
-        monitor.enter();
-        try {
-          enteredLatch.countDown();
-          awaitUninterruptibly(tearDownLatch);
-          guard.setSatisfied(true);
-        } finally {
-          monitor.leave();
-        }
-      }
-    });
+    startThread(
+        new Runnable() {
+          @Override
+          public void run() {
+            monitor.enter();
+            try {
+              enteredLatch.countDown();
+              awaitUninterruptibly(tearDownLatch);
+              guard.setSatisfied(true);
+            } finally {
+              monitor.leave();
+            }
+          }
+        });
     awaitUninterruptibly(enteredLatch);
   }
 
+  @CanIgnoreReturnValue
   static Thread startThread(Runnable runnable) {
     Thread thread = new Thread(runnable);
     thread.setDaemon(true);
@@ -719,17 +734,24 @@ public class GeneratedMonitorTest extends TestCase {
    * Generates a test case verifying that calling any enterXxx, tryEnterXxx, or waitForXxx method
    * with a guard that doesn't match the monitor produces an IllegalMonitorStateException.
    */
-  private static TestCase generateGuardWithWrongMonitorTestCase(final Method method,
-                                                                final boolean fair1,
-                                                                final boolean fair2) {
+  private static TestCase generateGuardWithWrongMonitorTestCase(
+      final Method method, final boolean fair1, final boolean fair2) {
     final boolean timed = isTimed(method); // Not going to bother with all timeouts, just 0ms.
     return new TestCase(method.getName() + (timed ? "(0ms)" : "()") + "/WrongMonitor->IMSE") {
-      @Override protected void runTest() throws Throwable {
+      @Override
+      protected void runTest() throws Throwable {
         Monitor monitor1 = new Monitor(fair1);
         Monitor monitor2 = new Monitor(fair2);
         FlagGuard guard = new FlagGuard(monitor2);
-        Object[] arguments =
-            (timed ? new Object[] {guard, 0L, TimeUnit.MILLISECONDS} : new Object[] {guard});
+        List<Object> arguments = new ArrayList<>();
+        arguments.add(guard);
+        if (isDurationBased(method)) {
+          arguments.add(Duration.ZERO);
+        }
+        if (isLongTimeUnitBased(method)) {
+          arguments.add(0L);
+          arguments.add(TimeUnit.MILLISECONDS);
+        }
         boolean occupyMonitor = isWaitFor(method);
         if (occupyMonitor) {
           // If we don't already occupy the monitor, we'll get an IMSE regardless of the guard (see
@@ -737,7 +759,7 @@ public class GeneratedMonitorTest extends TestCase {
           monitor1.enter();
         }
         try {
-          method.invoke(monitor1, arguments);
+          method.invoke(monitor1, arguments.toArray());
           fail("expected IllegalMonitorStateException");
         } catch (InvocationTargetException e) {
           assertEquals(IllegalMonitorStateException.class, e.getTargetException().getClass());
@@ -754,21 +776,30 @@ public class GeneratedMonitorTest extends TestCase {
    * Generates a test case verifying that calling any waitForXxx method when not occupying the
    * monitor produces an IllegalMonitorStateException.
    */
-  private static TestCase generateWaitForWhenNotOccupyingTestCase(final Method method,
-                                                                  final boolean fair) {
+  private static TestCase generateWaitForWhenNotOccupyingTestCase(
+      final Method method, final boolean fair) {
     final boolean timed = isTimed(method); // Not going to bother with all timeouts, just 0ms.
-    String testName = method.getName()
-        + (fair ? "(fair)" : "(nonfair)")
-        + (timed ? "(0ms)" : "()")
-        + "/NotOccupying->IMSE";
+    String testName =
+        method.getName()
+            + (fair ? "(fair)" : "(nonfair)")
+            + (timed ? "(0ms)" : "()")
+            + "/NotOccupying->IMSE";
     return new TestCase(testName) {
-      @Override protected void runTest() throws Throwable {
+      @Override
+      protected void runTest() throws Throwable {
         Monitor monitor = new Monitor(fair);
         FlagGuard guard = new FlagGuard(monitor);
-        Object[] arguments =
-            (timed ? new Object[] {guard, 0L, TimeUnit.MILLISECONDS} : new Object[] {guard});
+        List<Object> arguments = new ArrayList<>();
+        arguments.add(guard);
+        if (isDurationBased(method)) {
+          arguments.add(Duration.ZERO);
+        }
+        if (isLongTimeUnitBased(method)) {
+          arguments.add(0L);
+          arguments.add(TimeUnit.MILLISECONDS);
+        }
         try {
-          method.invoke(monitor, arguments);
+          method.invoke(monitor, arguments.toArray());
           fail("expected IllegalMonitorStateException");
         } catch (InvocationTargetException e) {
           assertEquals(IllegalMonitorStateException.class, e.getTargetException().getClass());
@@ -783,5 +814,4 @@ public class GeneratedMonitorTest extends TestCase {
     e.initCause(cause);
     return e;
   }
-
 }
