@@ -1,23 +1,23 @@
 /*
  * Copyright (C) 2011 The Guava Authors
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License. You may obtain a copy of the License at
  *
  * http://www.apache.org/licenses/LICENSE-2.0
  *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Unless required by applicable law or agreed to in writing, software distributed under the License
+ * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
+ * or implied. See the License for the specific language governing permissions and limitations under
+ * the License.
  */
 
 package com.google.common.cache;
 
 import static com.google.common.base.Preconditions.checkArgument;
+import static com.google.common.base.Strings.isNullOrEmpty;
 
+import com.google.common.annotations.GwtIncompatible;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.MoreObjects;
 import com.google.common.base.Objects;
@@ -25,64 +25,66 @@ import com.google.common.base.Splitter;
 import com.google.common.cache.LocalCache.Strength;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
-
 import java.util.List;
 import java.util.Locale;
 import java.util.concurrent.TimeUnit;
-
-import javax.annotation.Nullable;
+import javax.annotation.CheckForNull;
+import org.checkerframework.checker.nullness.qual.Nullable;
 
 /**
  * A specification of a {@link CacheBuilder} configuration.
  *
- * <p>{@code CacheBuilderSpec} supports parsing configuration off of a string, which
- * makes it especially useful for command-line configuration of a {@code CacheBuilder}.
+ * <p>{@code CacheBuilderSpec} supports parsing configuration off of a string, which makes it
+ * especially useful for command-line configuration of a {@code CacheBuilder}.
+ *
+ * <p>The string syntax is a series of comma-separated keys or key-value pairs, each corresponding
+ * to a {@code CacheBuilder} method.
  *
- * <p>The string syntax is a series of comma-separated keys or key-value pairs,
- * each corresponding to a {@code CacheBuilder} method.
  * <ul>
- * <li>{@code concurrencyLevel=[integer]}: sets {@link CacheBuilder#concurrencyLevel}.
- * <li>{@code initialCapacity=[integer]}: sets {@link CacheBuilder#initialCapacity}.
- * <li>{@code maximumSize=[long]}: sets {@link CacheBuilder#maximumSize}.
- * <li>{@code maximumWeight=[long]}: sets {@link CacheBuilder#maximumWeight}.
- * <li>{@code expireAfterAccess=[duration]}: sets {@link CacheBuilder#expireAfterAccess}.
- * <li>{@code expireAfterWrite=[duration]}: sets {@link CacheBuilder#expireAfterWrite}.
- * <li>{@code refreshAfterWrite=[duration]}: sets {@link CacheBuilder#refreshAfterWrite}.
- * <li>{@code weakKeys}: sets {@link CacheBuilder#weakKeys}.
- * <li>{@code softValues}: sets {@link CacheBuilder#softValues}.
- * <li>{@code weakValues}: sets {@link CacheBuilder#weakValues}.
- * <li>{@code recordStats}: sets {@link CacheBuilder#recordStats}.
+ *   <li>{@code concurrencyLevel=[integer]}: sets {@link CacheBuilder#concurrencyLevel}.
+ *   <li>{@code initialCapacity=[integer]}: sets {@link CacheBuilder#initialCapacity}.
+ *   <li>{@code maximumSize=[long]}: sets {@link CacheBuilder#maximumSize}.
+ *   <li>{@code maximumWeight=[long]}: sets {@link CacheBuilder#maximumWeight}.
+ *   <li>{@code expireAfterAccess=[duration]}: sets {@link CacheBuilder#expireAfterAccess}.
+ *   <li>{@code expireAfterWrite=[duration]}: sets {@link CacheBuilder#expireAfterWrite}.
+ *   <li>{@code refreshAfterWrite=[duration]}: sets {@link CacheBuilder#refreshAfterWrite}.
+ *   <li>{@code weakKeys}: sets {@link CacheBuilder#weakKeys}.
+ *   <li>{@code softValues}: sets {@link CacheBuilder#softValues}.
+ *   <li>{@code weakValues}: sets {@link CacheBuilder#weakValues}.
+ *   <li>{@code recordStats}: sets {@link CacheBuilder#recordStats}.
  * </ul>
  *
- * <p>The set of supported keys will grow as {@code CacheBuilder} evolves, but existing keys
- * will never be removed.
+ * <p>The set of supported keys will grow as {@code CacheBuilder} evolves, but existing keys will
+ * never be removed.
  *
- * <p>Durations are represented by an integer, followed by one of "d", "h", "m",
- * or "s", representing days, hours, minutes, or seconds respectively.  (There
- * is currently no syntax to request expiration in milliseconds, microseconds,
- * or nanoseconds.)
+ * <p>Durations are represented by an integer, followed by one of "d", "h", "m", or "s",
+ * representing days, hours, minutes, or seconds respectively. (There is currently no syntax to
+ * request expiration in milliseconds, microseconds, or nanoseconds.)
+ *
+ * <p>Whitespace before and after commas and equal signs is ignored. Keys may not be repeated; it is
+ * also illegal to use the following pairs of keys in a single value:
  *
- * <p>Whitespace before and after commas and equal signs is ignored.  Keys may
- * not be repeated;  it is also illegal to use the following pairs of keys in
- * a single value:
  * <ul>
- * <li>{@code maximumSize} and {@code maximumWeight}
- * <li>{@code softValues} and {@code weakValues}
+ *   <li>{@code maximumSize} and {@code maximumWeight}
+ *   <li>{@code softValues} and {@code weakValues}
  * </ul>
  *
- * <p>{@code CacheBuilderSpec} does not support configuring {@code CacheBuilder} methods
- * with non-value parameters.  These must be configured in code.
+ * <p>{@code CacheBuilderSpec} does not support configuring {@code CacheBuilder} methods with
+ * non-value parameters. These must be configured in code.
  *
- * <p>A new {@code CacheBuilder} can be instantiated from a {@code CacheBuilderSpec} using
- * {@link CacheBuilder#from(CacheBuilderSpec)} or {@link CacheBuilder#from(String)}.
+ * <p>A new {@code CacheBuilder} can be instantiated from a {@code CacheBuilderSpec} using {@link
+ * CacheBuilder#from(CacheBuilderSpec)} or {@link CacheBuilder#from(String)}.
  *
  * @author Adam Winer
  * @since 12.0
  */
+@SuppressWarnings("GoodTime") // lots of violations (nanosecond math)
+@GwtIncompatible
+@ElementTypesAreNonnullByDefault
 public final class CacheBuilderSpec {
   /** Parses a single value. */
   private interface ValueParser {
-    void parse(CacheBuilderSpec spec, String key, @Nullable String value);
+    void parse(CacheBuilderSpec spec, String key, @CheckForNull String value);
   }
 
   /** Splits each key-value pair. */
@@ -106,22 +108,22 @@ public final class CacheBuilderSpec {
           .put("expireAfterWrite", new WriteDurationParser())
           .put("refreshAfterWrite", new RefreshDurationParser())
           .put("refreshInterval", new RefreshDurationParser())
-          .build();
+          .buildOrThrow();
 
-  @VisibleForTesting Integer initialCapacity;
-  @VisibleForTesting Long maximumSize;
-  @VisibleForTesting Long maximumWeight;
-  @VisibleForTesting Integer concurrencyLevel;
-  @VisibleForTesting Strength keyStrength;
-  @VisibleForTesting Strength valueStrength;
-  @VisibleForTesting Boolean recordStats;
+  @VisibleForTesting @CheckForNull Integer initialCapacity;
+  @VisibleForTesting @CheckForNull Long maximumSize;
+  @VisibleForTesting @CheckForNull Long maximumWeight;
+  @VisibleForTesting @CheckForNull Integer concurrencyLevel;
+  @VisibleForTesting @CheckForNull Strength keyStrength;
+  @VisibleForTesting @CheckForNull Strength valueStrength;
+  @VisibleForTesting @CheckForNull Boolean recordStats;
   @VisibleForTesting long writeExpirationDuration;
-  @VisibleForTesting TimeUnit writeExpirationTimeUnit;
+  @VisibleForTesting @CheckForNull TimeUnit writeExpirationTimeUnit;
   @VisibleForTesting long accessExpirationDuration;
-  @VisibleForTesting TimeUnit accessExpirationTimeUnit;
+  @VisibleForTesting @CheckForNull TimeUnit accessExpirationTimeUnit;
   @VisibleForTesting long refreshDuration;
-  @VisibleForTesting TimeUnit refreshTimeUnit;
-  /** Specification;  used for toParseableString(). */
+  @VisibleForTesting @CheckForNull TimeUnit refreshTimeUnit;
+  /** Specification; used for toParseableString(). */
   private final String specification;
 
   private CacheBuilderSpec(String specification) {
@@ -139,8 +141,10 @@ public final class CacheBuilderSpec {
       for (String keyValuePair : KEYS_SPLITTER.split(cacheBuilderSpecification)) {
         List<String> keyAndValue = ImmutableList.copyOf(KEY_VALUE_SPLITTER.split(keyValuePair));
         checkArgument(!keyAndValue.isEmpty(), "blank key-value pair");
-        checkArgument(keyAndValue.size() <= 2,
-            "key-value pair %s with more than one equals sign", keyValuePair);
+        checkArgument(
+            keyAndValue.size() <= 2,
+            "key-value pair %s with more than one equals sign",
+            keyValuePair);
 
         // Find the ValueParser for the current key.
         String key = keyAndValue.get(0);
@@ -155,17 +159,13 @@ public final class CacheBuilderSpec {
     return spec;
   }
 
-  /**
-   * Returns a CacheBuilderSpec that will prevent caching.
-   */
+  /** Returns a CacheBuilderSpec that will prevent caching. */
   public static CacheBuilderSpec disableCaching() {
     // Maximum size of zero is one way to block caching
     return CacheBuilderSpec.parse("maximumSize=0");
   }
 
-  /**
-   * Returns a CacheBuilder configured according to this instance's specification.
-   */
+  /** Returns a CacheBuilder configured according to this instance's specification. */
   CacheBuilder<Object, Object> toCacheBuilder() {
     CacheBuilder<Object, Object> builder = CacheBuilder.newBuilder();
     if (initialCapacity != null) {
@@ -218,18 +218,17 @@ public final class CacheBuilderSpec {
   }
 
   /**
-   * Returns a string that can be used to parse an equivalent
-   * {@code CacheBuilderSpec}.  The order and form of this representation is
-   * not guaranteed, except that reparsing its output will produce
-   * a {@code CacheBuilderSpec} equal to this instance.
+   * Returns a string that can be used to parse an equivalent {@code CacheBuilderSpec}. The order
+   * and form of this representation is not guaranteed, except that reparsing its output will
+   * produce a {@code CacheBuilderSpec} equal to this instance.
    */
   public String toParsableString() {
     return specification;
   }
 
   /**
-   * Returns a string representation for this CacheBuilderSpec instance.
-   * The form of this representation is not guaranteed.
+   * Returns a string representation for this CacheBuilderSpec instance. The form of this
+   * representation is not guaranteed.
    */
   @Override
   public String toString() {
@@ -252,7 +251,7 @@ public final class CacheBuilderSpec {
   }
 
   @Override
-  public boolean equals(@Nullable Object obj) {
+  public boolean equals(@CheckForNull Object obj) {
     if (this == obj) {
       return true;
     }
@@ -267,19 +266,23 @@ public final class CacheBuilderSpec {
         && Objects.equal(keyStrength, that.keyStrength)
         && Objects.equal(valueStrength, that.valueStrength)
         && Objects.equal(recordStats, that.recordStats)
-        && Objects.equal(durationInNanos(writeExpirationDuration, writeExpirationTimeUnit),
+        && Objects.equal(
+            durationInNanos(writeExpirationDuration, writeExpirationTimeUnit),
             durationInNanos(that.writeExpirationDuration, that.writeExpirationTimeUnit))
-        && Objects.equal(durationInNanos(accessExpirationDuration, accessExpirationTimeUnit),
+        && Objects.equal(
+            durationInNanos(accessExpirationDuration, accessExpirationTimeUnit),
             durationInNanos(that.accessExpirationDuration, that.accessExpirationTimeUnit))
-        && Objects.equal(durationInNanos(refreshDuration, refreshTimeUnit),
+        && Objects.equal(
+            durationInNanos(refreshDuration, refreshTimeUnit),
             durationInNanos(that.refreshDuration, that.refreshTimeUnit));
   }
 
   /**
-   * Converts an expiration duration/unit pair into a single Long for hashing and equality.
-   * Uses nanos to match CacheBuilder implementation.
+   * Converts an expiration duration/unit pair into a single Long for hashing and equality. Uses
+   * nanos to match CacheBuilder implementation.
    */
-  @Nullable private static Long durationInNanos(long duration, @Nullable TimeUnit unit) {
+  @CheckForNull
+  private static Long durationInNanos(long duration, @CheckForNull TimeUnit unit) {
     return (unit == null) ? null : unit.toNanos(duration);
   }
 
@@ -288,8 +291,10 @@ public final class CacheBuilderSpec {
     protected abstract void parseInteger(CacheBuilderSpec spec, int value);
 
     @Override
-    public void parse(CacheBuilderSpec spec, String key, String value) {
-      checkArgument(value != null && !value.isEmpty(), "value of key %s omitted", key);
+    public void parse(CacheBuilderSpec spec, String key, @Nullable String value) {
+      if (isNullOrEmpty(value)) {
+        throw new IllegalArgumentException("value of key " + key + " omitted");
+      }
       try {
         parseInteger(spec, Integer.parseInt(value));
       } catch (NumberFormatException e) {
@@ -304,8 +309,10 @@ public final class CacheBuilderSpec {
     protected abstract void parseLong(CacheBuilderSpec spec, long value);
 
     @Override
-    public void parse(CacheBuilderSpec spec, String key, String value) {
-      checkArgument(value != null && !value.isEmpty(), "value of key %s omitted", key);
+    public void parse(CacheBuilderSpec spec, String key, @Nullable String value) {
+      if (isNullOrEmpty(value)) {
+        throw new IllegalArgumentException("value of key " + key + " omitted");
+      }
       try {
         parseLong(spec, Long.parseLong(value));
       } catch (NumberFormatException e) {
@@ -319,8 +326,10 @@ public final class CacheBuilderSpec {
   static class InitialCapacityParser extends IntegerParser {
     @Override
     protected void parseInteger(CacheBuilderSpec spec, int value) {
-      checkArgument(spec.initialCapacity == null,
-          "initial capacity was already set to ", spec.initialCapacity);
+      checkArgument(
+          spec.initialCapacity == null,
+          "initial capacity was already set to ",
+          spec.initialCapacity);
       spec.initialCapacity = value;
     }
   }
@@ -329,10 +338,9 @@ public final class CacheBuilderSpec {
   static class MaximumSizeParser extends LongParser {
     @Override
     protected void parseLong(CacheBuilderSpec spec, long value) {
-      checkArgument(spec.maximumSize == null,
-          "maximum size was already set to ", spec.maximumSize);
-      checkArgument(spec.maximumWeight == null,
-          "maximum weight was already set to ", spec.maximumWeight);
+      checkArgument(spec.maximumSize == null, "maximum size was already set to ", spec.maximumSize);
+      checkArgument(
+          spec.maximumWeight == null, "maximum weight was already set to ", spec.maximumWeight);
       spec.maximumSize = value;
     }
   }
@@ -341,10 +349,9 @@ public final class CacheBuilderSpec {
   static class MaximumWeightParser extends LongParser {
     @Override
     protected void parseLong(CacheBuilderSpec spec, long value) {
-      checkArgument(spec.maximumWeight == null,
-          "maximum weight was already set to ", spec.maximumWeight);
-      checkArgument(spec.maximumSize == null,
-          "maximum size was already set to ", spec.maximumSize);
+      checkArgument(
+          spec.maximumWeight == null, "maximum weight was already set to ", spec.maximumWeight);
+      checkArgument(spec.maximumSize == null, "maximum size was already set to ", spec.maximumSize);
       spec.maximumWeight = value;
     }
   }
@@ -353,8 +360,10 @@ public final class CacheBuilderSpec {
   static class ConcurrencyLevelParser extends IntegerParser {
     @Override
     protected void parseInteger(CacheBuilderSpec spec, int value) {
-      checkArgument(spec.concurrencyLevel == null,
-          "concurrency level was already set to ", spec.concurrencyLevel);
+      checkArgument(
+          spec.concurrencyLevel == null,
+          "concurrency level was already set to ",
+          spec.concurrencyLevel);
       spec.concurrencyLevel = value;
     }
   }
@@ -368,7 +377,7 @@ public final class CacheBuilderSpec {
     }
 
     @Override
-    public void parse(CacheBuilderSpec spec, String key, @Nullable String value) {
+    public void parse(CacheBuilderSpec spec, String key, @CheckForNull String value) {
       checkArgument(value == null, "key %s does not take values", key);
       checkArgument(spec.keyStrength == null, "%s was already set to %s", key, spec.keyStrength);
       spec.keyStrength = strength;
@@ -384,10 +393,10 @@ public final class CacheBuilderSpec {
     }
 
     @Override
-    public void parse(CacheBuilderSpec spec, String key, @Nullable String value) {
+    public void parse(CacheBuilderSpec spec, String key, @CheckForNull String value) {
       checkArgument(value == null, "key %s does not take values", key);
-      checkArgument(spec.valueStrength == null,
-        "%s was already set to %s", key, spec.valueStrength);
+      checkArgument(
+          spec.valueStrength == null, "%s was already set to %s", key, spec.valueStrength);
 
       spec.valueStrength = strength;
     }
@@ -397,7 +406,7 @@ public final class CacheBuilderSpec {
   static class RecordStatsParser implements ValueParser {
 
     @Override
-    public void parse(CacheBuilderSpec spec, String key, @Nullable String value) {
+    public void parse(CacheBuilderSpec spec, String key, @CheckForNull String value) {
       checkArgument(value == null, "recordStats does not take values");
       checkArgument(spec.recordStats == null, "recordStats already set");
       spec.recordStats = true;
@@ -406,14 +415,13 @@ public final class CacheBuilderSpec {
 
   /** Base class for parsing times with durations */
   abstract static class DurationParser implements ValueParser {
-    protected abstract void parseDuration(
-        CacheBuilderSpec spec,
-        long duration,
-        TimeUnit unit);
+    protected abstract void parseDuration(CacheBuilderSpec spec, long duration, TimeUnit unit);
 
     @Override
-    public void parse(CacheBuilderSpec spec, String key, String value) {
-      checkArgument(value != null && !value.isEmpty(), "value of key %s omitted", key);
+    public void parse(CacheBuilderSpec spec, String key, @CheckForNull String value) {
+      if (isNullOrEmpty(value)) {
+        throw new IllegalArgumentException("value of key " + key + " omitted");
+      }
       try {
         char lastChar = value.charAt(value.length() - 1);
         TimeUnit timeUnit;
@@ -432,8 +440,7 @@ public final class CacheBuilderSpec {
             break;
           default:
             throw new IllegalArgumentException(
-                format("key %s invalid format.  was %s, must end with one of [dDhHmMsS]",
-                    key, value));
+                format("key %s invalid unit: was %s, must end with one of [dhms]", key, value));
         }
 
         long duration = Long.parseLong(value.substring(0, value.length() - 1));
@@ -447,7 +454,8 @@ public final class CacheBuilderSpec {
 
   /** Parse expireAfterAccess */
   static class AccessDurationParser extends DurationParser {
-    @Override protected void parseDuration(CacheBuilderSpec spec, long duration, TimeUnit unit) {
+    @Override
+    protected void parseDuration(CacheBuilderSpec spec, long duration, TimeUnit unit) {
       checkArgument(spec.accessExpirationTimeUnit == null, "expireAfterAccess already set");
       spec.accessExpirationDuration = duration;
       spec.accessExpirationTimeUnit = unit;
@@ -456,7 +464,8 @@ public final class CacheBuilderSpec {
 
   /** Parse expireAfterWrite */
   static class WriteDurationParser extends DurationParser {
-    @Override protected void parseDuration(CacheBuilderSpec spec, long duration, TimeUnit unit) {
+    @Override
+    protected void parseDuration(CacheBuilderSpec spec, long duration, TimeUnit unit) {
       checkArgument(spec.writeExpirationTimeUnit == null, "expireAfterWrite already set");
       spec.writeExpirationDuration = duration;
       spec.writeExpirationTimeUnit = unit;
@@ -465,7 +474,8 @@ public final class CacheBuilderSpec {
 
   /** Parse refreshAfterWrite */
   static class RefreshDurationParser extends DurationParser {
-    @Override protected void parseDuration(CacheBuilderSpec spec, long duration, TimeUnit unit) {
+    @Override
+    protected void parseDuration(CacheBuilderSpec spec, long duration, TimeUnit unit) {
       checkArgument(spec.refreshTimeUnit == null, "refreshAfterWrite already set");
       spec.refreshDuration = duration;
       spec.refreshTimeUnit = unit;
