@@ -18,22 +18,15 @@ package com.google.common.collect;
 
 import static com.google.common.base.Preconditions.checkNotNull;
 import static com.google.common.collect.CollectPreconditions.checkNonnegative;
-import static com.google.common.collect.Maps.newLinkedHashMapWithExpectedSize;
 
-import com.google.common.annotations.Beta;
 import com.google.common.annotations.GwtCompatible;
 import com.google.common.base.Supplier;
-
 import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Comparator;
 import java.util.EnumMap;
 import java.util.EnumSet;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.LinkedHashMap;
-import java.util.LinkedHashSet;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
@@ -41,8 +34,7 @@ import java.util.Set;
 import java.util.SortedSet;
 import java.util.TreeMap;
 import java.util.TreeSet;
-
-import javax.annotation.CheckReturnValue;
+import org.checkerframework.checker.nullness.qual.Nullable;
 
 /**
  * A builder for a multimap implementation that allows customization of the backing map and value
@@ -51,28 +43,27 @@ import javax.annotation.CheckReturnValue;
  * <p>This can be used to easily configure multimap data structure implementations not provided
  * explicitly in {@code com.google.common.collect}, for example:
  *
- * <pre>   {@code
- *   ListMultimap<String, Integer> treeListMultimap =
- *       MultimapBuilder.treeKeys().arrayListValues().build();
- *   SetMultimap<Integer, MyEnum> hashEnumMultimap =
- *       MultimapBuilder.hashKeys().enumSetValues(MyEnum.class).build();}</pre>
+ * <pre>{@code
+ * ListMultimap<String, Integer> treeListMultimap =
+ *     MultimapBuilder.treeKeys().arrayListValues().build();
+ * SetMultimap<Integer, MyEnum> hashEnumMultimap =
+ *     MultimapBuilder.hashKeys().enumSetValues(MyEnum.class).build();
+ * }</pre>
  *
- * <p>{@code MultimapBuilder} instances are immutable.  Invoking a configuration method has no
- * effect on the receiving instance; you must store and use the new builder instance it returns
- * instead.
+ * <p>{@code MultimapBuilder} instances are immutable. Invoking a configuration method has no effect
+ * on the receiving instance; you must store and use the new builder instance it returns instead.
  *
- * <p>The generated multimaps are serializable if the key and value types are serializable,
- * unless stated otherwise in one of the configuration methods.
+ * <p>The generated multimaps are serializable if the key and value types are serializable, unless
+ * stated otherwise in one of the configuration methods.
  *
  * @author Louis Wasserman
  * @param <K0> An upper bound on the key type of the generated multimap.
  * @param <V0> An upper bound on the value type of the generated multimap.
  * @since 16.0
  */
-@Beta
 @GwtCompatible
-@CheckReturnValue
-public abstract class MultimapBuilder<K0, V0> {
+@ElementTypesAreNonnullByDefault
+public abstract class MultimapBuilder<K0 extends @Nullable Object, V0 extends @Nullable Object> {
   /*
    * Leaving K and V as upper bounds rather than the actual key and value types allows type
    * parameters to be left implicit more often. CacheBuilder uses the same technique.
@@ -82,56 +73,54 @@ public abstract class MultimapBuilder<K0, V0> {
 
   private static final int DEFAULT_EXPECTED_KEYS = 8;
 
-  /**
-   * Uses a {@link HashMap} to map keys to value collections.
-   */
-  public static MultimapBuilderWithKeys<Object> hashKeys() {
+  /** Uses a hash table to map keys to value collections. */
+  public static MultimapBuilderWithKeys<@Nullable Object> hashKeys() {
     return hashKeys(DEFAULT_EXPECTED_KEYS);
   }
 
   /**
-   * Uses a {@link HashMap} to map keys to value collections, initialized to expect the specified
-   * number of keys.
+   * Uses a hash table to map keys to value collections, initialized to expect the specified number
+   * of keys.
    *
    * @throws IllegalArgumentException if {@code expectedKeys < 0}
    */
-  public static MultimapBuilderWithKeys<Object> hashKeys(final int expectedKeys) {
+  public static MultimapBuilderWithKeys<@Nullable Object> hashKeys(int expectedKeys) {
     checkNonnegative(expectedKeys, "expectedKeys");
-    return new MultimapBuilderWithKeys<Object>() {
+    return new MultimapBuilderWithKeys<@Nullable Object>() {
       @Override
-      <K, V> Map<K, Collection<V>> createMap() {
-        return Maps.newHashMapWithExpectedSize(expectedKeys);
+      <K extends @Nullable Object, V extends @Nullable Object> Map<K, Collection<V>> createMap() {
+        return Platform.newHashMapWithExpectedSize(expectedKeys);
       }
     };
   }
 
   /**
-   * Uses a {@link LinkedHashMap} to map keys to value collections.
+   * Uses a hash table to map keys to value collections.
    *
-   * <p>The collections returned by {@link Multimap#keySet()}, {@link Multimap#keys()}, and
-   * {@link Multimap#asMap()} will iterate through the keys in the order that they were first added
-   * to the multimap, save that if all values associated with a key are removed and then the key is
-   * added back into the multimap, that key will come last in the key iteration order.
+   * <p>The collections returned by {@link Multimap#keySet()}, {@link Multimap#keys()}, and {@link
+   * Multimap#asMap()} will iterate through the keys in the order that they were first added to the
+   * multimap, save that if all values associated with a key are removed and then the key is added
+   * back into the multimap, that key will come last in the key iteration order.
    */
-  public static MultimapBuilderWithKeys<Object> linkedHashKeys() {
+  public static MultimapBuilderWithKeys<@Nullable Object> linkedHashKeys() {
     return linkedHashKeys(DEFAULT_EXPECTED_KEYS);
   }
 
   /**
-   * Uses a {@link LinkedHashMap} to map keys to value collections, initialized to expect the
-   * specified number of keys.
+   * Uses an hash table to map keys to value collections, initialized to expect the specified number
+   * of keys.
    *
-   * <p>The collections returned by {@link Multimap#keySet()}, {@link Multimap#keys()}, and
-   * {@link Multimap#asMap()} will iterate through the keys in the order that they were first added
-   * to the multimap, save that if all values associated with a key are removed and then the key is
-   * added back into the multimap, that key will come last in the key iteration order.
+   * <p>The collections returned by {@link Multimap#keySet()}, {@link Multimap#keys()}, and {@link
+   * Multimap#asMap()} will iterate through the keys in the order that they were first added to the
+   * multimap, save that if all values associated with a key are removed and then the key is added
+   * back into the multimap, that key will come last in the key iteration order.
    */
-  public static MultimapBuilderWithKeys<Object> linkedHashKeys(final int expectedKeys) {
+  public static MultimapBuilderWithKeys<@Nullable Object> linkedHashKeys(int expectedKeys) {
     checkNonnegative(expectedKeys, "expectedKeys");
-    return new MultimapBuilderWithKeys<Object>() {
+    return new MultimapBuilderWithKeys<@Nullable Object>() {
       @Override
-      <K, V> Map<K, Collection<V>> createMap() {
-        return newLinkedHashMapWithExpectedSize(expectedKeys);
+      <K extends @Nullable Object, V extends @Nullable Object> Map<K, Collection<V>> createMap() {
+        return Platform.newLinkedHashMapWithExpectedSize(expectedKeys);
       }
     };
   }
@@ -139,8 +128,8 @@ public abstract class MultimapBuilder<K0, V0> {
   /**
    * Uses a naturally-ordered {@link TreeMap} to map keys to value collections.
    *
-   * <p>The collections returned by {@link Multimap#keySet()}, {@link Multimap#keys()}, and
-   * {@link Multimap#asMap()} will iterate through the keys in sorted order.
+   * <p>The collections returned by {@link Multimap#keySet()}, {@link Multimap#keys()}, and {@link
+   * Multimap#asMap()} will iterate through the keys in sorted order.
    *
    * <p>For all multimaps generated by the resulting builder, the {@link Multimap#keySet()} can be
    * safely cast to a {@link java.util.SortedSet}, and the {@link Multimap#asMap()} can safely be
@@ -154,8 +143,8 @@ public abstract class MultimapBuilder<K0, V0> {
   /**
    * Uses a {@link TreeMap} sorted by the specified comparator to map keys to value collections.
    *
-   * <p>The collections returned by {@link Multimap#keySet()}, {@link Multimap#keys()}, and
-   * {@link Multimap#asMap()} will iterate through the keys in sorted order.
+   * <p>The collections returned by {@link Multimap#keySet()}, {@link Multimap#keys()}, and {@link
+   * Multimap#asMap()} will iterate through the keys in sorted order.
    *
    * <p>For all multimaps generated by the resulting builder, the {@link Multimap#keySet()} can be
    * safely cast to a {@link java.util.SortedSet}, and the {@link Multimap#asMap()} can safely be
@@ -164,26 +153,28 @@ public abstract class MultimapBuilder<K0, V0> {
    * <p>Multimaps generated by the resulting builder will not be serializable if {@code comparator}
    * is not serializable.
    */
-  public static <K0> MultimapBuilderWithKeys<K0> treeKeys(final Comparator<K0> comparator) {
+  public static <K0 extends @Nullable Object> MultimapBuilderWithKeys<K0> treeKeys(
+      Comparator<K0> comparator) {
     checkNotNull(comparator);
     return new MultimapBuilderWithKeys<K0>() {
       @Override
-      <K extends K0, V> Map<K, Collection<V>> createMap() {
-        return new TreeMap<K, Collection<V>>(comparator);
+      <K extends K0, V extends @Nullable Object> Map<K, Collection<V>> createMap() {
+        return new TreeMap<>(comparator);
       }
     };
   }
 
   /**
    * Uses an {@link EnumMap} to map keys to value collections.
+   *
+   * @since 16.0
    */
-  public static <K0 extends Enum<K0>> MultimapBuilderWithKeys<K0> enumKeys(
-      final Class<K0> keyClass) {
+  public static <K0 extends Enum<K0>> MultimapBuilderWithKeys<K0> enumKeys(Class<K0> keyClass) {
     checkNotNull(keyClass);
     return new MultimapBuilderWithKeys<K0>() {
       @SuppressWarnings("unchecked")
       @Override
-      <K extends K0, V> Map<K, Collection<V>> createMap() {
+      <K extends K0, V extends @Nullable Object> Map<K, Collection<V>> createMap() {
         // K must actually be K0, since enums are effectively final
         // (their subclasses are inaccessible)
         return (Map<K, Collection<V>>) new EnumMap<K0, Collection<V>>(keyClass);
@@ -191,7 +182,8 @@ public abstract class MultimapBuilder<K0, V0> {
     };
   }
 
-  private static final class ArrayListSupplier<V> implements Supplier<List<V>>, Serializable {
+  private static final class ArrayListSupplier<V extends @Nullable Object>
+      implements Supplier<List<V>>, Serializable {
     private final int expectedValuesPerKey;
 
     ArrayListSupplier(int expectedValuesPerKey) {
@@ -200,14 +192,14 @@ public abstract class MultimapBuilder<K0, V0> {
 
     @Override
     public List<V> get() {
-      return new ArrayList<V>(expectedValuesPerKey);
+      return new ArrayList<>(expectedValuesPerKey);
     }
   }
 
-  private enum LinkedListSupplier implements Supplier<List<Object>> {
+  private enum LinkedListSupplier implements Supplier<List<?>> {
     INSTANCE;
 
-    public static <V> Supplier<List<V>> instance() {
+    public static <V extends @Nullable Object> Supplier<List<V>> instance() {
       // Each call generates a fresh LinkedList, which can serve as a List<V> for any V.
       @SuppressWarnings({"rawtypes", "unchecked"})
       Supplier<List<V>> result = (Supplier) INSTANCE;
@@ -215,12 +207,13 @@ public abstract class MultimapBuilder<K0, V0> {
     }
 
     @Override
-    public List<Object> get() {
-      return new LinkedList<Object>();
+    public List<?> get() {
+      return new LinkedList<>();
     }
   }
 
-  private static final class HashSetSupplier<V> implements Supplier<Set<V>>, Serializable {
+  private static final class HashSetSupplier<V extends @Nullable Object>
+      implements Supplier<Set<V>>, Serializable {
     private final int expectedValuesPerKey;
 
     HashSetSupplier(int expectedValuesPerKey) {
@@ -229,11 +222,12 @@ public abstract class MultimapBuilder<K0, V0> {
 
     @Override
     public Set<V> get() {
-      return Sets.newHashSetWithExpectedSize(expectedValuesPerKey);
+      return Platform.newHashSetWithExpectedSize(expectedValuesPerKey);
     }
   }
 
-  private static final class LinkedHashSetSupplier<V> implements Supplier<Set<V>>, Serializable {
+  private static final class LinkedHashSetSupplier<V extends @Nullable Object>
+      implements Supplier<Set<V>>, Serializable {
     private final int expectedValuesPerKey;
 
     LinkedHashSetSupplier(int expectedValuesPerKey) {
@@ -242,11 +236,12 @@ public abstract class MultimapBuilder<K0, V0> {
 
     @Override
     public Set<V> get() {
-      return Sets.newLinkedHashSetWithExpectedSize(expectedValuesPerKey);
+      return Platform.newLinkedHashSetWithExpectedSize(expectedValuesPerKey);
     }
   }
 
-  private static final class TreeSetSupplier<V> implements Supplier<SortedSet<V>>, Serializable {
+  private static final class TreeSetSupplier<V extends @Nullable Object>
+      implements Supplier<SortedSet<V>>, Serializable {
     private final Comparator<? super V> comparator;
 
     TreeSetSupplier(Comparator<? super V> comparator) {
@@ -255,7 +250,7 @@ public abstract class MultimapBuilder<K0, V0> {
 
     @Override
     public SortedSet<V> get() {
-      return new TreeSet<V>(comparator);
+      return new TreeSet<>(comparator);
     }
   }
 
@@ -278,19 +273,18 @@ public abstract class MultimapBuilder<K0, V0> {
    * implementation has been specified, but the value collection implementation has not.
    *
    * @param <K0> The upper bound on the key type of the generated multimap.
+   * @since 16.0
    */
-  public abstract static class MultimapBuilderWithKeys<K0> {
+  public abstract static class MultimapBuilderWithKeys<K0 extends @Nullable Object> {
 
     private static final int DEFAULT_EXPECTED_VALUES_PER_KEY = 2;
 
     MultimapBuilderWithKeys() {}
 
-    abstract <K extends K0, V> Map<K, Collection<V>> createMap();
+    abstract <K extends K0, V extends @Nullable Object> Map<K, Collection<V>> createMap();
 
-    /**
-     * Uses an {@link ArrayList} to store value collections.
-     */
-    public ListMultimapBuilder<K0, Object> arrayListValues() {
+    /** Uses an {@link ArrayList} to store value collections. */
+    public ListMultimapBuilder<K0, @Nullable Object> arrayListValues() {
       return arrayListValues(DEFAULT_EXPECTED_VALUES_PER_KEY);
     }
 
@@ -300,11 +294,11 @@ public abstract class MultimapBuilder<K0, V0> {
      *
      * @throws IllegalArgumentException if {@code expectedValuesPerKey < 0}
      */
-    public ListMultimapBuilder<K0, Object> arrayListValues(final int expectedValuesPerKey) {
+    public ListMultimapBuilder<K0, @Nullable Object> arrayListValues(int expectedValuesPerKey) {
       checkNonnegative(expectedValuesPerKey, "expectedValuesPerKey");
-      return new ListMultimapBuilder<K0, Object>() {
+      return new ListMultimapBuilder<K0, @Nullable Object>() {
         @Override
-        public <K extends K0, V> ListMultimap<K, V> build() {
+        public <K extends K0, V extends @Nullable Object> ListMultimap<K, V> build() {
           return Multimaps.newListMultimap(
               MultimapBuilderWithKeys.this.<K, V>createMap(),
               new ArrayListSupplier<V>(expectedValuesPerKey));
@@ -312,37 +306,33 @@ public abstract class MultimapBuilder<K0, V0> {
       };
     }
 
-    /**
-     * Uses a {@link LinkedList} to store value collections.
-     */
-    public ListMultimapBuilder<K0, Object> linkedListValues() {
-      return new ListMultimapBuilder<K0, Object>() {
+    /** Uses a {@link LinkedList} to store value collections. */
+    public ListMultimapBuilder<K0, @Nullable Object> linkedListValues() {
+      return new ListMultimapBuilder<K0, @Nullable Object>() {
         @Override
-        public <K extends K0, V> ListMultimap<K, V> build() {
+        public <K extends K0, V extends @Nullable Object> ListMultimap<K, V> build() {
           return Multimaps.newListMultimap(
               MultimapBuilderWithKeys.this.<K, V>createMap(), LinkedListSupplier.<V>instance());
         }
       };
     }
 
-    /**
-     * Uses a {@link HashSet} to store value collections.
-     */
-    public SetMultimapBuilder<K0, Object> hashSetValues() {
+    /** Uses a hash-based {@code Set} to store value collections. */
+    public SetMultimapBuilder<K0, @Nullable Object> hashSetValues() {
       return hashSetValues(DEFAULT_EXPECTED_VALUES_PER_KEY);
     }
 
     /**
-     * Uses a {@link HashSet} to store value collections, initialized to expect the specified number
-     * of values per key.
+     * Uses a hash-based {@code Set} to store value collections, initialized to expect the specified
+     * number of values per key.
      *
      * @throws IllegalArgumentException if {@code expectedValuesPerKey < 0}
      */
-    public SetMultimapBuilder<K0, Object> hashSetValues(final int expectedValuesPerKey) {
+    public SetMultimapBuilder<K0, @Nullable Object> hashSetValues(int expectedValuesPerKey) {
       checkNonnegative(expectedValuesPerKey, "expectedValuesPerKey");
-      return new SetMultimapBuilder<K0, Object>() {
+      return new SetMultimapBuilder<K0, @Nullable Object>() {
         @Override
-        public <K extends K0, V> SetMultimap<K, V> build() {
+        public <K extends K0, V extends @Nullable Object> SetMultimap<K, V> build() {
           return Multimaps.newSetMultimap(
               MultimapBuilderWithKeys.this.<K, V>createMap(),
               new HashSetSupplier<V>(expectedValuesPerKey));
@@ -350,24 +340,22 @@ public abstract class MultimapBuilder<K0, V0> {
       };
     }
 
-    /**
-     * Uses a {@link LinkedHashSet} to store value collections.
-     */
-    public SetMultimapBuilder<K0, Object> linkedHashSetValues() {
+    /** Uses an insertion-ordered hash-based {@code Set} to store value collections. */
+    public SetMultimapBuilder<K0, @Nullable Object> linkedHashSetValues() {
       return linkedHashSetValues(DEFAULT_EXPECTED_VALUES_PER_KEY);
     }
 
     /**
-     * Uses a {@link LinkedHashSet} to store value collections, initialized to expect the specified
-     * number of values per key.
+     * Uses an insertion-ordered hash-based {@code Set} to store value collections, initialized to
+     * expect the specified number of values per key.
      *
      * @throws IllegalArgumentException if {@code expectedValuesPerKey < 0}
      */
-    public SetMultimapBuilder<K0, Object> linkedHashSetValues(final int expectedValuesPerKey) {
+    public SetMultimapBuilder<K0, @Nullable Object> linkedHashSetValues(int expectedValuesPerKey) {
       checkNonnegative(expectedValuesPerKey, "expectedValuesPerKey");
-      return new SetMultimapBuilder<K0, Object>() {
+      return new SetMultimapBuilder<K0, @Nullable Object>() {
         @Override
-        public <K extends K0, V> SetMultimap<K, V> build() {
+        public <K extends K0, V extends @Nullable Object> SetMultimap<K, V> build() {
           return Multimaps.newSetMultimap(
               MultimapBuilderWithKeys.this.<K, V>createMap(),
               new LinkedHashSetSupplier<V>(expectedValuesPerKey));
@@ -375,9 +363,7 @@ public abstract class MultimapBuilder<K0, V0> {
       };
     }
 
-    /**
-     * Uses a naturally-ordered {@link TreeSet} to store value collections.
-     */
+    /** Uses a naturally-ordered {@link TreeSet} to store value collections. */
     @SuppressWarnings("rawtypes")
     public SortedSetMultimapBuilder<K0, Comparable> treeSetValues() {
       return treeSetValues(Ordering.natural());
@@ -386,10 +372,11 @@ public abstract class MultimapBuilder<K0, V0> {
     /**
      * Uses a {@link TreeSet} ordered by the specified comparator to store value collections.
      *
-     * <p>Multimaps generated by the resulting builder will not be serializable if
-     * {@code comparator} is not serializable.
+     * <p>Multimaps generated by the resulting builder will not be serializable if {@code
+     * comparator} is not serializable.
      */
-    public <V0> SortedSetMultimapBuilder<K0, V0> treeSetValues(final Comparator<V0> comparator) {
+    public <V0 extends @Nullable Object> SortedSetMultimapBuilder<K0, V0> treeSetValues(
+        Comparator<V0> comparator) {
       checkNotNull(comparator, "comparator");
       return new SortedSetMultimapBuilder<K0, V0>() {
         @Override
@@ -400,11 +387,8 @@ public abstract class MultimapBuilder<K0, V0> {
       };
     }
 
-    /**
-     * Uses an {@link EnumSet} to store value collections.
-     */
-    public <V0 extends Enum<V0>> SetMultimapBuilder<K0, V0> enumSetValues(
-        final Class<V0> valueClass) {
+    /** Uses an {@link EnumSet} to store value collections. */
+    public <V0 extends Enum<V0>> SetMultimapBuilder<K0, V0> enumSetValues(Class<V0> valueClass) {
       checkNotNull(valueClass, "valueClass");
       return new SetMultimapBuilder<K0, V0>() {
         @Override
@@ -419,9 +403,7 @@ public abstract class MultimapBuilder<K0, V0> {
     }
   }
 
-  /**
-   * Returns a new, empty {@code Multimap} with the specified implementation.
-   */
+  /** Returns a new, empty {@code Multimap} with the specified implementation. */
   public abstract <K extends K0, V extends V0> Multimap<K, V> build();
 
   /**
@@ -437,8 +419,12 @@ public abstract class MultimapBuilder<K0, V0> {
 
   /**
    * A specialization of {@link MultimapBuilder} that generates {@link ListMultimap} instances.
+   *
+   * @since 16.0
    */
-  public abstract static class ListMultimapBuilder<K0, V0> extends MultimapBuilder<K0, V0> {
+  public abstract static class ListMultimapBuilder<
+          K0 extends @Nullable Object, V0 extends @Nullable Object>
+      extends MultimapBuilder<K0, V0> {
     ListMultimapBuilder() {}
 
     @Override
@@ -453,8 +439,12 @@ public abstract class MultimapBuilder<K0, V0> {
 
   /**
    * A specialization of {@link MultimapBuilder} that generates {@link SetMultimap} instances.
+   *
+   * @since 16.0
    */
-  public abstract static class SetMultimapBuilder<K0, V0> extends MultimapBuilder<K0, V0> {
+  public abstract static class SetMultimapBuilder<
+          K0 extends @Nullable Object, V0 extends @Nullable Object>
+      extends MultimapBuilder<K0, V0> {
     SetMultimapBuilder() {}
 
     @Override
@@ -469,8 +459,12 @@ public abstract class MultimapBuilder<K0, V0> {
 
   /**
    * A specialization of {@link MultimapBuilder} that generates {@link SortedSetMultimap} instances.
+   *
+   * @since 16.0
    */
-  public abstract static class SortedSetMultimapBuilder<K0, V0> extends SetMultimapBuilder<K0, V0> {
+  public abstract static class SortedSetMultimapBuilder<
+          K0 extends @Nullable Object, V0 extends @Nullable Object>
+      extends SetMultimapBuilder<K0, V0> {
     SortedSetMultimapBuilder() {}
 
     @Override
