@@ -23,6 +23,8 @@ import static com.google.common.base.Predicates.in;
 import static com.google.common.base.Predicates.not;
 import static com.google.common.collect.Maps.safeContainsKey;
 import static com.google.common.collect.Maps.safeGet;
+import static com.google.common.collect.NullnessCasts.uncheckedCastNullableTToT;
+import static java.util.Objects.requireNonNull;
 
 import com.google.common.annotations.GwtCompatible;
 import com.google.common.base.Function;
@@ -31,8 +33,8 @@ import com.google.common.base.Supplier;
 import com.google.common.collect.Maps.IteratorBasedAbstractMap;
 import com.google.common.collect.Maps.ViewCachingAbstractMap;
 import com.google.common.collect.Sets.ImprovedAbstractSet;
+import com.google.errorprone.annotations.CanIgnoreReturnValue;
 import com.google.j2objc.annotations.WeakOuter;
-
 import java.io.Serializable;
 import java.util.Collection;
 import java.util.Iterator;
@@ -40,32 +42,31 @@ import java.util.LinkedHashMap;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Set;
-
-import javax.annotation.Nullable;
+import java.util.Spliterator;
+import java.util.Spliterators;
+import javax.annotation.CheckForNull;
 
 /**
- * {@link Table} implementation backed by a map that associates row keys with
- * column key / value secondary maps. This class provides rapid access to
- * records by the row key alone or by both keys, but not by just the column key.
+ * {@link Table} implementation backed by a map that associates row keys with column key / value
+ * secondary maps. This class provides rapid access to records by the row key alone or by both keys,
+ * but not by just the column key.
  *
- * <p>The views returned by {@link #column}, {@link #columnKeySet()}, and {@link
- * #columnMap()} have iterators that don't support {@code remove()}. Otherwise,
- * all optional operations are supported. Null row keys, columns keys, and
- * values are not supported.
+ * <p>The views returned by {@link #column}, {@link #columnKeySet()}, and {@link #columnMap()} have
+ * iterators that don't support {@code remove()}. Otherwise, all optional operations are supported.
+ * Null row keys, columns keys, and values are not supported.
  *
- * <p>Lookups by row key are often faster than lookups by column key, because
- * the data is stored in a {@code Map<R, Map<C, V>>}. A method call like {@code
- * column(columnKey).get(rowKey)} still runs quickly, since the row key is
- * provided. However, {@code column(columnKey).size()} takes longer, since an
- * iteration across all row keys occurs.
+ * <p>Lookups by row key are often faster than lookups by column key, because the data is stored in
+ * a {@code Map<R, Map<C, V>>}. A method call like {@code column(columnKey).get(rowKey)} still runs
+ * quickly, since the row key is provided. However, {@code column(columnKey).size()} takes longer,
+ * since an iteration across all row keys occurs.
  *
- * <p>Note that this implementation is not synchronized. If multiple threads
- * access this table concurrently and one of the threads modifies the table, it
- * must be synchronized externally.
+ * <p>Note that this implementation is not synchronized. If multiple threads access this table
+ * concurrently and one of the threads modifies the table, it must be synchronized externally.
  *
  * @author Jared Levy
  */
 @GwtCompatible
+@ElementTypesAreNonnullByDefault
 class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializable {
   @GwtTransient final Map<R, Map<C, V>> backingMap;
   @GwtTransient final Supplier<? extends Map<C, V>> factory;
@@ -78,12 +79,12 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
   // Accessors
 
   @Override
-  public boolean contains(@Nullable Object rowKey, @Nullable Object columnKey) {
+  public boolean contains(@CheckForNull Object rowKey, @CheckForNull Object columnKey) {
     return rowKey != null && columnKey != null && super.contains(rowKey, columnKey);
   }
 
   @Override
-  public boolean containsColumn(@Nullable Object columnKey) {
+  public boolean containsColumn(@CheckForNull Object columnKey) {
     if (columnKey == null) {
       return false;
     }
@@ -96,17 +97,18 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
   }
 
   @Override
-  public boolean containsRow(@Nullable Object rowKey) {
+  public boolean containsRow(@CheckForNull Object rowKey) {
     return rowKey != null && safeContainsKey(backingMap, rowKey);
   }
 
   @Override
-  public boolean containsValue(@Nullable Object value) {
+  public boolean containsValue(@CheckForNull Object value) {
     return value != null && super.containsValue(value);
   }
 
   @Override
-  public V get(@Nullable Object rowKey, @Nullable Object columnKey) {
+  @CheckForNull
+  public V get(@CheckForNull Object rowKey, @CheckForNull Object columnKey) {
     return (rowKey == null || columnKey == null) ? null : super.get(rowKey, columnKey);
   }
 
@@ -140,7 +142,9 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
     return map;
   }
 
+  @CanIgnoreReturnValue
   @Override
+  @CheckForNull
   public V put(R rowKey, C columnKey, V value) {
     checkNotNull(rowKey);
     checkNotNull(columnKey);
@@ -148,8 +152,10 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
     return getOrCreate(rowKey).put(columnKey, value);
   }
 
+  @CanIgnoreReturnValue
   @Override
-  public V remove(@Nullable Object rowKey, @Nullable Object columnKey) {
+  @CheckForNull
+  public V remove(@CheckForNull Object rowKey, @CheckForNull Object columnKey) {
     if ((rowKey == null) || (columnKey == null)) {
       return null;
     }
@@ -164,8 +170,9 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
     return value;
   }
 
-  private Map<R, V> removeColumn(Object column) {
-    Map<R, V> output = new LinkedHashMap<R, V>();
+  @CanIgnoreReturnValue
+  private Map<R, V> removeColumn(@CheckForNull Object column) {
+    Map<R, V> output = new LinkedHashMap<>();
     Iterator<Entry<R, Map<C, V>>> iterator = backingMap.entrySet().iterator();
     while (iterator.hasNext()) {
       Entry<R, Map<C, V>> entry = iterator.next();
@@ -180,12 +187,14 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
     return output;
   }
 
-  private boolean containsMapping(Object rowKey, Object columnKey, Object value) {
+  private boolean containsMapping(
+      @CheckForNull Object rowKey, @CheckForNull Object columnKey, @CheckForNull Object value) {
     return value != null && value.equals(get(rowKey, columnKey));
   }
 
   /** Remove a row key / column key / value mapping, if present. */
-  private boolean removeMapping(Object rowKey, Object columnKey, Object value) {
+  private boolean removeMapping(
+      @CheckForNull Object rowKey, @CheckForNull Object columnKey, @CheckForNull Object value) {
     if (containsMapping(rowKey, columnKey, value)) {
       remove(rowKey, columnKey);
       return true;
@@ -196,8 +205,8 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
   // Views
 
   /**
-   * Abstract set whose {@code isEmpty()} returns whether the table is empty and
-   * whose {@code clear()} clears all table mappings.
+   * Abstract set whose {@code isEmpty()} returns whether the table is empty and whose {@code
+   * clear()} clears all table mappings.
    */
   @WeakOuter
   private abstract class TableSet<T> extends ImprovedAbstractSet<T> {
@@ -215,12 +224,11 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
   /**
    * {@inheritDoc}
    *
-   * <p>The set's iterator traverses the mappings for the first row, the
-   * mappings for the second row, and so on.
+   * <p>The set's iterator traverses the mappings for the first row, the mappings for the second
+   * row, and so on.
    *
-   * <p>Each cell is an immutable snapshot of a row key / column key / value
-   * mapping, taken at the time the cell is returned by a method call to the
-   * set or its iterator.
+   * <p>Each cell is an immutable snapshot of a row key / column key / value mapping, taken at the
+   * time the cell is returned by a method call to the set or its iterator.
    */
   @Override
   public Set<Cell<R, C, V>> cellSet() {
@@ -234,7 +242,7 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
 
   private class CellIterator implements Iterator<Cell<R, C, V>> {
     final Iterator<Entry<R, Map<C, V>>> rowIterator = backingMap.entrySet().iterator();
-    Entry<R, Map<C, V>> rowEntry;
+    @CheckForNull Entry<R, Map<C, V>> rowEntry;
     Iterator<Entry<C, V>> columnIterator = Iterators.emptyModifiableIterator();
 
     @Override
@@ -248,6 +256,20 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
         rowEntry = rowIterator.next();
         columnIterator = rowEntry.getValue().entrySet().iterator();
       }
+      /*
+       * requireNonNull is safe because:
+       *
+       * - columnIterator started off pointing to an empty iterator, so we must have entered the
+       *   `if` body above at least once. Thus, if we got this far, that `if` body initialized
+       *   rowEntry at least once.
+       *
+       * - The only case in which rowEntry is cleared (during remove() below) happens only if the
+       *   caller removed every element from columnIterator. During that process, we would have had
+       *   to iterate it to exhaustion. Then we can apply the logic above about an empty
+       *   columnIterator. (This assumes no concurrent modification, but behavior under concurrent
+       *   modification is undefined, anyway.)
+       */
+      requireNonNull(rowEntry);
       Entry<C, V> columnEntry = columnIterator.next();
       return Tables.immutableCell(rowEntry.getKey(), columnEntry.getKey(), columnEntry.getValue());
     }
@@ -255,12 +277,37 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
     @Override
     public void remove() {
       columnIterator.remove();
-      if (rowEntry.getValue().isEmpty()) {
+      /*
+       * requireNonNull is safe because:
+       *
+       * - columnIterator.remove() succeeded, so it must have returned a value, so it must have been
+       * initialized by next() -- which initializes rowEntry, too.
+       *
+       * - rowEntry isn't cleared except below. If it was cleared below, then either
+       *   columnIterator.remove() would have failed above (if the user hasn't called next() since
+       *   then) or rowEntry would have been initialized by next() (as discussed above).
+       */
+      if (requireNonNull(rowEntry).getValue().isEmpty()) {
         rowIterator.remove();
+        rowEntry = null;
       }
     }
   }
 
+  @Override
+  Spliterator<Cell<R, C, V>> cellSpliterator() {
+    return CollectSpliterators.flatMap(
+        backingMap.entrySet().spliterator(),
+        (Entry<R, Map<C, V>> rowEntry) ->
+            CollectSpliterators.map(
+                rowEntry.getValue().entrySet().spliterator(),
+                (Entry<C, V> columnEntry) ->
+                    Tables.immutableCell(
+                        rowEntry.getKey(), columnEntry.getKey(), columnEntry.getValue())),
+        Spliterator.DISTINCT | Spliterator.SIZED,
+        size());
+  }
+
   @Override
   public Map<C, V> row(R rowKey) {
     return new Row(rowKey);
@@ -273,39 +320,43 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
       this.rowKey = checkNotNull(rowKey);
     }
 
-    Map<C, V> backingRowMap;
+    @CheckForNull Map<C, V> backingRowMap;
 
-    Map<C, V> backingRowMap() {
-      return (backingRowMap == null || (backingRowMap.isEmpty() && backingMap.containsKey(rowKey)))
-          ? backingRowMap = computeBackingRowMap()
-          : backingRowMap;
+    final void updateBackingRowMapField() {
+      if (backingRowMap == null || (backingRowMap.isEmpty() && backingMap.containsKey(rowKey))) {
+        backingRowMap = computeBackingRowMap();
+      }
     }
 
+    @CheckForNull
     Map<C, V> computeBackingRowMap() {
       return backingMap.get(rowKey);
     }
 
     // Call this every time we perform a removal.
     void maintainEmptyInvariant() {
-      if (backingRowMap() != null && backingRowMap.isEmpty()) {
+      updateBackingRowMapField();
+      if (backingRowMap != null && backingRowMap.isEmpty()) {
         backingMap.remove(rowKey);
         backingRowMap = null;
       }
     }
 
     @Override
-    public boolean containsKey(Object key) {
-      Map<C, V> backingRowMap = backingRowMap();
+    public boolean containsKey(@CheckForNull Object key) {
+      updateBackingRowMapField();
       return (key != null && backingRowMap != null) && Maps.safeContainsKey(backingRowMap, key);
     }
 
     @Override
-    public V get(Object key) {
-      Map<C, V> backingRowMap = backingRowMap();
+    @CheckForNull
+    public V get(@CheckForNull Object key) {
+      updateBackingRowMapField();
       return (key != null && backingRowMap != null) ? Maps.safeGet(backingRowMap, key) : null;
     }
 
     @Override
+    @CheckForNull
     public V put(C key, V value) {
       checkNotNull(key);
       checkNotNull(value);
@@ -316,8 +367,9 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
     }
 
     @Override
-    public V remove(Object key) {
-      Map<C, V> backingRowMap = backingRowMap();
+    @CheckForNull
+    public V remove(@CheckForNull Object key) {
+      updateBackingRowMapField();
       if (backingRowMap == null) {
         return null;
       }
@@ -328,7 +380,7 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
 
     @Override
     public void clear() {
-      Map<C, V> backingRowMap = backingRowMap();
+      updateBackingRowMapField();
       if (backingRowMap != null) {
         backingRowMap.clear();
       }
@@ -337,17 +389,17 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
 
     @Override
     public int size() {
-      Map<C, V> map = backingRowMap();
-      return (map == null) ? 0 : map.size();
+      updateBackingRowMapField();
+      return (backingRowMap == null) ? 0 : backingRowMap.size();
     }
 
     @Override
     Iterator<Entry<C, V>> entryIterator() {
-      final Map<C, V> map = backingRowMap();
-      if (map == null) {
+      updateBackingRowMapField();
+      if (backingRowMap == null) {
         return Iterators.emptyModifiableIterator();
       }
-      final Iterator<Entry<C, V>> iterator = map.entrySet().iterator();
+      final Iterator<Entry<C, V>> iterator = backingRowMap.entrySet().iterator();
       return new Iterator<Entry<C, V>>() {
         @Override
         public boolean hasNext() {
@@ -356,24 +408,7 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
 
         @Override
         public Entry<C, V> next() {
-          final Entry<C, V> entry = iterator.next();
-          return new ForwardingMapEntry<C, V>() {
-            @Override
-            protected Entry<C, V> delegate() {
-              return entry;
-            }
-
-            @Override
-            public V setValue(V value) {
-              return super.setValue(checkNotNull(value));
-            }
-
-            @Override
-            public boolean equals(Object object) {
-              // TODO(lowasser): identify why this affects GWT tests
-              return standardEquals(object);
-            }
-          };
+          return wrapEntry(iterator.next());
         }
 
         @Override
@@ -383,13 +418,41 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
         }
       };
     }
+
+    @Override
+    Spliterator<Entry<C, V>> entrySpliterator() {
+      updateBackingRowMapField();
+      if (backingRowMap == null) {
+        return Spliterators.emptySpliterator();
+      }
+      return CollectSpliterators.map(backingRowMap.entrySet().spliterator(), this::wrapEntry);
+    }
+
+    Entry<C, V> wrapEntry(final Entry<C, V> entry) {
+      return new ForwardingMapEntry<C, V>() {
+        @Override
+        protected Entry<C, V> delegate() {
+          return entry;
+        }
+
+        @Override
+        public V setValue(V value) {
+          return super.setValue(checkNotNull(value));
+        }
+
+        @Override
+        public boolean equals(@CheckForNull Object object) {
+          // TODO(lowasser): identify why this affects GWT tests
+          return standardEquals(object);
+        }
+      };
+    }
   }
 
   /**
    * {@inheritDoc}
    *
-   * <p>The returned map's views have iterators that don't support
-   * {@code remove()}.
+   * <p>The returned map's views have iterators that don't support {@code remove()}.
    */
   @Override
   public Map<R, V> column(C columnKey) {
@@ -404,29 +467,30 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
     }
 
     @Override
+    @CheckForNull
     public V put(R key, V value) {
       return StandardTable.this.put(key, columnKey, value);
     }
 
     @Override
-    public V get(Object key) {
+    @CheckForNull
+    public V get(@CheckForNull Object key) {
       return StandardTable.this.get(key, columnKey);
     }
 
     @Override
-    public boolean containsKey(Object key) {
+    public boolean containsKey(@CheckForNull Object key) {
       return StandardTable.this.contains(key, columnKey);
     }
 
     @Override
-    public V remove(Object key) {
+    @CheckForNull
+    public V remove(@CheckForNull Object key) {
       return StandardTable.this.remove(key, columnKey);
     }
 
-    /**
-     * Removes all {@code Column} mappings whose row key and value satisfy the
-     * given predicate.
-     */
+    /** Removes all {@code Column} mappings whose row key and value satisfy the given predicate. */
+    @CanIgnoreReturnValue
     boolean removeFromColumnIf(Predicate<? super Entry<R, V>> predicate) {
       boolean changed = false;
       Iterator<Entry<R, Map<C, V>>> iterator = backingMap.entrySet().iterator();
@@ -479,7 +543,7 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
       }
 
       @Override
-      public boolean contains(Object o) {
+      public boolean contains(@CheckForNull Object o) {
         if (o instanceof Entry) {
           Entry<?, ?> entry = (Entry<?, ?>) o;
           return containsMapping(entry.getKey(), columnKey, entry.getValue());
@@ -488,7 +552,7 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
       }
 
       @Override
-      public boolean remove(Object obj) {
+      public boolean remove(@CheckForNull Object obj) {
         if (obj instanceof Entry) {
           Entry<?, ?> entry = (Entry<?, ?>) obj;
           return removeMapping(entry.getKey(), columnKey, entry.getValue());
@@ -506,6 +570,7 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
       final Iterator<Entry<R, Map<C, V>>> iterator = backingMap.entrySet().iterator();
 
       @Override
+      @CheckForNull
       protected Entry<R, V> computeNext() {
         while (iterator.hasNext()) {
           final Entry<R, Map<C, V>> entry = iterator.next();
@@ -524,7 +589,22 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
 
               @Override
               public V setValue(V value) {
-                return entry.getValue().put(columnKey, checkNotNull(value));
+                /*
+                 * The cast is safe because of the containsKey check above. (Well, it's possible for
+                 * the map to change between that call and this one. But if that happens, the
+                 * behavior is undefined because of the concurrent mutation.)
+                 *
+                 * (Our prototype checker happens to be "smart" enough to understand this for the
+                 * *get* call in getValue but not for the *put* call here.)
+                 *
+                 * (Arguably we should use requireNonNull rather than uncheckedCastNullableTToT: We
+                 * know that V is a non-null type because that's the only kind of value type that
+                 * StandardTable supports. Thus, requireNonNull is safe as long as the cell is still
+                 * present. (And if it's not present, behavior is undefined.) However, that's a
+                 * behavior change relative to the old code, so it didn't seem worth risking.)
+                 */
+                return uncheckedCastNullableTToT(
+                    entry.getValue().put(columnKey, checkNotNull(value)));
               }
             }
             return new EntryImpl();
@@ -546,12 +626,12 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
       }
 
       @Override
-      public boolean contains(Object obj) {
+      public boolean contains(@CheckForNull Object obj) {
         return StandardTable.this.contains(obj, columnKey);
       }
 
       @Override
-      public boolean remove(Object obj) {
+      public boolean remove(@CheckForNull Object obj) {
         return StandardTable.this.remove(obj, columnKey) != null;
       }
 
@@ -573,7 +653,7 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
       }
 
       @Override
-      public boolean remove(Object obj) {
+      public boolean remove(@CheckForNull Object obj) {
         return obj != null && removeFromColumnIf(Maps.<V>valuePredicateOnEntries(equalTo(obj)));
       }
 
@@ -594,16 +674,15 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
     return rowMap().keySet();
   }
 
-  private transient Set<C> columnKeySet;
+  @CheckForNull private transient Set<C> columnKeySet;
 
   /**
    * {@inheritDoc}
    *
    * <p>The returned set has an iterator that does not support {@code remove()}.
    *
-   * <p>The set's iterator traverses the columns of the first row, the
-   * columns of the second row, etc., skipping any columns that have
-   * appeared previously.
+   * <p>The set's iterator traverses the columns of the first row, the columns of the second row,
+   * etc., skipping any columns that have appeared previously.
    */
   @Override
   public Set<C> columnKeySet() {
@@ -624,7 +703,7 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
     }
 
     @Override
-    public boolean remove(Object obj) {
+    public boolean remove(@CheckForNull Object obj) {
       if (obj == null) {
         return false;
       }
@@ -679,15 +758,12 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
     }
 
     @Override
-    public boolean contains(Object obj) {
+    public boolean contains(@CheckForNull Object obj) {
       return containsColumn(obj);
     }
   }
 
-  /**
-   * Creates an iterator that returns each column value with duplicates
-   * omitted.
-   */
+  /** Creates an iterator that returns each column value with duplicates omitted. */
   Iterator<C> createColumnKeyIterator() {
     return new ColumnKeyIterator();
   }
@@ -700,6 +776,7 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
     Iterator<Entry<C, V>> entryIterator = Iterators.emptyIterator();
 
     @Override
+    @CheckForNull
     protected C computeNext() {
       while (true) {
         if (entryIterator.hasNext()) {
@@ -720,15 +797,15 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
   /**
    * {@inheritDoc}
    *
-   * <p>The collection's iterator traverses the values for the first row,
-   * the values for the second row, and so on.
+   * <p>The collection's iterator traverses the values for the first row, the values for the second
+   * row, and so on.
    */
   @Override
   public Collection<V> values() {
     return super.values();
   }
 
-  private transient Map<R, Map<C, V>> rowMap;
+  @CheckForNull private transient Map<R, Map<C, V>> rowMap;
 
   @Override
   public Map<R, Map<C, V>> rowMap() {
@@ -743,19 +820,22 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
   @WeakOuter
   class RowMap extends ViewCachingAbstractMap<R, Map<C, V>> {
     @Override
-    public boolean containsKey(Object key) {
+    public boolean containsKey(@CheckForNull Object key) {
       return containsRow(key);
     }
 
     // performing cast only when key is in backing map and has the correct type
     @SuppressWarnings("unchecked")
     @Override
-    public Map<C, V> get(Object key) {
-      return containsRow(key) ? row((R) key) : null;
+    @CheckForNull
+    public Map<C, V> get(@CheckForNull Object key) {
+      // requireNonNull is safe because of the containsRow check.
+      return containsRow(key) ? row((R) requireNonNull(key)) : null;
     }
 
     @Override
-    public Map<C, V> remove(Object key) {
+    @CheckForNull
+    public Map<C, V> remove(@CheckForNull Object key) {
       return (key == null) ? null : backingMap.remove(key);
     }
 
@@ -784,7 +864,7 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
       }
 
       @Override
-      public boolean contains(Object obj) {
+      public boolean contains(@CheckForNull Object obj) {
         if (obj instanceof Entry) {
           Entry<?, ?> entry = (Entry<?, ?>) obj;
           return entry.getKey() != null
@@ -795,7 +875,7 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
       }
 
       @Override
-      public boolean remove(Object obj) {
+      public boolean remove(@CheckForNull Object obj) {
         if (obj instanceof Entry) {
           Entry<?, ?> entry = (Entry<?, ?>) obj;
           return entry.getKey() != null
@@ -807,7 +887,7 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
     }
   }
 
-  private transient ColumnMap columnMap;
+  @CheckForNull private transient ColumnMap columnMap;
 
   @Override
   public Map<C, Map<R, V>> columnMap() {
@@ -821,17 +901,20 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
     // has the correct type.
     @SuppressWarnings("unchecked")
     @Override
-    public Map<R, V> get(Object key) {
-      return containsColumn(key) ? column((C) key) : null;
+    @CheckForNull
+    public Map<R, V> get(@CheckForNull Object key) {
+      // requireNonNull is safe because of the containsColumn check.
+      return containsColumn(key) ? column((C) requireNonNull(key)) : null;
     }
 
     @Override
-    public boolean containsKey(Object key) {
+    public boolean containsKey(@CheckForNull Object key) {
       return containsColumn(key);
     }
 
     @Override
-    public Map<R, V> remove(Object key) {
+    @CheckForNull
+    public Map<R, V> remove(@CheckForNull Object key) {
       return containsColumn(key) ? removeColumn(key) : null;
     }
 
@@ -870,23 +953,24 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
       }
 
       @Override
-      public boolean contains(Object obj) {
+      public boolean contains(@CheckForNull Object obj) {
         if (obj instanceof Entry) {
           Entry<?, ?> entry = (Entry<?, ?>) obj;
           if (containsColumn(entry.getKey())) {
-            // The cast to C occurs only when the key is in the map, implying
-            // that it has the correct type.
-            @SuppressWarnings("unchecked")
-            C columnKey = (C) entry.getKey();
-            return get(columnKey).equals(entry.getValue());
+            // requireNonNull is safe because of the containsColumn check.
+            return requireNonNull(get(entry.getKey())).equals(entry.getValue());
           }
         }
         return false;
       }
 
       @Override
-      public boolean remove(Object obj) {
-        if (contains(obj)) {
+      public boolean remove(@CheckForNull Object obj) {
+        /*
+         * `o instanceof Entry` is guaranteed by `contains`, but we check it here to satisfy our
+         * nullness checker.
+         */
+        if (contains(obj) && obj instanceof Entry) {
           Entry<?, ?> entry = (Entry<?, ?>) obj;
           removeColumn(entry.getKey());
           return true;
@@ -927,7 +1011,7 @@ class StandardTable<R, C, V> extends AbstractTable<R, C, V> implements Serializa
       }
 
       @Override
-      public boolean remove(Object obj) {
+      public boolean remove(@CheckForNull Object obj) {
         for (Entry<C, Map<R, V>> entry : ColumnMap.this.entrySet()) {
           if (entry.getValue().equals(obj)) {
             removeColumn(entry.getKey());
