@@ -21,7 +21,7 @@ import com.google.common.annotations.Beta;
 import com.google.common.annotations.GwtIncompatible;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.MoreObjects;
-
+import java.io.Serializable;
 import java.util.Collection;
 import java.util.Comparator;
 import java.util.Iterator;
@@ -30,8 +30,7 @@ import java.util.NavigableMap;
 import java.util.NoSuchElementException;
 import java.util.Set;
 import java.util.TreeMap;
-
-import javax.annotation.Nullable;
+import javax.annotation.CheckForNull;
 
 /**
  * An implementation of {@link RangeSet} backed by a {@link TreeMap}.
@@ -40,33 +39,46 @@ import javax.annotation.Nullable;
  * @since 14.0
  */
 @Beta
-@GwtIncompatible("uses NavigableMap")
-public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
+@GwtIncompatible // uses NavigableMap
+@ElementTypesAreNonnullByDefault
+public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C>
+    implements Serializable {
 
   @VisibleForTesting final NavigableMap<Cut<C>, Range<C>> rangesByLowerBound;
 
-  /**
-   * Creates an empty {@code TreeRangeSet} instance.
-   */
+  /** Creates an empty {@code TreeRangeSet} instance. */
   public static <C extends Comparable<?>> TreeRangeSet<C> create() {
-    return new TreeRangeSet<C>(new TreeMap<Cut<C>, Range<C>>());
+    return new TreeRangeSet<>(new TreeMap<Cut<C>, Range<C>>());
   }
 
-  /**
-   * Returns a {@code TreeRangeSet} initialized with the ranges in the specified range set.
-   */
+  /** Returns a {@code TreeRangeSet} initialized with the ranges in the specified range set. */
   public static <C extends Comparable<?>> TreeRangeSet<C> create(RangeSet<C> rangeSet) {
     TreeRangeSet<C> result = create();
     result.addAll(rangeSet);
     return result;
   }
 
+  /**
+   * Returns a {@code TreeRangeSet} representing the union of the specified ranges.
+   *
+   * <p>This is the smallest {@code RangeSet} which encloses each of the specified ranges. An
+   * element will be contained in this {@code RangeSet} if and only if it is contained in at least
+   * one {@code Range} in {@code ranges}.
+   *
+   * @since 21.0
+   */
+  public static <C extends Comparable<?>> TreeRangeSet<C> create(Iterable<Range<C>> ranges) {
+    TreeRangeSet<C> result = create();
+    result.addAll(ranges);
+    return result;
+  }
+
   private TreeRangeSet(NavigableMap<Cut<C>, Range<C>> rangesByLowerCut) {
     this.rangesByLowerBound = rangesByLowerCut;
   }
 
-  private transient Set<Range<C>> asRanges;
-  private transient Set<Range<C>> asDescendingSetOfRanges;
+  @CheckForNull private transient Set<Range<C>> asRanges;
+  @CheckForNull private transient Set<Range<C>> asDescendingSetOfRanges;
 
   @Override
   public Set<Range<C>> asRanges() {
@@ -101,13 +113,13 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
     }
 
     @Override
-    public boolean equals(@Nullable Object o) {
+    public boolean equals(@CheckForNull Object o) {
       return Sets.equalsImpl(this, o);
     }
   }
 
   @Override
-  @Nullable
+  @CheckForNull
   public Range<C> rangeContaining(C value) {
     checkNotNull(value);
     Entry<Cut<C>, Range<C>> floorEntry = rangesByLowerBound.floorEntry(Cut.belowValue(value));
@@ -119,6 +131,21 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
     }
   }
 
+  @Override
+  public boolean intersects(Range<C> range) {
+    checkNotNull(range);
+    Entry<Cut<C>, Range<C>> ceilingEntry = rangesByLowerBound.ceilingEntry(range.lowerBound);
+    if (ceilingEntry != null
+        && ceilingEntry.getValue().isConnected(range)
+        && !ceilingEntry.getValue().intersection(range).isEmpty()) {
+      return true;
+    }
+    Entry<Cut<C>, Range<C>> priorEntry = rangesByLowerBound.lowerEntry(range.lowerBound);
+    return priorEntry != null
+        && priorEntry.getValue().isConnected(range)
+        && !priorEntry.getValue().intersection(range).isEmpty();
+  }
+
   @Override
   public boolean encloses(Range<C> range) {
     checkNotNull(range);
@@ -126,7 +153,7 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
     return floorEntry != null && floorEntry.getValue().encloses(range);
   }
 
-  @Nullable
+  @CheckForNull
   private Range<C> rangeEnclosing(Range<C> range) {
     checkNotNull(range);
     Entry<Cut<C>, Range<C>> floorEntry = rangesByLowerBound.floorEntry(range.lowerBound);
@@ -139,7 +166,11 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
   public Range<C> span() {
     Entry<Cut<C>, Range<C>> firstEntry = rangesByLowerBound.firstEntry();
     Entry<Cut<C>, Range<C>> lastEntry = rangesByLowerBound.lastEntry();
-    if (firstEntry == null) {
+    if (firstEntry == null || lastEntry == null) {
+      /*
+       * Either both are null or neither is: Either the set is empty, or it's not. But we check both
+       * to make the nullness checker happy.
+       */
       throw new NoSuchElementException();
     }
     return Range.create(firstEntry.getValue().lowerBound, lastEntry.getValue().upperBound);
@@ -243,7 +274,7 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
     }
   }
 
-  private transient RangeSet<C> complement;
+  @CheckForNull private transient RangeSet<C> complement;
 
   @Override
   public RangeSet<C> complement() {
@@ -275,7 +306,7 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
 
     private NavigableMap<Cut<C>, Range<C>> subMap(Range<Cut<C>> window) {
       if (window.isConnected(upperBoundWindow)) {
-        return new RangesByUpperBound<C>(rangesByLowerBound, window.intersection(upperBoundWindow));
+        return new RangesByUpperBound<>(rangesByLowerBound, window.intersection(upperBoundWindow));
       } else {
         return ImmutableSortedMap.of();
       }
@@ -306,12 +337,13 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
     }
 
     @Override
-    public boolean containsKey(@Nullable Object key) {
+    public boolean containsKey(@CheckForNull Object key) {
       return get(key) != null;
     }
 
     @Override
-    public Range<C> get(@Nullable Object key) {
+    @CheckForNull
+    public Range<C> get(@CheckForNull Object key) {
       if (key instanceof Cut) {
         try {
           @SuppressWarnings("unchecked") // we catch CCEs
@@ -336,7 +368,7 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
        * We want to start the iteration at the first range where the upper bound is in
        * upperBoundWindow.
        */
-      final Iterator<Range<C>> backingItr;
+      Iterator<Range<C>> backingItr;
       if (!upperBoundWindow.hasLowerBound()) {
         backingItr = rangesByLowerBound.values().iterator();
       } else {
@@ -347,14 +379,16 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
         } else if (upperBoundWindow.lowerBound.isLessThan(lowerEntry.getValue().upperBound)) {
           backingItr = rangesByLowerBound.tailMap(lowerEntry.getKey(), true).values().iterator();
         } else {
-          backingItr = rangesByLowerBound
-              .tailMap(upperBoundWindow.lowerEndpoint(), true)
-              .values()
-              .iterator();
+          backingItr =
+              rangesByLowerBound
+                  .tailMap(upperBoundWindow.lowerEndpoint(), true)
+                  .values()
+                  .iterator();
         }
       }
       return new AbstractIterator<Entry<Cut<C>, Range<C>>>() {
         @Override
+        @CheckForNull
         protected Entry<Cut<C>, Range<C>> computeNext() {
           if (!backingItr.hasNext()) {
             return endOfData();
@@ -381,13 +415,14 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
       } else {
         candidates = rangesByLowerBound.descendingMap().values();
       }
-      final PeekingIterator<Range<C>> backingItr = Iterators.peekingIterator(candidates.iterator());
+      PeekingIterator<Range<C>> backingItr = Iterators.peekingIterator(candidates.iterator());
       if (backingItr.hasNext()
           && upperBoundWindow.upperBound.isLessThan(backingItr.peek().upperBound)) {
         backingItr.next();
       }
       return new AbstractIterator<Entry<Cut<C>, Range<C>>>() {
         @Override
+        @CheckForNull
         protected Entry<Cut<C>, Range<C>> computeNext() {
           if (!backingItr.hasNext()) {
             return endOfData();
@@ -435,7 +470,7 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
     private ComplementRangesByLowerBound(
         NavigableMap<Cut<C>, Range<C>> positiveRangesByLowerBound, Range<Cut<C>> window) {
       this.positiveRangesByLowerBound = positiveRangesByLowerBound;
-      this.positiveRangesByUpperBound = new RangesByUpperBound<C>(positiveRangesByLowerBound);
+      this.positiveRangesByUpperBound = new RangesByUpperBound<>(positiveRangesByLowerBound);
       this.complementLowerBoundWindow = window;
     }
 
@@ -444,7 +479,7 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
         return ImmutableSortedMap.of();
       } else {
         subWindow = subWindow.intersection(complementLowerBoundWindow);
-        return new ComplementRangesByLowerBound<C>(positiveRangesByLowerBound, subWindow);
+        return new ComplementRangesByLowerBound<>(positiveRangesByLowerBound, subWindow);
       }
     }
 
@@ -494,9 +529,8 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
       } else {
         positiveRanges = positiveRangesByUpperBound.values();
       }
-      final PeekingIterator<Range<C>> positiveItr =
-          Iterators.peekingIterator(positiveRanges.iterator());
-      final Cut<C> firstComplementRangeLowerBound;
+      PeekingIterator<Range<C>> positiveItr = Iterators.peekingIterator(positiveRanges.iterator());
+      Cut<C> firstComplementRangeLowerBound;
       if (complementLowerBoundWindow.contains(Cut.<C>belowAll())
           && (!positiveItr.hasNext() || positiveItr.peek().lowerBound != Cut.<C>belowAll())) {
         firstComplementRangeLowerBound = Cut.belowAll();
@@ -509,6 +543,7 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
         Cut<C> nextComplementRangeLowerBound = firstComplementRangeLowerBound;
 
         @Override
+        @CheckForNull
         protected Entry<Cut<C>, Range<C>> computeNext() {
           if (complementLowerBoundWindow.upperBound.isLessThan(nextComplementRangeLowerBound)
               || nextComplementRangeLowerBound == Cut.<C>aboveAll()) {
@@ -545,7 +580,7 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
       boolean inclusive =
           complementLowerBoundWindow.hasUpperBound()
               && complementLowerBoundWindow.upperBoundType() == BoundType.CLOSED;
-      final PeekingIterator<Range<C>> positiveItr =
+      PeekingIterator<Range<C>> positiveItr =
           Iterators.peekingIterator(
               positiveRangesByUpperBound
                   .headMap(startingPoint, inclusive)
@@ -554,21 +589,22 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
                   .iterator());
       Cut<C> cut;
       if (positiveItr.hasNext()) {
-        cut = (positiveItr.peek().upperBound == Cut.<C>aboveAll())
-            ? positiveItr.next().lowerBound
-            : positiveRangesByLowerBound.higherKey(positiveItr.peek().upperBound);
+        cut =
+            (positiveItr.peek().upperBound == Cut.<C>aboveAll())
+                ? positiveItr.next().lowerBound
+                : positiveRangesByLowerBound.higherKey(positiveItr.peek().upperBound);
       } else if (!complementLowerBoundWindow.contains(Cut.<C>belowAll())
           || positiveRangesByLowerBound.containsKey(Cut.belowAll())) {
         return Iterators.emptyIterator();
       } else {
         cut = positiveRangesByLowerBound.higherKey(Cut.<C>belowAll());
       }
-      final Cut<C> firstComplementRangeUpperBound =
-          MoreObjects.firstNonNull(cut, Cut.<C>aboveAll());
+      Cut<C> firstComplementRangeUpperBound = MoreObjects.firstNonNull(cut, Cut.<C>aboveAll());
       return new AbstractIterator<Entry<Cut<C>, Range<C>>>() {
         Cut<C> nextComplementRangeUpperBound = firstComplementRangeUpperBound;
 
         @Override
+        @CheckForNull
         protected Entry<Cut<C>, Range<C>> computeNext() {
           if (nextComplementRangeUpperBound == Cut.<C>belowAll()) {
             return endOfData();
@@ -596,8 +632,8 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
     }
 
     @Override
-    @Nullable
-    public Range<C> get(Object key) {
+    @CheckForNull
+    public Range<C> get(@CheckForNull Object key) {
       if (key instanceof Cut) {
         try {
           @SuppressWarnings("unchecked")
@@ -615,7 +651,7 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
     }
 
     @Override
-    public boolean containsKey(Object key) {
+    public boolean containsKey(@CheckForNull Object key) {
       return get(key) != null;
     }
   }
@@ -670,14 +706,14 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
       this.lowerBoundWindow = checkNotNull(lowerBoundWindow);
       this.restriction = checkNotNull(restriction);
       this.rangesByLowerBound = checkNotNull(rangesByLowerBound);
-      this.rangesByUpperBound = new RangesByUpperBound<C>(rangesByLowerBound);
+      this.rangesByUpperBound = new RangesByUpperBound<>(rangesByLowerBound);
     }
 
     private NavigableMap<Cut<C>, Range<C>> subMap(Range<Cut<C>> window) {
       if (!window.isConnected(lowerBoundWindow)) {
         return ImmutableSortedMap.of();
       } else {
-        return new SubRangeSetRangesByLowerBound<C>(
+        return new SubRangeSetRangesByLowerBound<>(
             lowerBoundWindow.intersection(window), restriction, rangesByLowerBound);
       }
     }
@@ -709,13 +745,13 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
     }
 
     @Override
-    public boolean containsKey(@Nullable Object key) {
+    public boolean containsKey(@CheckForNull Object key) {
       return get(key) != null;
     }
 
     @Override
-    @Nullable
-    public Range<C> get(@Nullable Object key) {
+    @CheckForNull
+    public Range<C> get(@CheckForNull Object key) {
       if (key instanceof Cut) {
         try {
           @SuppressWarnings("unchecked") // we catch CCE's
@@ -748,7 +784,7 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
       if (restriction.isEmpty()) {
         return Iterators.emptyIterator();
       }
-      final Iterator<Range<C>> completeRangeItr;
+      Iterator<Range<C>> completeRangeItr;
       if (lowerBoundWindow.upperBound.isLessThan(restriction.lowerBound)) {
         return Iterators.emptyIterator();
       } else if (lowerBoundWindow.lowerBound.isLessThan(restriction.lowerBound)) {
@@ -765,11 +801,12 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
                 .values()
                 .iterator();
       }
-      final Cut<Cut<C>> upperBoundOnLowerBounds =
+      Cut<Cut<C>> upperBoundOnLowerBounds =
           Ordering.natural()
               .min(lowerBoundWindow.upperBound, Cut.belowValue(restriction.upperBound));
       return new AbstractIterator<Entry<Cut<C>, Range<C>>>() {
         @Override
+        @CheckForNull
         protected Entry<Cut<C>, Range<C>> computeNext() {
           if (!completeRangeItr.hasNext()) {
             return endOfData();
@@ -793,7 +830,7 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
       Cut<Cut<C>> upperBoundOnLowerBounds =
           Ordering.natural()
               .min(lowerBoundWindow.upperBound, Cut.belowValue(restriction.upperBound));
-      final Iterator<Range<C>> completeRangeItr =
+      Iterator<Range<C>> completeRangeItr =
           rangesByLowerBound
               .headMap(
                   upperBoundOnLowerBounds.endpoint(),
@@ -803,6 +840,7 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
               .iterator();
       return new AbstractIterator<Entry<Cut<C>, Range<C>>>() {
         @Override
+        @CheckForNull
         protected Entry<Cut<C>, Range<C>> computeNext() {
           if (!completeRangeItr.hasNext()) {
             return endOfData();
@@ -836,8 +874,9 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
     private final Range<C> restriction;
 
     SubRangeSet(Range<C> restriction) {
-      super(new SubRangeSetRangesByLowerBound<C>(
-          Range.<Cut<C>>all(), restriction, TreeRangeSet.this.rangesByLowerBound));
+      super(
+          new SubRangeSetRangesByLowerBound<C>(
+              Range.<Cut<C>>all(), restriction, TreeRangeSet.this.rangesByLowerBound));
       this.restriction = restriction;
     }
 
@@ -851,7 +890,7 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
     }
 
     @Override
-    @Nullable
+    @CheckForNull
     public Range<C> rangeContaining(C value) {
       if (!restriction.contains(value)) {
         return null;
@@ -867,7 +906,7 @@ public class TreeRangeSet<C extends Comparable<?>> extends AbstractRangeSet<C> {
           "Cannot add range %s to subRangeSet(%s)",
           rangeToAdd,
           restriction);
-      super.add(rangeToAdd);
+      TreeRangeSet.this.add(rangeToAdd);
     }
 
     @Override
