@@ -1,17 +1,15 @@
 /*
  * Copyright (C) 2008 The Guava Authors
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License. You may obtain a copy of the License at
  *
  * http://www.apache.org/licenses/LICENSE-2.0
  *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Unless required by applicable law or agreed to in writing, software distributed under the License
+ * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
+ * or implied. See the License for the specific language governing permissions and limitations under
+ * the License.
  */
 
 package com.google.common.primitives;
@@ -25,7 +23,6 @@ import com.google.common.annotations.Beta;
 import com.google.common.annotations.GwtCompatible;
 import com.google.common.annotations.GwtIncompatible;
 import com.google.common.base.Converter;
-
 import java.io.Serializable;
 import java.util.AbstractList;
 import java.util.Arrays;
@@ -34,30 +31,29 @@ import java.util.Collections;
 import java.util.Comparator;
 import java.util.List;
 import java.util.RandomAccess;
-
+import java.util.Spliterator;
+import java.util.Spliterators;
 import javax.annotation.CheckForNull;
-import javax.annotation.CheckReturnValue;
-import javax.annotation.Nullable;
 
 /**
- * Static utility methods pertaining to {@code int} primitives, that are not
- * already found in either {@link Integer} or {@link Arrays}.
+ * Static utility methods pertaining to {@code int} primitives, that are not already found in either
+ * {@link Integer} or {@link Arrays}.
  *
- * <p>See the Guava User Guide article on <a href=
- * "https://github.com/google/guava/wiki/PrimitivesExplained">
- * primitive utilities</a>.
+ * <p>See the Guava User Guide article on <a
+ * href="https://github.com/google/guava/wiki/PrimitivesExplained">primitive utilities</a>.
  *
  * @author Kevin Bourrillion
  * @since 1.0
  */
-@CheckReturnValue
 @GwtCompatible(emulated = true)
-public final class Ints {
+@ElementTypesAreNonnullByDefault
+public final class Ints extends IntsMethodsForWeb {
   private Ints() {}
 
   /**
-   * The number of bytes required to represent a primitive {@code int}
-   * value.
+   * The number of bytes required to represent a primitive {@code int} value.
+   *
+   * <p><b>Java 8 users:</b> use {@link Integer#BYTES} instead.
    */
   public static final int BYTES = Integer.SIZE / Byte.SIZE;
 
@@ -69,8 +65,10 @@ public final class Ints {
   public static final int MAX_POWER_OF_TWO = 1 << (Integer.SIZE - 2);
 
   /**
-   * Returns a hash code for {@code value}; equal to the result of invoking
-   * {@code ((Integer) value).hashCode()}.
+   * Returns a hash code for {@code value}; equal to the result of invoking {@code ((Integer)
+   * value).hashCode()}.
+   *
+   * <p><b>Java 8 users:</b> use {@link Integer#hashCode(int)} instead.
    *
    * @param value a primitive {@code int} value
    * @return a hash code for the value
@@ -84,15 +82,12 @@ public final class Ints {
    *
    * @param value any value in the range of the {@code int} type
    * @return the {@code int} value that equals {@code value}
-   * @throws IllegalArgumentException if {@code value} is greater than {@link
-   *     Integer#MAX_VALUE} or less than {@link Integer#MIN_VALUE}
+   * @throws IllegalArgumentException if {@code value} is greater than {@link Integer#MAX_VALUE} or
+   *     less than {@link Integer#MIN_VALUE}
    */
   public static int checkedCast(long value) {
     int result = (int) value;
-    if (result != value) {
-      // don't use checkArgument here, to avoid boxing
-      throw new IllegalArgumentException("Out of range: " + value);
-    }
+    checkArgument(result == value, "Out of range: %s", value);
     return result;
   }
 
@@ -100,9 +95,9 @@ public final class Ints {
    * Returns the {@code int} nearest in value to {@code value}.
    *
    * @param value any {@code long} value
-   * @return the same value cast to {@code int} if it is in the range of the
-   *     {@code int} type, {@link Integer#MAX_VALUE} if it is too large,
-   *     or {@link Integer#MIN_VALUE} if it is too small
+   * @return the same value cast to {@code int} if it is in the range of the {@code int} type,
+   *     {@link Integer#MAX_VALUE} if it is too large, or {@link Integer#MIN_VALUE} if it is too
+   *     small
    */
   public static int saturatedCast(long value) {
     if (value > Integer.MAX_VALUE) {
@@ -115,29 +110,27 @@ public final class Ints {
   }
 
   /**
-   * Compares the two specified {@code int} values. The sign of the value
-   * returned is the same as that of {@code ((Integer) a).compareTo(b)}.
+   * Compares the two specified {@code int} values. The sign of the value returned is the same as
+   * that of {@code ((Integer) a).compareTo(b)}.
    *
-   * <p><b>Note for Java 7 and later:</b> this method should be treated as
-   * deprecated; use the equivalent {@link Integer#compare} method instead.
+   * <p><b>Note for Java 7 and later:</b> this method should be treated as deprecated; use the
+   * equivalent {@link Integer#compare} method instead.
    *
    * @param a the first {@code int} to compare
    * @param b the second {@code int} to compare
-   * @return a negative value if {@code a} is less than {@code b}; a positive
-   *     value if {@code a} is greater than {@code b}; or zero if they are equal
+   * @return a negative value if {@code a} is less than {@code b}; a positive value if {@code a} is
+   *     greater than {@code b}; or zero if they are equal
    */
   public static int compare(int a, int b) {
     return (a < b) ? -1 : ((a > b) ? 1 : 0);
   }
 
   /**
-   * Returns {@code true} if {@code target} is present as an element anywhere in
-   * {@code array}.
+   * Returns {@code true} if {@code target} is present as an element anywhere in {@code array}.
    *
    * @param array an array of {@code int} values, possibly empty
    * @param target a primitive {@code int} value
-   * @return {@code true} if {@code array[i] == target} for some value of {@code
-   *     i}
+   * @return {@code true} if {@code array[i] == target} for some value of {@code i}
    */
   public static boolean contains(int[] array, int target) {
     for (int value : array) {
@@ -149,13 +142,12 @@ public final class Ints {
   }
 
   /**
-   * Returns the index of the first appearance of the value {@code target} in
-   * {@code array}.
+   * Returns the index of the first appearance of the value {@code target} in {@code array}.
    *
    * @param array an array of {@code int} values, possibly empty
    * @param target a primitive {@code int} value
-   * @return the least index {@code i} for which {@code array[i] == target}, or
-   *     {@code -1} if no such index exists.
+   * @return the least index {@code i} for which {@code array[i] == target}, or {@code -1} if no
+   *     such index exists.
    */
   public static int indexOf(int[] array, int target) {
     return indexOf(array, target, 0, array.length);
@@ -172,12 +164,11 @@ public final class Ints {
   }
 
   /**
-   * Returns the start position of the first occurrence of the specified {@code
-   * target} within {@code array}, or {@code -1} if there is no such occurrence.
+   * Returns the start position of the first occurrence of the specified {@code target} within
+   * {@code array}, or {@code -1} if there is no such occurrence.
    *
-   * <p>More formally, returns the lowest index {@code i} such that {@code
-   * java.util.Arrays.copyOfRange(array, i, i + target.length)} contains exactly
-   * the same elements as {@code target}.
+   * <p>More formally, returns the lowest index {@code i} such that {@code Arrays.copyOfRange(array,
+   * i, i + target.length)} contains exactly the same elements as {@code target}.
    *
    * @param array the array to search for the sequence {@code target}
    * @param target the array to search for as a sub-sequence of {@code array}
@@ -202,13 +193,12 @@ public final class Ints {
   }
 
   /**
-   * Returns the index of the last appearance of the value {@code target} in
-   * {@code array}.
+   * Returns the index of the last appearance of the value {@code target} in {@code array}.
    *
    * @param array an array of {@code int} values, possibly empty
    * @param target a primitive {@code int} value
-   * @return the greatest index {@code i} for which {@code array[i] == target},
-   *     or {@code -1} if no such index exists.
+   * @return the greatest index {@code i} for which {@code array[i] == target}, or {@code -1} if no
+   *     such index exists.
    */
   public static int lastIndexOf(int[] array, int target) {
     return lastIndexOf(array, target, 0, array.length);
@@ -228,10 +218,12 @@ public final class Ints {
    * Returns the least value present in {@code array}.
    *
    * @param array a <i>nonempty</i> array of {@code int} values
-   * @return the value present in {@code array} that is less than or equal to
-   *     every other value in the array
+   * @return the value present in {@code array} that is less than or equal to every other value in
+   *     the array
    * @throws IllegalArgumentException if {@code array} is empty
    */
+  @GwtIncompatible(
+      "Available in GWT! Annotation is to avoid conflict with GWT specialization of base class.")
   public static int min(int... array) {
     checkArgument(array.length > 0);
     int min = array[0];
@@ -247,10 +239,12 @@ public final class Ints {
    * Returns the greatest value present in {@code array}.
    *
    * @param array a <i>nonempty</i> array of {@code int} values
-   * @return the value present in {@code array} that is greater than or equal to
-   *     every other value in the array
+   * @return the value present in {@code array} that is greater than or equal to every other value
+   *     in the array
    * @throws IllegalArgumentException if {@code array} is empty
    */
+  @GwtIncompatible(
+      "Available in GWT! Annotation is to avoid conflict with GWT specialization of base class.")
   public static int max(int... array) {
     checkArgument(array.length > 0);
     int max = array[0];
@@ -263,13 +257,30 @@ public final class Ints {
   }
 
   /**
-   * Returns the values from each provided array combined into a single array.
-   * For example, {@code concat(new int[] {a, b}, new int[] {}, new
-   * int[] {c}} returns the array {@code {a, b, c}}.
+   * Returns the value nearest to {@code value} which is within the closed range {@code [min..max]}.
+   *
+   * <p>If {@code value} is within the range {@code [min..max]}, {@code value} is returned
+   * unchanged. If {@code value} is less than {@code min}, {@code min} is returned, and if {@code
+   * value} is greater than {@code max}, {@code max} is returned.
+   *
+   * @param value the {@code int} value to constrain
+   * @param min the lower bound (inclusive) of the range to constrain {@code value} to
+   * @param max the upper bound (inclusive) of the range to constrain {@code value} to
+   * @throws IllegalArgumentException if {@code min > max}
+   * @since 21.0
+   */
+  @Beta
+  public static int constrainToRange(int value, int min, int max) {
+    checkArgument(min <= max, "min (%s) must be less than or equal to max (%s)", min, max);
+    return Math.min(Math.max(value, min), max);
+  }
+
+  /**
+   * Returns the values from each provided array combined into a single array. For example, {@code
+   * concat(new int[] {a, b}, new int[] {}, new int[] {c}} returns the array {@code {a, b, c}}.
    *
    * @param arrays zero or more {@code int} arrays
-   * @return a single array containing all the values from the source arrays, in
-   *     order
+   * @return a single array containing all the values from the source arrays, in order
    */
   public static int[] concat(int[]... arrays) {
     int length = 0;
@@ -286,52 +297,42 @@ public final class Ints {
   }
 
   /**
-   * Returns a big-endian representation of {@code value} in a 4-element byte
-   * array; equivalent to {@code ByteBuffer.allocate(4).putInt(value).array()}.
-   * For example, the input value {@code 0x12131415} would yield the byte array
-   * {@code {0x12, 0x13, 0x14, 0x15}}.
+   * Returns a big-endian representation of {@code value} in a 4-element byte array; equivalent to
+   * {@code ByteBuffer.allocate(4).putInt(value).array()}. For example, the input value {@code
+   * 0x12131415} would yield the byte array {@code {0x12, 0x13, 0x14, 0x15}}.
    *
-   * <p>If you need to convert and concatenate several values (possibly even of
-   * different types), use a shared {@link java.nio.ByteBuffer} instance, or use
-   * {@link com.google.common.io.ByteStreams#newDataOutput()} to get a growable
-   * buffer.
+   * <p>If you need to convert and concatenate several values (possibly even of different types),
+   * use a shared {@link java.nio.ByteBuffer} instance, or use {@link
+   * com.google.common.io.ByteStreams#newDataOutput()} to get a growable buffer.
    */
-  @GwtIncompatible("doesn't work")
   public static byte[] toByteArray(int value) {
     return new byte[] {
-      (byte) (value >> 24),
-      (byte) (value >> 16),
-      (byte) (value >> 8),
-      (byte) value
+      (byte) (value >> 24), (byte) (value >> 16), (byte) (value >> 8), (byte) value
     };
   }
 
   /**
-   * Returns the {@code int} value whose big-endian representation is stored in
-   * the first 4 bytes of {@code bytes}; equivalent to {@code
-   * ByteBuffer.wrap(bytes).getInt()}. For example, the input byte array {@code
-   * {0x12, 0x13, 0x14, 0x15, 0x33}} would yield the {@code int} value {@code
+   * Returns the {@code int} value whose big-endian representation is stored in the first 4 bytes of
+   * {@code bytes}; equivalent to {@code ByteBuffer.wrap(bytes).getInt()}. For example, the input
+   * byte array {@code {0x12, 0x13, 0x14, 0x15, 0x33}} would yield the {@code int} value {@code
    * 0x12131415}.
    *
-   * <p>Arguably, it's preferable to use {@link java.nio.ByteBuffer}; that
-   * library exposes much more flexibility at little cost in readability.
+   * <p>Arguably, it's preferable to use {@link java.nio.ByteBuffer}; that library exposes much more
+   * flexibility at little cost in readability.
    *
    * @throws IllegalArgumentException if {@code bytes} has fewer than 4 elements
    */
-  @GwtIncompatible("doesn't work")
   public static int fromByteArray(byte[] bytes) {
     checkArgument(bytes.length >= BYTES, "array too small: %s < %s", bytes.length, BYTES);
     return fromBytes(bytes[0], bytes[1], bytes[2], bytes[3]);
   }
 
   /**
-   * Returns the {@code int} value whose byte representation is the given 4
-   * bytes, in big-endian order; equivalent to {@code Ints.fromByteArray(new
-   * byte[] {b1, b2, b3, b4})}.
+   * Returns the {@code int} value whose byte representation is the given 4 bytes, in big-endian
+   * order; equivalent to {@code Ints.fromByteArray(new byte[] {b1, b2, b3, b4})}.
    *
    * @since 7.0
    */
-  @GwtIncompatible("doesn't work")
   public static int fromBytes(byte b1, byte b2, byte b3, byte b4) {
     return b1 << 24 | (b2 & 0xFF) << 16 | (b3 & 0xFF) << 8 | (b4 & 0xFF);
   }
@@ -363,8 +364,13 @@ public final class Ints {
   }
 
   /**
-   * Returns a serializable converter object that converts between strings and
-   * integers using {@link Integer#decode} and {@link Integer#toString()}.
+   * Returns a serializable converter object that converts between strings and integers using {@link
+   * Integer#decode} and {@link Integer#toString()}. The returned converter throws {@link
+   * NumberFormatException} if the input string is invalid.
+   *
+   * <p><b>Warning:</b> please see {@link Integer#decode} to understand exactly how strings are
+   * parsed. For example, the string {@code "0123"} is treated as <i>octal</i> and converted to the
+   * value {@code 83}.
    *
    * @since 16.0
    */
@@ -374,43 +380,30 @@ public final class Ints {
   }
 
   /**
-   * Returns an array containing the same values as {@code array}, but
-   * guaranteed to be of a specified minimum length. If {@code array} already
-   * has a length of at least {@code minLength}, it is returned directly.
-   * Otherwise, a new array of size {@code minLength + padding} is returned,
-   * containing the values of {@code array}, and zeroes in the remaining places.
+   * Returns an array containing the same values as {@code array}, but guaranteed to be of a
+   * specified minimum length. If {@code array} already has a length of at least {@code minLength},
+   * it is returned directly. Otherwise, a new array of size {@code minLength + padding} is
+   * returned, containing the values of {@code array}, and zeroes in the remaining places.
    *
    * @param array the source array
    * @param minLength the minimum length the returned array must guarantee
-   * @param padding an extra amount to "grow" the array by if growth is
-   *     necessary
-   * @throws IllegalArgumentException if {@code minLength} or {@code padding} is
-   *     negative
-   * @return an array containing the values of {@code array}, with guaranteed
-   *     minimum length {@code minLength}
+   * @param padding an extra amount to "grow" the array by if growth is necessary
+   * @throws IllegalArgumentException if {@code minLength} or {@code padding} is negative
+   * @return an array containing the values of {@code array}, with guaranteed minimum length {@code
+   *     minLength}
    */
   public static int[] ensureCapacity(int[] array, int minLength, int padding) {
     checkArgument(minLength >= 0, "Invalid minLength: %s", minLength);
     checkArgument(padding >= 0, "Invalid padding: %s", padding);
-    return (array.length < minLength)
-        ? copyOf(array, minLength + padding)
-        : array;
-  }
-
-  // Arrays.copyOf() requires Java 6
-  private static int[] copyOf(int[] original, int length) {
-    int[] copy = new int[length];
-    System.arraycopy(original, 0, copy, 0, Math.min(original.length, length));
-    return copy;
+    return (array.length < minLength) ? Arrays.copyOf(array, minLength + padding) : array;
   }
 
   /**
-   * Returns a string containing the supplied {@code int} values separated
-   * by {@code separator}. For example, {@code join("-", 1, 2, 3)} returns
-   * the string {@code "1-2-3"}.
+   * Returns a string containing the supplied {@code int} values separated by {@code separator}. For
+   * example, {@code join("-", 1, 2, 3)} returns the string {@code "1-2-3"}.
    *
-   * @param separator the text that should appear between consecutive values in
-   *     the resulting string (but not at the start or end)
+   * @param separator the text that should appear between consecutive values in the resulting string
+   *     (but not at the start or end)
    * @param array an array of {@code int} values, possibly empty
    */
   public static String join(String separator, int... array) {
@@ -429,18 +422,15 @@ public final class Ints {
   }
 
   /**
-   * Returns a comparator that compares two {@code int} arrays
-   * lexicographically. That is, it compares, using {@link
-   * #compare(int, int)}), the first pair of values that follow any
-   * common prefix, or when one array is a prefix of the other, treats the
-   * shorter array as the lesser. For example, {@code [] < [1] < [1, 2] < [2]}.
+   * Returns a comparator that compares two {@code int} arrays <a
+   * href="http://en.wikipedia.org/wiki/Lexicographical_order">lexicographically</a>. That is, it
+   * compares, using {@link #compare(int, int)}), the first pair of values that follow any common
+   * prefix, or when one array is a prefix of the other, treats the shorter array as the lesser. For
+   * example, {@code [] < [1] < [1, 2] < [2]}.
    *
-   * <p>The returned comparator is inconsistent with {@link
-   * Object#equals(Object)} (since arrays support only identity equality), but
-   * it is consistent with {@link Arrays#equals(int[], int[])}.
+   * <p>The returned comparator is inconsistent with {@link Object#equals(Object)} (since arrays
+   * support only identity equality), but it is consistent with {@link Arrays#equals(int[], int[])}.
    *
-   * @see <a href="http://en.wikipedia.org/wiki/Lexicographical_order">
-   *     Lexicographical order article at Wikipedia</a>
    * @since 2.0
    */
   public static Comparator<int[]> lexicographicalComparator() {
@@ -461,21 +451,78 @@ public final class Ints {
       }
       return left.length - right.length;
     }
+
+    @Override
+    public String toString() {
+      return "Ints.lexicographicalComparator()";
+    }
   }
 
   /**
-   * Returns an array containing each value of {@code collection}, converted to
-   * a {@code int} value in the manner of {@link Number#intValue}.
+   * Sorts the elements of {@code array} in descending order.
    *
-   * <p>Elements are copied from the argument collection as if by {@code
-   * collection.toArray()}.  Calling this method is as thread-safe as calling
-   * that method.
+   * @since 23.1
+   */
+  public static void sortDescending(int[] array) {
+    checkNotNull(array);
+    sortDescending(array, 0, array.length);
+  }
+
+  /**
+   * Sorts the elements of {@code array} between {@code fromIndex} inclusive and {@code toIndex}
+   * exclusive in descending order.
+   *
+   * @since 23.1
+   */
+  public static void sortDescending(int[] array, int fromIndex, int toIndex) {
+    checkNotNull(array);
+    checkPositionIndexes(fromIndex, toIndex, array.length);
+    Arrays.sort(array, fromIndex, toIndex);
+    reverse(array, fromIndex, toIndex);
+  }
+
+  /**
+   * Reverses the elements of {@code array}. This is equivalent to {@code
+   * Collections.reverse(Ints.asList(array))}, but is likely to be more efficient.
+   *
+   * @since 23.1
+   */
+  public static void reverse(int[] array) {
+    checkNotNull(array);
+    reverse(array, 0, array.length);
+  }
+
+  /**
+   * Reverses the elements of {@code array} between {@code fromIndex} inclusive and {@code toIndex}
+   * exclusive. This is equivalent to {@code
+   * Collections.reverse(Ints.asList(array).subList(fromIndex, toIndex))}, but is likely to be more
+   * efficient.
+   *
+   * @throws IndexOutOfBoundsException if {@code fromIndex < 0}, {@code toIndex > array.length}, or
+   *     {@code toIndex > fromIndex}
+   * @since 23.1
+   */
+  public static void reverse(int[] array, int fromIndex, int toIndex) {
+    checkNotNull(array);
+    checkPositionIndexes(fromIndex, toIndex, array.length);
+    for (int i = fromIndex, j = toIndex - 1; i < j; i++, j--) {
+      int tmp = array[i];
+      array[i] = array[j];
+      array[j] = tmp;
+    }
+  }
+
+  /**
+   * Returns an array containing each value of {@code collection}, converted to a {@code int} value
+   * in the manner of {@link Number#intValue}.
+   *
+   * <p>Elements are copied from the argument collection as if by {@code collection.toArray()}.
+   * Calling this method is as thread-safe as calling that method.
    *
    * @param collection a collection of {@code Number} instances
-   * @return an array containing the same values as {@code collection}, in the
-   *     same order, converted to primitives
-   * @throws NullPointerException if {@code collection} or any of its elements
-   *     is null
+   * @return an array containing the same values as {@code collection}, in the same order, converted
+   *     to primitives
+   * @throws NullPointerException if {@code collection} or any of its elements is null
    * @since 1.0 (parameter was {@code Collection<Integer>} before 12.0)
    */
   public static int[] toArray(Collection<? extends Number> collection) {
@@ -495,14 +542,15 @@ public final class Ints {
 
   /**
    * Returns a fixed-size list backed by the specified array, similar to {@link
-   * Arrays#asList(Object[])}. The list supports {@link List#set(int, Object)},
-   * but any attempt to set a value to {@code null} will result in a {@link
-   * NullPointerException}.
+   * Arrays#asList(Object[])}. The list supports {@link List#set(int, Object)}, but any attempt to
+   * set a value to {@code null} will result in a {@link NullPointerException}.
    *
-   * <p>The returned list maintains the values, but not the identities, of
-   * {@code Integer} objects written to or read from it.  For example, whether
-   * {@code list.get(0) == list.get(0)} is true for the returned list is
-   * unspecified.
+   * <p>The returned list maintains the values, but not the identities, of {@code Integer} objects
+   * written to or read from it. For example, whether {@code list.get(0) == list.get(0)} is true for
+   * the returned list is unspecified.
+   *
+   * <p><b>Note:</b> when possible, you should represent your data as an {@link ImmutableIntArray}
+   * instead, which has an {@link ImmutableIntArray#asList asList} view.
    *
    * @param backingArray the array to back the list
    * @return a list view of the array
@@ -548,13 +596,18 @@ public final class Ints {
     }
 
     @Override
-    public boolean contains(Object target) {
+    public Spliterator.OfInt spliterator() {
+      return Spliterators.spliterator(array, start, end, 0);
+    }
+
+    @Override
+    public boolean contains(@CheckForNull Object target) {
       // Overridden to prevent a ton of boxing
       return (target instanceof Integer) && Ints.indexOf(array, (Integer) target, start, end) != -1;
     }
 
     @Override
-    public int indexOf(Object target) {
+    public int indexOf(@CheckForNull Object target) {
       // Overridden to prevent a ton of boxing
       if (target instanceof Integer) {
         int i = Ints.indexOf(array, (Integer) target, start, end);
@@ -566,7 +619,7 @@ public final class Ints {
     }
 
     @Override
-    public int lastIndexOf(Object target) {
+    public int lastIndexOf(@CheckForNull Object target) {
       // Overridden to prevent a ton of boxing
       if (target instanceof Integer) {
         int i = Ints.lastIndexOf(array, (Integer) target, start, end);
@@ -597,7 +650,7 @@ public final class Ints {
     }
 
     @Override
-    public boolean equals(@Nullable Object object) {
+    public boolean equals(@CheckForNull Object object) {
       if (object == this) {
         return true;
       }
@@ -637,71 +690,58 @@ public final class Ints {
     }
 
     int[] toIntArray() {
-      // Arrays.copyOfRange() is not available under GWT
-      int size = size();
-      int[] result = new int[size];
-      System.arraycopy(array, start, result, 0, size);
-      return result;
+      return Arrays.copyOfRange(array, start, end);
     }
 
     private static final long serialVersionUID = 0;
   }
 
   /**
-   * Parses the specified string as a signed decimal integer value. The ASCII
-   * character {@code '-'} (<code>'&#92;u002D'</code>) is recognized as the
-   * minus sign.
+   * Parses the specified string as a signed decimal integer value. The ASCII character {@code '-'}
+   * (<code>'&#92;u002D'</code>) is recognized as the minus sign.
    *
-   * <p>Unlike {@link Integer#parseInt(String)}, this method returns
-   * {@code null} instead of throwing an exception if parsing fails.
-   * Additionally, this method only accepts ASCII digits, and returns
-   * {@code null} if non-ASCII digits are present in the string.
+   * <p>Unlike {@link Integer#parseInt(String)}, this method returns {@code null} instead of
+   * throwing an exception if parsing fails. Additionally, this method only accepts ASCII digits,
+   * and returns {@code null} if non-ASCII digits are present in the string.
    *
-   * <p>Note that strings prefixed with ASCII {@code '+'} are rejected, even
-   * under JDK 7, despite the change to {@link Integer#parseInt(String)} for
-   * that version.
+   * <p>Note that strings prefixed with ASCII {@code '+'} are rejected, even under JDK 7, despite
+   * the change to {@link Integer#parseInt(String)} for that version.
    *
    * @param string the string representation of an integer value
-   * @return the integer value represented by {@code string}, or {@code null} if
-   *     {@code string} has a length of zero or cannot be parsed as an integer
-   *     value
+   * @return the integer value represented by {@code string}, or {@code null} if {@code string} has
+   *     a length of zero or cannot be parsed as an integer value
+   * @throws NullPointerException if {@code string} is {@code null}
    * @since 11.0
    */
   @Beta
-  @Nullable
   @CheckForNull
   public static Integer tryParse(String string) {
     return tryParse(string, 10);
   }
 
   /**
-   * Parses the specified string as a signed integer value using the specified
-   * radix. The ASCII character {@code '-'} (<code>'&#92;u002D'</code>) is
-   * recognized as the minus sign.
+   * Parses the specified string as a signed integer value using the specified radix. The ASCII
+   * character {@code '-'} (<code>'&#92;u002D'</code>) is recognized as the minus sign.
    *
-   * <p>Unlike {@link Integer#parseInt(String, int)}, this method returns
-   * {@code null} instead of throwing an exception if parsing fails.
-   * Additionally, this method only accepts ASCII digits, and returns
-   * {@code null} if non-ASCII digits are present in the string.
+   * <p>Unlike {@link Integer#parseInt(String, int)}, this method returns {@code null} instead of
+   * throwing an exception if parsing fails. Additionally, this method only accepts ASCII digits,
+   * and returns {@code null} if non-ASCII digits are present in the string.
    *
-   * <p>Note that strings prefixed with ASCII {@code '+'} are rejected, even
-   * under JDK 7, despite the change to {@link Integer#parseInt(String, int)}
-   * for that version.
+   * <p>Note that strings prefixed with ASCII {@code '+'} are rejected, even under JDK 7, despite
+   * the change to {@link Integer#parseInt(String, int)} for that version.
    *
    * @param string the string representation of an integer value
    * @param radix the radix to use when parsing
-   * @return the integer value represented by {@code string} using
-   *     {@code radix}, or {@code null} if {@code string} has a length of zero
-   *     or cannot be parsed as an integer value
-   * @throws IllegalArgumentException if {@code radix < Character.MIN_RADIX} or
-   *     {@code radix > Character.MAX_RADIX}
+   * @return the integer value represented by {@code string} using {@code radix}, or {@code null} if
+   *     {@code string} has a length of zero or cannot be parsed as an integer value
+   * @throws IllegalArgumentException if {@code radix < Character.MIN_RADIX} or {@code radix >
+   *     Character.MAX_RADIX}
+   * @throws NullPointerException if {@code string} is {@code null}
    * @since 19.0
    */
   @Beta
-  @Nullable
   @CheckForNull
-  public static Integer tryParse(
-      String string, int radix) {
+  public static Integer tryParse(String string, int radix) {
     Long result = Longs.tryParse(string, radix);
     if (result == null || result.longValue() != result.intValue()) {
       return null;
