@@ -1,18 +1,17 @@
 /*
  * Copyright (C) 2012 The Guava Authors
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License. You may obtain a copy of the License at
  *
  * http://www.apache.org/licenses/LICENSE-2.0
  *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Unless required by applicable law or agreed to in writing, software distributed under the License
+ * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
+ * or implied. See the License for the specific language governing permissions and limitations under
+ * the License.
  */
+
 package com.google.common.util.concurrent;
 
 import static com.google.common.base.Preconditions.checkArgument;
@@ -22,6 +21,7 @@ import static com.google.common.base.Predicates.equalTo;
 import static com.google.common.base.Predicates.in;
 import static com.google.common.base.Predicates.instanceOf;
 import static com.google.common.base.Predicates.not;
+import static com.google.common.util.concurrent.Internal.toNanosSaturated;
 import static com.google.common.util.concurrent.MoreExecutors.directExecutor;
 import static com.google.common.util.concurrent.Service.State.FAILED;
 import static com.google.common.util.concurrent.Service.State.NEW;
@@ -31,7 +31,7 @@ import static com.google.common.util.concurrent.Service.State.STOPPING;
 import static com.google.common.util.concurrent.Service.State.TERMINATED;
 import static java.util.concurrent.TimeUnit.MILLISECONDS;
 
-import com.google.common.annotations.Beta;
+import com.google.common.annotations.GwtIncompatible;
 import com.google.common.base.Function;
 import com.google.common.base.MoreObjects;
 import com.google.common.base.Stopwatch;
@@ -39,7 +39,6 @@ import com.google.common.collect.Collections2;
 import com.google.common.collect.ImmutableCollection;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
-import com.google.common.collect.ImmutableMultimap;
 import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.ImmutableSetMultimap;
 import com.google.common.collect.Lists;
@@ -49,12 +48,13 @@ import com.google.common.collect.Multimaps;
 import com.google.common.collect.Multiset;
 import com.google.common.collect.Ordering;
 import com.google.common.collect.SetMultimap;
-import com.google.common.util.concurrent.ListenerCallQueue.Callback;
 import com.google.common.util.concurrent.Service.State;
+import com.google.errorprone.annotations.CanIgnoreReturnValue;
+import com.google.errorprone.annotations.concurrent.GuardedBy;
+import com.google.j2objc.annotations.J2ObjCIncompatible;
 import com.google.j2objc.annotations.WeakOuter;
-
 import java.lang.ref.WeakReference;
-import java.util.ArrayList;
+import java.time.Duration;
 import java.util.Collections;
 import java.util.EnumSet;
 import java.util.List;
@@ -66,8 +66,6 @@ import java.util.concurrent.TimeoutException;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
-import javax.annotation.concurrent.GuardedBy;
-
 /**
  * A manager for monitoring and controlling a set of {@linkplain Service services}. This class
  * provides methods for {@linkplain #startAsync() starting}, {@linkplain #stopAsync() stopping} and
@@ -81,7 +79,8 @@ import javax.annotation.concurrent.GuardedBy;
  * when appropriate and {@link #awaitHealthy} will still work as expected.
  *
  * <p>Here is a simple example of how to use a {@code ServiceManager} to start a server.
- * <pre>   {@code
+ *
+ * <pre>{@code
  * class Server {
  *   public static void main(String[] args) {
  *     Set<Service> services = ...;
@@ -112,7 +111,8 @@ import javax.annotation.concurrent.GuardedBy;
  *     });
  *     manager.startAsync();  // start all the services asynchronously
  *   }
- * }}</pre>
+ * }
+ * }</pre>
  *
  * <p>This class uses the ServiceManager's methods to start all of its services, to respond to
  * service failure and to ensure that when the JVM is shutting down all the services are stopped.
@@ -120,19 +120,34 @@ import javax.annotation.concurrent.GuardedBy;
  * @author Luke Sandberg
  * @since 14.0
  */
-@Beta
-public final class ServiceManager {
+@GwtIncompatible
+@ElementTypesAreNonnullByDefault
+public final class ServiceManager implements ServiceManagerBridge {
   private static final Logger logger = Logger.getLogger(ServiceManager.class.getName());
-  private static final Callback<Listener> HEALTHY_CALLBACK = new Callback<Listener>("healthy()") {
-    @Override void call(Listener listener) {
-      listener.healthy();
-    }
-  };
-  private static final Callback<Listener> STOPPED_CALLBACK = new Callback<Listener>("stopped()") {
-    @Override void call(Listener listener) {
-      listener.stopped();
-    }
-  };
+  private static final ListenerCallQueue.Event<Listener> HEALTHY_EVENT =
+      new ListenerCallQueue.Event<Listener>() {
+        @Override
+        public void call(Listener listener) {
+          listener.healthy();
+        }
+
+        @Override
+        public String toString() {
+          return "healthy()";
+        }
+      };
+  private static final ListenerCallQueue.Event<Listener> STOPPED_EVENT =
+      new ListenerCallQueue.Event<Listener>() {
+        @Override
+        public void call(Listener listener) {
+          listener.stopped();
+        }
+
+        @Override
+        public String toString() {
+          return "stopped()";
+        }
+      };
 
   /**
    * A listener for the aggregate state changes of the services that are under management. Users
@@ -143,15 +158,14 @@ public final class ServiceManager {
    * @author Luke Sandberg
    * @since 15.0 (present as an interface in 14.0)
    */
-  @Beta  // Should come out of Beta when ServiceManager does
   public abstract static class Listener {
     /**
      * Called when the service initially becomes healthy.
      *
-     * <p>This will be called at most once after all the services have entered the
-     * {@linkplain State#RUNNING running} state. If any services fail during start up or
-     * {@linkplain State#FAILED fail}/{@linkplain State#TERMINATED terminate} before all other
-     * services have started {@linkplain State#RUNNING running} then this method will not be called.
+     * <p>This will be called at most once after all the services have entered the {@linkplain
+     * State#RUNNING running} state. If any services fail during start up or {@linkplain
+     * State#FAILED fail}/{@linkplain State#TERMINATED terminate} before all other services have
+     * started {@linkplain State#RUNNING running} then this method will not be called.
      */
     public void healthy() {}
 
@@ -171,36 +185,36 @@ public final class ServiceManager {
 
   /**
    * An encapsulation of all of the state that is accessed by the {@linkplain ServiceListener
-   * service listeners}.  This is extracted into its own object so that {@link ServiceListener}
-   * could be made {@code static} and its instances can be safely constructed and added in the
-   * {@link ServiceManager} constructor without having to close over the partially constructed
-   * {@link ServiceManager} instance (i.e. avoid leaking a pointer to {@code this}).
+   * service listeners}. This is extracted into its own object so that {@link ServiceListener} could
+   * be made {@code static} and its instances can be safely constructed and added in the {@link
+   * ServiceManager} constructor without having to close over the partially constructed {@link
+   * ServiceManager} instance (i.e. avoid leaking a pointer to {@code this}).
    */
   private final ServiceManagerState state;
+
   private final ImmutableList<Service> services;
 
   /**
    * Constructs a new instance for managing the given services.
    *
    * @param services The services to manage
-   *
    * @throws IllegalArgumentException if not all services are {@linkplain State#NEW new} or if there
-   * are any duplicate services.
+   *     are any duplicate services.
    */
   public ServiceManager(Iterable<? extends Service> services) {
     ImmutableList<Service> copy = ImmutableList.copyOf(services);
     if (copy.isEmpty()) {
       // Having no services causes the manager to behave strangely. Notably, listeners are never
-      // fired.  To avoid this we substitute a placeholder service.
-      logger.log(Level.WARNING,
+      // fired. To avoid this we substitute a placeholder service.
+      logger.log(
+          Level.WARNING,
           "ServiceManager configured with no services.  Is your application configured properly?",
           new EmptyServiceManagerWarning());
       copy = ImmutableList.<Service>of(new NoOpService());
     }
     this.state = new ServiceManagerState(copy);
     this.services = copy;
-    WeakReference<ServiceManagerState> stateReference =
-        new WeakReference<ServiceManagerState>(state);
+    WeakReference<ServiceManagerState> stateReference = new WeakReference<>(state);
     for (Service service : copy) {
       service.addListener(new ServiceListener(service, stateReference), directExecutor());
       // We check the state after adding the listener as a way to ensure that our listener was added
@@ -214,9 +228,9 @@ public final class ServiceManager {
 
   /**
    * Registers a {@link Listener} to be {@linkplain Executor#execute executed} on the given
-   * executor. The listener will not have previous state changes replayed, so it is
-   * suggested that listeners are added before any of the managed services are
-   * {@linkplain Service#startAsync started}.
+   * executor. The listener will not have previous state changes replayed, so it is suggested that
+   * listeners are added before any of the managed services are {@linkplain Service#startAsync
+   * started}.
    *
    * <p>{@code addListener} guarantees execution ordering across calls to a given listener but not
    * across calls to multiple listeners. Specifically, a given listener will have its callbacks
@@ -229,8 +243,9 @@ public final class ServiceManager {
    * during {@code Executor.execute} (e.g., a {@code RejectedExecutionException}) will be caught and
    * logged.
    *
-   * <p> For fast, lightweight listeners that would be safe to execute in any thread, consider
-   * calling {@link #addListener(Listener)}.
+   * <p>When selecting an executor, note that {@code directExecutor} is dangerous in some cases. See
+   * the discussion in the {@link ListenableFuture#addListener ListenableFuture.addListener}
+   * documentation.
    *
    * @param listener the listener to run when the manager changes state
    * @param executor the executor in which the listeners callback methods will be run.
@@ -240,37 +255,17 @@ public final class ServiceManager {
   }
 
   /**
-   * Registers a {@link Listener} to be run when this {@link ServiceManager} changes state. The
-   * listener will not have previous state changes replayed, so it is suggested that listeners are
-   * added before any of the managed services are {@linkplain Service#startAsync started}.
-   *
-   * <p>{@code addListener} guarantees execution ordering across calls to a given listener but not
-   * across calls to multiple listeners. Specifically, a given listener will have its callbacks
-   * invoked in the same order as the underlying service enters those states. Additionally, at most
-   * one of the listener's callbacks will execute at once. However, multiple listeners' callbacks
-   * may execute concurrently, and listeners may execute in an order different from the one in which
-   * they were registered.
-   *
-   * <p>RuntimeExceptions thrown by a listener will be caught and logged.
-   *
-   * @param listener the listener to run when the manager changes state
-   */
-  public void addListener(Listener listener) {
-    state.addListener(listener, directExecutor());
-  }
-
-  /**
-   * Initiates service {@linkplain Service#startAsync startup} on all the services being managed.
-   * It is only valid to call this method if all of the services are {@linkplain State#NEW new}.
+   * Initiates service {@linkplain Service#startAsync startup} on all the services being managed. It
+   * is only valid to call this method if all of the services are {@linkplain State#NEW new}.
    *
    * @return this
    * @throws IllegalStateException if any of the Services are not {@link State#NEW new} when the
    *     method is called.
    */
+  @CanIgnoreReturnValue
   public ServiceManager startAsync() {
     for (Service service : services) {
-      State state = service.state();
-      checkState(state == NEW, "Service %s is %s, cannot start it.", service, state);
+      checkState(service.state() == NEW, "Not all services are NEW, cannot start %s", this);
     }
     for (Service service : services) {
       try {
@@ -288,7 +283,7 @@ public final class ServiceManager {
   }
 
   /**
-   * Waits for the {@link ServiceManager} to become {@linkplain #isHealthy() healthy}.  The manager
+   * Waits for the {@link ServiceManager} to become {@linkplain #isHealthy() healthy}. The manager
    * will become healthy after all the component services have reached the {@linkplain State#RUNNING
    * running} state.
    *
@@ -301,7 +296,22 @@ public final class ServiceManager {
 
   /**
    * Waits for the {@link ServiceManager} to become {@linkplain #isHealthy() healthy} for no more
-   * than the given time.  The manager will become healthy after all the component services have
+   * than the given time. The manager will become healthy after all the component services have
+   * reached the {@linkplain State#RUNNING running} state.
+   *
+   * @param timeout the maximum time to wait
+   * @throws TimeoutException if not all of the services have finished starting within the deadline
+   * @throws IllegalStateException if the service manager reaches a state from which it cannot
+   *     become {@linkplain #isHealthy() healthy}.
+   * @since 28.0
+   */
+  public void awaitHealthy(Duration timeout) throws TimeoutException {
+    awaitHealthy(toNanosSaturated(timeout), TimeUnit.NANOSECONDS);
+  }
+
+  /**
+   * Waits for the {@link ServiceManager} to become {@linkplain #isHealthy() healthy} for no more
+   * than the given time. The manager will become healthy after all the component services have
    * reached the {@linkplain State#RUNNING running} state.
    *
    * @param timeout the maximum time to wait
@@ -310,6 +320,7 @@ public final class ServiceManager {
    * @throws IllegalStateException if the service manager reaches a state from which it cannot
    *     become {@linkplain #isHealthy() healthy}.
    */
+  @SuppressWarnings("GoodTime") // should accept a java.time.Duration
   public void awaitHealthy(long timeout, TimeUnit unit) throws TimeoutException {
     state.awaitHealthy(timeout, unit);
   }
@@ -320,6 +331,7 @@ public final class ServiceManager {
    *
    * @return this
    */
+  @CanIgnoreReturnValue
   public ServiceManager stopAsync() {
     for (Service service : services) {
       service.stopAsync();
@@ -336,6 +348,19 @@ public final class ServiceManager {
     state.awaitStopped();
   }
 
+  /**
+   * Waits for the all the services to reach a terminal state for no more than the given time. After
+   * this method returns all services will either be {@linkplain Service.State#TERMINATED
+   * terminated} or {@linkplain Service.State#FAILED failed}.
+   *
+   * @param timeout the maximum time to wait
+   * @throws TimeoutException if not all of the services have stopped within the deadline
+   * @since 28.0
+   */
+  public void awaitStopped(Duration timeout) throws TimeoutException {
+    awaitStopped(toNanosSaturated(timeout), TimeUnit.NANOSECONDS);
+  }
+
   /**
    * Waits for the all the services to reach a terminal state for no more than the given time. After
    * this method returns all services will either be {@linkplain Service.State#TERMINATED
@@ -345,6 +370,7 @@ public final class ServiceManager {
    * @param unit the time unit of the timeout argument
    * @throws TimeoutException if not all of the services have stopped within the deadline
    */
+  @SuppressWarnings("GoodTime") // should accept a java.time.Duration
   public void awaitStopped(long timeout, TimeUnit unit) throws TimeoutException {
     state.awaitStopped(timeout, unit);
   }
@@ -369,8 +395,11 @@ public final class ServiceManager {
    *
    * <p>N.B. This snapshot is guaranteed to be consistent, i.e. the set of states returned will
    * correspond to a point in time view of the services.
+   *
+   * @since 29.0 (present with return type {@code ImmutableMultimap} since 14.0)
    */
-  public ImmutableMultimap<State, Service> servicesByState() {
+  @Override
+  public ImmutableSetMultimap<State, Service> servicesByState() {
     return state.servicesByState();
   }
 
@@ -385,7 +414,22 @@ public final class ServiceManager {
     return state.startupTimes();
   }
 
-  @Override public String toString() {
+  /**
+   * Returns the service load times. This value will only return startup times for services that
+   * have finished starting.
+   *
+   * @return Map of services and their corresponding startup time, the map entries will be ordered
+   *     by startup time.
+   * @since 31.0
+   */
+  @J2ObjCIncompatible
+  public ImmutableMap<Service, Duration> startupDurations() {
+    return ImmutableMap.copyOf(
+        Maps.<Service, Long, Duration>transformValues(startupTimes(), Duration::ofMillis));
+  }
+
+  @Override
+  public String toString() {
     return MoreObjects.toStringHelper(ServiceManager.class)
         .add("services", Collections2.filter(services, not(instanceOf(NoOpService.class))))
         .toString();
@@ -410,10 +454,12 @@ public final class ServiceManager {
 
     /**
      * These two booleans are used to mark the state as ready to start.
-     * {@link #ready}: is set by {@link #markReady} to indicate that all listeners have been
-     *     correctly installed
-     * {@link #transitioned}: is set by {@link #transitionService} to indicate that some transition
-     *     has been performed.
+     *
+     * <p>{@link #ready}: is set by {@link #markReady} to indicate that all listeners have been
+     * correctly installed
+     *
+     * <p>{@link #transitioned}: is set by {@link #transitionService} to indicate that some
+     * transition has been performed.
      *
      * <p>Together, they allow us to enforce that all services have their listeners installed prior
      * to any service performing a transition, then we can fail in the ServiceManager constructor
@@ -428,8 +474,8 @@ public final class ServiceManager {
     final int numberOfServices;
 
     /**
-     * Controls how long to wait for all the services to either become healthy or reach a
-     * state from which it is guaranteed that it can never become healthy.
+     * Controls how long to wait for all the services to either become healthy or reach a state from
+     * which it is guaranteed that it can never become healthy.
      */
     final Monitor.Guard awaitHealthGuard = new AwaitHealthGuard();
 
@@ -439,7 +485,9 @@ public final class ServiceManager {
         super(ServiceManagerState.this.monitor);
       }
 
-      @Override public boolean isSatisfied() {
+      @Override
+      @GuardedBy("ServiceManagerState.this.monitor")
+      public boolean isSatisfied() {
         // All services have started or some service has terminated/failed.
         return states.count(RUNNING) == numberOfServices
             || states.contains(STOPPING)
@@ -448,9 +496,7 @@ public final class ServiceManager {
       }
     }
 
-    /**
-     * Controls how long to wait for all services to reach a terminal state.
-     */
+    /** Controls how long to wait for all services to reach a terminal state. */
     final Monitor.Guard stoppedGuard = new StoppedGuard();
 
     @WeakOuter
@@ -459,19 +505,19 @@ public final class ServiceManager {
         super(ServiceManagerState.this.monitor);
       }
 
-      @Override public boolean isSatisfied() {
+      @Override
+      @GuardedBy("ServiceManagerState.this.monitor")
+      public boolean isSatisfied() {
         return states.count(TERMINATED) + states.count(FAILED) == numberOfServices;
       }
     }
 
     /** The listeners to notify during a state transition. */
-    @GuardedBy("monitor")
-    final List<ListenerCallQueue<Listener>> listeners =
-        Collections.synchronizedList(new ArrayList<ListenerCallQueue<Listener>>());
+    final ListenerCallQueue<Listener> listeners = new ListenerCallQueue<>();
 
     /**
      * It is implicitly assumed that all the services are NEW and that they will all remain NEW
-     * until all the Listeners are installed and {@link #markReady()} is called.  It is our caller's
+     * until all the Listeners are installed and {@link #markReady()} is called. It is our caller's
      * responsibility to only call {@link #markReady()} if all services were new at the time this
      * method was called and when all the listeners were installed.
      */
@@ -481,8 +527,8 @@ public final class ServiceManager {
     }
 
     /**
-     * Attempts to start the timer immediately prior to the service being started via
-     * {@link Service#startAsync()}.
+     * Attempts to start the timer immediately prior to the service being started via {@link
+     * Service#startAsync()}.
      */
     void tryStartTiming(Service service) {
       monitor.enter();
@@ -514,8 +560,10 @@ public final class ServiceManager {
               servicesInBadStates.add(service);
             }
           }
-          throw new IllegalArgumentException("Services started transitioning asynchronously before "
-              + "the ServiceManager was constructed: " + servicesInBadStates);
+          throw new IllegalArgumentException(
+              "Services started transitioning asynchronously before "
+                  + "the ServiceManager was constructed: "
+                  + servicesInBadStates);
         }
       } finally {
         monitor.leave();
@@ -523,17 +571,7 @@ public final class ServiceManager {
     }
 
     void addListener(Listener listener, Executor executor) {
-      checkNotNull(listener, "listener");
-      checkNotNull(executor, "executor");
-      monitor.enter();
-      try {
-        // no point in adding a listener that will never be called
-        if (!stoppedGuard.isSatisfied()) {
-          listeners.add(new ListenerCallQueue<Listener>(listener, executor));
-        }
-      } finally {
-        monitor.leave();
-      }
+      listeners.addListener(listener, executor);
     }
 
     void awaitHealthy() {
@@ -549,9 +587,10 @@ public final class ServiceManager {
       monitor.enter();
       try {
         if (!monitor.waitForUninterruptibly(awaitHealthGuard, timeout, unit)) {
-          throw new TimeoutException("Timeout waiting for the services to become healthy. The "
-              + "following services have not started: "
-              + Multimaps.filterKeys(servicesByState, in(ImmutableSet.of(NEW, STARTING))));
+          throw new TimeoutException(
+              "Timeout waiting for the services to become healthy. The "
+                  + "following services have not started: "
+                  + Multimaps.filterKeys(servicesByState, in(ImmutableSet.of(NEW, STARTING))));
         }
         checkHealthy();
       } finally {
@@ -568,17 +607,17 @@ public final class ServiceManager {
       monitor.enter();
       try {
         if (!monitor.waitForUninterruptibly(stoppedGuard, timeout, unit)) {
-          throw new TimeoutException("Timeout waiting for the services to stop. The following "
-              + "services have not stopped: "
-              + Multimaps.filterKeys(servicesByState,
-                  not(in(EnumSet.of(TERMINATED, FAILED)))));
+          throw new TimeoutException(
+              "Timeout waiting for the services to stop. The following "
+                  + "services have not stopped: "
+                  + Multimaps.filterKeys(servicesByState, not(in(EnumSet.of(TERMINATED, FAILED)))));
         }
       } finally {
         monitor.leave();
       }
     }
 
-    ImmutableMultimap<State, Service> servicesByState() {
+    ImmutableSetMultimap<State, Service> servicesByState() {
       ImmutableSetMultimap.Builder<State, Service> builder = ImmutableSetMultimap.builder();
       monitor.enter();
       try {
@@ -601,20 +640,24 @@ public final class ServiceManager {
         // N.B. There will only be an entry in the map if the service has started
         for (Entry<Service, Stopwatch> entry : startupTimers.entrySet()) {
           Service service = entry.getKey();
-          Stopwatch stopWatch = entry.getValue();
-          if (!stopWatch.isRunning() && !(service instanceof NoOpService)) {
-            loadTimes.add(Maps.immutableEntry(service, stopWatch.elapsed(MILLISECONDS)));
+          Stopwatch stopwatch = entry.getValue();
+          if (!stopwatch.isRunning() && !(service instanceof NoOpService)) {
+            loadTimes.add(Maps.immutableEntry(service, stopwatch.elapsed(MILLISECONDS)));
           }
         }
       } finally {
         monitor.leave();
       }
-      Collections.sort(loadTimes, Ordering.natural()
-          .onResultOf(new Function<Entry<Service, Long>, Long>() {
-            @Override public Long apply(Map.Entry<Service, Long> input) {
-              return input.getValue();
-            }
-          }));
+      Collections.sort(
+          loadTimes,
+          Ordering.natural()
+              .onResultOf(
+                  new Function<Entry<Service, Long>, Long>() {
+                    @Override
+                    public Long apply(Entry<Service, Long> input) {
+                      return input.getValue();
+                    }
+                  }));
       return ImmutableMap.copyOf(loadTimes);
     }
 
@@ -622,11 +665,12 @@ public final class ServiceManager {
      * Updates the state with the given service transition.
      *
      * <p>This method performs the main logic of ServiceManager in the following steps.
+     *
      * <ol>
-     *      <li>Update the {@link #servicesByState()}
-     *      <li>Update the {@link #startupTimers}
-     *      <li>Based on the new state queue listeners to run
-     *      <li>Run the listeners (outside of the lock)
+     *   <li>Update the {@link #servicesByState()}
+     *   <li>Update the {@link #startupTimers}
+     *   <li>Based on the new state queue listeners to run
+     *   <li>Run the listeners (outside of the lock)
      * </ol>
      */
     void transitionService(final Service service, State from, State to) {
@@ -639,10 +683,16 @@ public final class ServiceManager {
           return;
         }
         // Update state.
-        checkState(servicesByState.remove(from, service),
-            "Service %s not at the expected location in the state map %s", service, from);
-        checkState(servicesByState.put(to, service),
-            "Service %s in the state map unexpectedly at %s", service, to);
+        checkState(
+            servicesByState.remove(from, service),
+            "Service %s not at the expected location in the state map %s",
+            service,
+            from);
+        checkState(
+            servicesByState.put(to, service),
+            "Service %s in the state map unexpectedly at %s",
+            service,
+            to);
         // Update the timer
         Stopwatch stopwatch = startupTimers.get(service);
         if (stopwatch == null) {
@@ -661,58 +711,64 @@ public final class ServiceManager {
 
         // Did a service fail?
         if (to == FAILED) {
-          fireFailedListeners(service);
+          enqueueFailedEvent(service);
         }
 
         if (states.count(RUNNING) == numberOfServices) {
           // This means that the manager is currently healthy. N.B. If other threads call isHealthy
           // they are not guaranteed to get 'true', because any service could fail right now.
-          fireHealthyListeners();
+          enqueueHealthyEvent();
         } else if (states.count(TERMINATED) + states.count(FAILED) == numberOfServices) {
-          fireStoppedListeners();
+          enqueueStoppedEvent();
         }
       } finally {
         monitor.leave();
         // Run our executors outside of the lock
-        executeListeners();
+        dispatchListenerEvents();
       }
     }
 
-    @GuardedBy("monitor")
-    void fireStoppedListeners() {
-      STOPPED_CALLBACK.enqueueOn(listeners);
+    void enqueueStoppedEvent() {
+      listeners.enqueue(STOPPED_EVENT);
     }
 
-    @GuardedBy("monitor")
-    void fireHealthyListeners() {
-      HEALTHY_CALLBACK.enqueueOn(listeners);
+    void enqueueHealthyEvent() {
+      listeners.enqueue(HEALTHY_EVENT);
     }
 
-    @GuardedBy("monitor")
-    void fireFailedListeners(final Service service) {
-      new Callback<Listener>("failed({service=" + service + "})") {
-        @Override void call(Listener listener) {
-          listener.failure(service);
-        }
-      }.enqueueOn(listeners);
+    void enqueueFailedEvent(final Service service) {
+      listeners.enqueue(
+          new ListenerCallQueue.Event<Listener>() {
+            @Override
+            public void call(Listener listener) {
+              listener.failure(service);
+            }
+
+            @Override
+            public String toString() {
+              return "failed({service=" + service + "})";
+            }
+          });
     }
 
     /** Attempts to execute all the listeners in {@link #listeners}. */
-    void executeListeners() {
-      checkState(!monitor.isOccupiedByCurrentThread(),
+    void dispatchListenerEvents() {
+      checkState(
+          !monitor.isOccupiedByCurrentThread(),
           "It is incorrect to execute listeners with the monitor held.");
-      // iterate by index to avoid concurrent modification exceptions
-      for (int i = 0; i < listeners.size(); i++) {
-        listeners.get(i).execute();
-      }
+      listeners.dispatch();
     }
 
     @GuardedBy("monitor")
     void checkHealthy() {
       if (states.count(RUNNING) != numberOfServices) {
-        IllegalStateException exception = new IllegalStateException(
-            "Expected to be healthy after starting. The following services are not running: "
-                + Multimaps.filterKeys(servicesByState, not(equalTo(RUNNING))));
+        IllegalStateException exception =
+            new IllegalStateException(
+                "Expected to be healthy after starting. The following services are not running: "
+                    + Multimaps.filterKeys(servicesByState, not(equalTo(RUNNING))));
+        for (Service service : servicesByState.get(State.FAILED)) {
+          exception.addSuppressed(new FailedService(service));
+        }
         throw exception;
       }
     }
@@ -720,8 +776,8 @@ public final class ServiceManager {
 
   /**
    * A {@link Service} that wraps another service and times how long it takes for it to start and
-   * also calls the {@link ServiceManagerState#transitionService(Service, State, State)},
-   * to record the state transitions.
+   * also calls the {@link ServiceManagerState#transitionService(Service, State, State)}, to record
+   * the state transitions.
    */
   private static final class ServiceListener extends Service.Listener {
     final Service service;
@@ -734,7 +790,8 @@ public final class ServiceManager {
       this.state = state;
     }
 
-    @Override public void starting() {
+    @Override
+    public void starting() {
       ServiceManagerState state = this.state.get();
       if (state != null) {
         state.transitionService(service, NEW, STARTING);
@@ -744,38 +801,52 @@ public final class ServiceManager {
       }
     }
 
-    @Override public void running() {
+    @Override
+    public void running() {
       ServiceManagerState state = this.state.get();
       if (state != null) {
         state.transitionService(service, STARTING, RUNNING);
       }
     }
 
-    @Override public void stopping(State from) {
+    @Override
+    public void stopping(State from) {
       ServiceManagerState state = this.state.get();
       if (state != null) {
         state.transitionService(service, from, STOPPING);
       }
     }
 
-    @Override public void terminated(State from) {
+    @Override
+    public void terminated(State from) {
       ServiceManagerState state = this.state.get();
       if (state != null) {
         if (!(service instanceof NoOpService)) {
-          logger.log(Level.FINE, "Service {0} has terminated. Previous state was: {1}",
+          logger.log(
+              Level.FINE,
+              "Service {0} has terminated. Previous state was: {1}",
               new Object[] {service, from});
         }
         state.transitionService(service, from, TERMINATED);
       }
     }
 
-    @Override public void failed(State from, Throwable failure) {
+    @Override
+    public void failed(State from, Throwable failure) {
       ServiceManagerState state = this.state.get();
       if (state != null) {
         // Log before the transition, so that if the process exits in response to server failure,
         // there is a higher likelihood that the cause will be in the logs.
-        if (!(service instanceof NoOpService) ) {
-          logger.log(Level.SEVERE, "Service " + service + " has failed in the " + from + " state.",
+        boolean log = !(service instanceof NoOpService);
+        /*
+         * We have already exposed startup exceptions to the user in the form of suppressed
+         * exceptions. We don't need to log those exceptions again.
+         */
+        log &= from != State.STARTING;
+        if (log) {
+          logger.log(
+              Level.SEVERE,
+              "Service " + service + " has failed in the " + from + " state.",
               failure);
         }
         state.transitionService(service, from, FAILED);
@@ -784,18 +855,35 @@ public final class ServiceManager {
   }
 
   /**
-   * A {@link Service} instance that does nothing.  This is only useful as a placeholder to
-   * ensure that the {@link ServiceManager} functions properly even when it is managing no services.
+   * A {@link Service} instance that does nothing. This is only useful as a placeholder to ensure
+   * that the {@link ServiceManager} functions properly even when it is managing no services.
    *
    * <p>The use of this class is considered an implementation detail of ServiceManager and as such
    * it is excluded from {@link #servicesByState}, {@link #startupTimes}, {@link #toString} and all
    * logging statements.
    */
   private static final class NoOpService extends AbstractService {
-    @Override protected void doStart() { notifyStarted(); }
-    @Override protected void doStop() { notifyStopped(); }
+    @Override
+    protected void doStart() {
+      notifyStarted();
+    }
+
+    @Override
+    protected void doStop() {
+      notifyStopped();
+    }
   }
 
   /** This is never thrown but only used for logging. */
   private static final class EmptyServiceManagerWarning extends Throwable {}
+
+  private static final class FailedService extends Throwable {
+    FailedService(Service service) {
+      super(
+          service.toString(),
+          service.failureCause(),
+          false /* don't enable suppression */,
+          false /* don't calculate a stack trace. */);
+    }
+  }
 }
